/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  createProdFrgUndisp.c
* Description:  Contains routines for product files handling
*
* History:
* 25-Apr-03     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <cpl.h>
#include <time.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "iauWrite.h"
#include "midiFitsUtility.h"
#include "createProdFrgUndisp.h"
#include "geometry.h"
#include "memoryHandling.h"
#include "errorHandling.h"
#include "fileHandling.h"
#include "qfits.h"

/**********************************************************
*   Global Variables
**********************************************************/

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Function definitions
**********************************************************/
static void set_fnan(float *f)
{

#ifndef WORDS_BIGENDIAN
    static unsigned char fnan_pat[] = {0, 0, 0xc0, 0x7f};
#else
    static unsigned char fnan_pat[] = {0x7f, 0xc0, 0, 0};
#endif

memcpy(f, fnan_pat, 4);
}

/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createFrgProdUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Creates an output fits file and copies keywords from the
*               input header to the output header
*
*
* History:
* 21-Apr-03     (csabet) Created
******************************************************************************/
void createFrgProdUndisp (
    MidiFiles                *fileNames,
    ImageFormat                *format,
    FilterData                *filterInfo,
    RawVisibility            *rawVis,
    TransferFunction        *trfFunction,
    CalibratorParam            *calibrator,
    CalibratedVisibility    *calibVis,
    PhotometryResult        *photom,
    int                        *error)
{

    /*  Local Declarations
    --------------------*/
    const char      routine[] = "createFrgProdUndisp";
    int                iwave, numOfTelescopes, extNumber;
    char            *stringQfits, *classification, *stringTemp;
    FILE            *inFitsBatchPtr = NULL;
    IauExchange     *iauData;
    UVW             *uvw;
    float           *visAmp, *visSqrd;
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating Product files for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"-------------------------------- \n");
    fprintf (midiReportPtr, "\nCreating Product files for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "-------------------------------- \n");

    /*  Reset status */
    *error = 0;

    /* Allocate memory */
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (stringTemp);
        free (classification);
        return;
    }

    /*  Read the name of the first file in the list and get it's full path name */
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);
    fclose (inFitsBatchPtr);

    /* Copy keywords from the raw FITS file into the QC log */
    createQcLog (fileNames->inFitsName, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot copy keywords to QC log");
        free (stringTemp);
        free (classification);
        return;
    }

    /*  Allocate memory for data structures relating to IAU exchange product FITS file
    --------------------------------------------------------------------------------*/
    /*  Get number of array elements. Number of telescopes */
    extNumber = getFitsExtensionNumber (fileNames->inFitsName, "IMAGING_DATA", error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot get IMAGING_DATA extension number from input FITS file");
        free (stringTemp);
        free (classification);
        return;
    }
    stringQfits = qfits_query_ext (fileNames->inFitsName, "MAXTEL", extNumber);
    if (stringQfits == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read MAXTEL from input FITS file");
        free (stringTemp);
        free (classification);
        return;
    }
    else sscanf (stringQfits, "%d", &numOfTelescopes);

    /* Allocate memory */
    iauData = callocIauExchange (numOfTelescopes, 1);
    uvw = (UVW *) calloc (1, sizeof (UVW));
    visAmp = (float *) calloc (iauData->wavelength->nwave, sizeof (float));
    visSqrd = (float *) calloc (iauData->wavelength->nwave, sizeof (float));

    /* Get default visibilities */
    for (iwave = 0; iwave < iauData->wavelength->nwave; iwave++)
    {
        visAmp[iwave] = rawVis->vis;
        visSqrd[iwave] = rawVis->visSqrd;
    }

    /* Overwite if calibrated visibility is available */
    if ((strcmp (format->obsCatg, "SCIENCE") == 0) && calibVis->exists)
    {
        for (iwave = 0; iwave < iauData->wavelength->nwave; iwave++)
        {
            visAmp[iwave] = calibVis->vis[iwave];
            visSqrd[iwave] = calibVis->visSqrd[iwave];
        }
    }

    /* Load output data into the IAU exchange structures */
    loadFrgOutputDataUndisp (fileNames->inFitsName, filterInfo, visAmp, visSqrd, iauData->array, iauData->targets,
        iauData->wavelength, iauData->vis, iauData->vis2, uvw, error);
    if (*error == 1)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load data structure");
        freeIauExchange (iauData);
        free (uvw);
        free (visAmp);
        free (visSqrd);
        free (stringTemp);
        free (classification);
        return;
    }

    /* Add product info to QC log */
    addProdInfoToFrgQcLogUndisp (format, fileNames, rawVis, calibVis, uvw, photom,
        filterInfo, trfFunction, calibrator, error);
    if (*error == 1)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot add product info to QC log");
        freeIauExchange (iauData);
        free (uvw);
        free (visAmp);
        free (visSqrd);
        free (stringTemp);
        free (classification);
        return;
    }

    /*  Create the primary header extension */
    createFrgPrimHeadUndisp (fileNames, format, rawVis, calibVis,
        uvw, photom, filterInfo, trfFunction, calibrator, error);
    if (*error == 1)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Primary Header extension");
        freeIauExchange (iauData);
        free (uvw);
        free (visAmp);
        free (visSqrd);
        free (stringTemp);
        free (classification);
        return;
    }

    /*  At this stage output FITS file has probably been created with appropriate
    header. Now we create the tables and write the data in accordance with the IAU exchange */
    writeFrgFitsFileUndisp (fileNames->outFitsName, iauData->array, iauData->targets, iauData->wavelength,
        iauData->vis, iauData->vis2, error);
    if (*error == 1)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot write product FITS file");
        freeIauExchange (iauData);
        free (uvw);
        free (visAmp);
        free (visSqrd);
        free (stringTemp);
        free (classification);
        return;
    }

    if (diagnostic)cpl_msg_info(cpl_func,"Created Output FITS file: %s \n", fileNames->outFitsName);
    fprintf (midiReportPtr, "Created Output FITS file: %s \n", fileNames->outFitsName);

    /*  Release memory */
    freeIauExchange (iauData);
    free (uvw);
    free (visAmp);
    free (visSqrd);
    free (stringTemp);
    free (classification);

    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createFrgPrimHeadUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Creates the primary header
*
*
* History:
* 02-May-03     (csabet) Created
******************************************************************************/
void createFrgPrimHeadUndisp (
    MidiFiles                *fileNames,
    ImageFormat                *format,
    RawVisibility            *rawVis,
    CalibratedVisibility    *calibVis,
    UVW                        *uvw,
    PhotometryResult        *photom,
    FilterData                *filterInfo,
    TransferFunction        *trfFunction,
    CalibratorParam            *calibrator,
    int                        *error)
{

    /*  Local Declarations
    --------------------*/
    const char      routine[] = "createFrgPrimHeadUndisp";
    qfits_header    *outFitsHeader;
    FILE            *inFitsBatchPtr, *outFitsPtr;
    int             i;
    char            *textBuff, *stringQfits, *messageBuffer, *currentTime, *cleanString,
                    *stringTemp, *classification;
    unsigned int    stringLength;
    time_t          now;
    struct tm       *newTime;
    struct stat     buf;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    *error = 0;

    /*  Allocate memory */
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    /*  Get current time/date */
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    /*  Copy primary header from the raw file to the output header */
    outFitsHeader = qfits_header_read (fileNames->inFitsName);
    if (outFitsHeader == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load header from the input FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    /*  Now write all the required product keaywords to the output header
    -------------------------------------------------------------------*/
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TYPE", format->obsType, "MIDI pipeline product type", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO CATG", "REDUCED_UNDISPERSED", "Pipeline product category", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO ARCFILE", fileNames->archFileName, "Arcfile name of first raw file", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO PIPEDATE", currentTime, "Pipeline run date", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO VERSION", MIDI_PIPE_VERSION, "Pipeline version", "");
    qfits_header_add (outFitsHeader, "PIPEFILE", fileNames->pipeFileName, "Pipeline product file name", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO DID", MIDI_QC_DIC_VERSION, "QC dictionary version", "");

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    /* Loop through the list of files and write into the primary header */
    i = 1;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        /* If classification is not given get it from the raw file */
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL)
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d NAME", i);
        qfits_header_add (outFitsHeader, textBuff, messageBuffer, "FITS file name", "");
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d CATG", i++);
        qfits_header_add (outFitsHeader, textBuff, classification, "Observation Categoty", "");
    }
    fclose (inFitsBatchPtr);

    /*  Write the mask file */
    sprintf (messageBuffer, "%s", fileNames->maskFileName);
    removePathName (messageBuffer, midiReportPtr);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO REC2 RAW1 NAME", messageBuffer, "Pipeline Mask File", "");

    /*  Write the transfer function file */
    if (strcmp (format->obsCatg, "CALIB") == 0) sprintf (messageBuffer, "%s", fileNames->trfNameWrite);
    else sprintf (messageBuffer, "%s", fileNames->trfNameRead);
    removePathName (messageBuffer, midiReportPtr);
    stringLength = strlen (messageBuffer);
    sprintf (textBuff, "r.");
    strncat (textBuff, messageBuffer, stringLength-4);
    sprintf (messageBuffer, "%s_tpl_0001.dummy", textBuff);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO REC2 RAW2 NAME", messageBuffer, "Transfer Function File", "");

    /*  Scan parameters */
    sprintf (messageBuffer, "%d", format->numOfScansProcessed);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC SCN PROCESSED", messageBuffer, "Number of scans processed", "");
    sprintf (messageBuffer, "%d", format->numOfScansRejected);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC SCN REJECTED", messageBuffer, "Number of scans rejected", "");

    /*  Add Filter information */
    sprintf (messageBuffer, "%f", filterInfo->optFreqLo);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC WAVE MIN", messageBuffer, "Filter minimum wavelength in micron", "");
    sprintf (messageBuffer, "%f", filterInfo->optFreqHi);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC WAVE MAX", messageBuffer, "Filter maximum wavelength in micron", "");

    /*  Transfer functions */
    if (trfFunction->exists)
    {
        qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TRF", "AVAILABLE", "Flag for Transfer Function availability", NULL);
        if (strcmp (format->obsCatg, "CALIB") == 0)
        {
            sprintf (messageBuffer, "%f", calibrator->calibVisSqrd);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC THEOVIS", messageBuffer, "Theoretical visibility", "");
            sprintf (messageBuffer, "%f", calibrator->calibVisSqrdErr);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DTHEOVIS", messageBuffer, "(sigma) Error on theoretical visibility", "");

            sprintf (messageBuffer, "%f", trfFunction->trf[3]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC T0", messageBuffer, "Measured transfer function (ratio of square visibilities)", "");
            sprintf (messageBuffer, "%f", trfFunction->trf[4]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC T1", messageBuffer, "Measured transfer function (ratio of square visibilities)", "");
            sprintf (messageBuffer, "%f", trfFunction->trf[5]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC T2", messageBuffer, "Measured transfer function (ratio of square visibilities)", "");

            sprintf (messageBuffer, "%f", trfFunction->trfErr[3]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DT0", messageBuffer, "(sigma) Error on measured transfer function", "");
            sprintf (messageBuffer, "%f", trfFunction->trfErr[4]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DT1", messageBuffer, "(sigma) Error on measured transfer function", "");
            sprintf (messageBuffer, "%f", trfFunction->trfErr[5]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DT2", messageBuffer, "(sigma) Error on measured transfer function", "");

            sprintf (messageBuffer, "%f", 1.0/trfFunction->trf[3]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC COT0", messageBuffer, "Inverse transfer function", "");
            sprintf (messageBuffer, "%f", 1.0/trfFunction->trf[4]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC COT1", messageBuffer, "Inverse transfer function", "");
            sprintf (messageBuffer, "%f", 1.0/trfFunction->trf[5]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC COT2", messageBuffer, "Inverse transfer function", "");

            sprintf (messageBuffer, "%f", 1.0/trfFunction->trfErr[3]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DCOT0", messageBuffer, "(sigma) Error on inverse transfer function", "");
            sprintf (messageBuffer, "%f", 1.0/trfFunction->trfErr[4]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DCOT1", messageBuffer, "(sigma) Error on inverse transfer function", "");
            sprintf (messageBuffer, "%f", 1.0/trfFunction->trfErr[5]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DCOT2", messageBuffer, "(sigma) Error on inverse transfer function", "");
        }
        else
        {
            sprintf (messageBuffer, "%f", trfFunction->trf[3]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC AT0", messageBuffer, "Applied transfer function", "");
            sprintf (messageBuffer, "%f", trfFunction->trf[4]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC AT1", messageBuffer, "Applied transfer function", "");
            sprintf (messageBuffer, "%f", trfFunction->trf[5]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC AT2", messageBuffer, "Applied transfer function", "");
            sprintf (messageBuffer, "%f", trfFunction->trfErr[3]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DAT0", messageBuffer, "(sigma) Error on applied transfer function", "");
            sprintf (messageBuffer, "%f", trfFunction->trfErr[4]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DAT1", messageBuffer, "(sigma) Error on applied transfer function", "");
            sprintf (messageBuffer, "%f", trfFunction->trfErr[5]);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DAT2", messageBuffer, "(sigma) Error on applied transfer function", "");
        }
    }
    else
        qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TRF", UNAV, "Flag for Transfer Function availability", NULL);

    /*  Visibilities */
    sprintf (messageBuffer, "%f", rawVis->visSqrd);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCALV0", messageBuffer, "Uncalibrated square visibility", "");
    sprintf (messageBuffer, "%f", rawVis->visSqrd1);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCALV1", messageBuffer, "Ucalibrated square visibility", "");
    sprintf (messageBuffer, "%f", rawVis->visSqrd2);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCALV2", messageBuffer, "Uncalibrated square visibility", "");
    sprintf (messageBuffer, "%f", rawVis->visSqrdErr);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC DUNCALV0", messageBuffer, "Variance of uncalibrated square visibility", "");
    sprintf (messageBuffer, "%f", rawVis->visSqrd1Err);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC DUNCALV1", messageBuffer, "Variance of uncalibrated square visibility", "");
    sprintf (messageBuffer, "%f", rawVis->visSqrd2Err);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC DUNCALV2", messageBuffer, "Variance of uncalibrated square visibility", "");
    if (calibVis->exists)
    {
        sprintf (messageBuffer, "%f", calibVis->visSqrd[0]);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALV0", messageBuffer, "Calibrated square visibility", "");
        sprintf (messageBuffer, "%f", calibVis->visSqrd[1]);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALV1", messageBuffer, "Calibrated square visibility", "");
        sprintf (messageBuffer, "%f", calibVis->visSqrd[2]);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALV2", messageBuffer, "Calibrated square visibility", "");
        sprintf (messageBuffer, "%f", calibVis->visSqrdErr[0]);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DCALV0", messageBuffer, "Variance of calibrated square visibility", "");
        sprintf (messageBuffer, "%f", calibVis->visSqrdErr[1]);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DCALV1", messageBuffer, "Variance of calibrated square visibility", "");
        sprintf (messageBuffer, "%f", calibVis->visSqrdErr[2]);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DCALV2", messageBuffer, "Variance of calibrated square visibility", "");
    }

    /*  Photometry parameters */
    sprintf (messageBuffer, "%f", photom->photomA[0]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA0", messageBuffer, "Number of photons", "");
    sprintf (messageBuffer, "%f", photom->photomAErr[0]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC DPHOTA0", messageBuffer, "(sigma) Error in number of photons", "");
    sprintf (messageBuffer, "%f", photom->photomA[1]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA1", messageBuffer, "Number of photons", "");
    sprintf (messageBuffer, "%f", photom->photomAErr[1]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC DPHOTA1", messageBuffer, "(sigma) Error in number of photons", "");
    sprintf (messageBuffer, "%f", photom->photomB[0]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB0", messageBuffer, "Number of photons", "");
    sprintf (messageBuffer, "%f", photom->photomBErr[0]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC DPHOTB0", messageBuffer, "(sigma) Error in number of photons", "");
    sprintf (messageBuffer, "%f", photom->photomB[1]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB1", messageBuffer, "Number of photons", "");
    sprintf (messageBuffer, "%f", photom->photomBErr[1]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC DPHOTB1", messageBuffer, "(sigma) Error in number of photons", "");
    sprintf (messageBuffer, "%f", photom->ratioPhotomA0toA1);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA01", messageBuffer, "Ratio of number of photons", "");
    sprintf (messageBuffer, "%f", photom->ratioPhotomB0toB1);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB01", messageBuffer, "Ratio of number of photons", "");
    sprintf (messageBuffer, "%f", photom->chop2ChopAErr[0]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC RMS CHOPA0", messageBuffer, "RMS of chopping cycles", "");
    sprintf (messageBuffer, "%f", photom->chop2ChopAErr[1]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC RMS CHOPA1", messageBuffer, "RMS of chopping cycles", "");
    sprintf (messageBuffer, "%f", photom->chop2ChopBErr[0]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC RMS CHOPB0", messageBuffer, "RMS of chopping cycles", "");
    sprintf (messageBuffer, "%f", photom->chop2ChopBErr[1]);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC RMS CHOPB1", messageBuffer, "RMS of chopping cycles", "");

    /*  Add Calibrator keywords */
    if (strcmp (format->obsCatg, "CALIB") == 0)
    {
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB POS", "UNKNOWN", "Position of the calibrator entry", "");
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB NAME", calibrator->calibName, "Name of the calibrator entry", "");
        sprintf (messageBuffer, "%f", calibrator->calibRA);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB RA", messageBuffer, "Calibrator Right Ascension", "");
        sprintf (messageBuffer, "%f", calibrator->calibDEC);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DEC", messageBuffer, "Calibrator Declination", "");
        sprintf (messageBuffer, "%f", RAD_TO_ARCSEC * calibrator->calibDistance);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DIST", messageBuffer, "Calibrator Radial Distance in arcsec", "");
        sprintf (messageBuffer, "%f", calibrator->calibDiameter);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DIAM", messageBuffer, "Calibrator Diameter in milliarcsec", "");
        sprintf (messageBuffer, "%f", calibrator->calibDiameterErr);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DDIAM", messageBuffer, "(sigma) Error on calibrator Diameter in milliarcsec", "");
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB FLAG", "UNKNOWN", "Calibrator quality flag", "");
        sprintf (messageBuffer, "%f", calibrator->calibVisSqrd);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB VIS", messageBuffer, "Calibrator estimated square visibility", "");
        sprintf (messageBuffer, "%f", calibrator->calibVisSqrdErr);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DVIS", messageBuffer, "Variance of calibrator estimated square visibility", "");
    }

    /*  Add Target keywords */
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC TARG DIAM", "UNKNOWN", "Estimated diameter of the target (mas, Un.Disk model)", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC TARG DDIAM", "UNKNOWN", "(sigma) Error on estimated target diameter in milliarcsec", "");

    /*  Add UVW. The values of uvw will be added during pipeline execution */
    sprintf (messageBuffer, "%f", uvw->uCoord);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL UVW1", messageBuffer, "u component of UVW vector", "");
    sprintf (messageBuffer, "%f", uvw->vCoord);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL UVW2", messageBuffer, "v component of UVW vector", "");
    sprintf (messageBuffer, "%f", uvw->wCoord);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL UVW3", messageBuffer, "w component of UVW vector", "");

    /*  Baseline parameters */
    if (trfFunction->exists)
    {
        if (strcmp (format->obsCatg, "CALIB") == 0)
        {
            sprintf (messageBuffer, "%f", calibrator->calibPblAverage);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL LENGTH", messageBuffer, "Baseline vector length", "");
            sprintf (messageBuffer, "%f", calibrator->calibParangAverage);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL ANGLE", messageBuffer, "Baseline vector angle", "");
        }
    }

    /*  Place Holders */
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH1", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH2", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH3", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH4", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH5", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH6", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH7", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH8", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH9", "TBD", "TBD", "");

    /*  Create the output fits file */
    if (stat (fileNames->outFitsName, &buf) == 0)   /*  If the file exist delete it */
        remove (fileNames->outFitsName);
    outFitsPtr = fopen (fileNames->outFitsName, "w");
    if (!outFitsPtr)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create output FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    /*  Write header into the output file */
    qfits_header_sort (&outFitsHeader);
    qfits_header_dump (outFitsHeader, outFitsPtr);
    qfits_header_destroy (outFitsHeader);
    fclose (outFitsPtr);

    /*  release memory */
    free (messageBuffer);
    free (currentTime);
    free (cleanString);
    free (stringTemp);
    free (classification);
    free (textBuff);

    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  addProdInfoToFrgQcLogUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Adds product information to QC log
*
*
* History:
* 16-Jan-04     (csabet) Created
******************************************************************************/
void addProdInfoToFrgQcLogUndisp (
    ImageFormat                *format,
    MidiFiles               *fileNames,
    RawVisibility           *rawVis,
    CalibratedVisibility    *calibVis,
    UVW                        *uvw,
    PhotometryResult        *photom,
    FilterData                *filterInfo,
    TransferFunction        *trfFunction,
    CalibratorParam            *calibrator,
    int                        *error)
{

    /*  Local Declarations
    --------------------*/
    const char      routine[] = "addProdInfoToFrgQcLogUndisp";
    int             i;
    FILE            *inFitsBatchPtr;
    char            *stringQfits, *messageBuffer, *currentTime, *textBuff, *cleanString,
                    *stringTemp, *classification;
    unsigned int    stringLength;
    time_t          now;
    struct tm       *newTime;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset parameters */
    *error = 0;

    /*  Allocate memory */
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    /*  Get current time/date */
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Add keyword to QC log file
    fprintf (midiQcLogPtr, "PRO.TYPE            \"%s\" \n", format->obsType);
    fprintf (midiQcLogPtr, "PRO.CATG            \"REDUCED_UNDISPERSED\" \n");
    fprintf (midiQcLogPtr, "PRO.ARCFILE         \"%s\" \n", fileNames->archFileName);
    fprintf (midiQcLogPtr, "PRO.PIPEDATE        \"%s\" \n", currentTime);
    fprintf (midiQcLogPtr, "PRO.VERSION         \"%s\" \n", MIDI_PIPE_VERSION);
    fprintf (midiQcLogPtr, "PRO.PIPEFILE        \"%s\" \n", fileNames->pipeFileName);
    fprintf (midiQcLogPtr, "PRO.DID             \"%s\" \n", MIDI_QC_DIC_VERSION);

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    /* Loop through the list of files and write into the QC log */
    i = 1;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        /* If classification is not given get it from the raw file */
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL)
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.NAME  \"%s\" \n", i, messageBuffer);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.CATG  \"%s\" \n", i++, classification);
    }
    fclose (inFitsBatchPtr);

    /* Add name of Mask file to QC log file */
    sprintf (messageBuffer, "%s", fileNames->maskFileName);
    removePathName (messageBuffer, midiReportPtr);
    fprintf (midiQcLogPtr, "PRO.REC2.RAW1.NAME  \"%s\" \n", messageBuffer);

    /* Add name of Transfer Function file to QC log file */
    if (strcmp (format->obsCatg, "CALIB") == 0) sprintf (messageBuffer, "%s", fileNames->trfNameWrite);
    else sprintf (messageBuffer, "%s", fileNames->trfNameRead);
    removePathName (messageBuffer, midiReportPtr);
    stringLength = strlen (messageBuffer);
    sprintf (textBuff, "r.");
    strncat (textBuff, messageBuffer, stringLength-4);
    sprintf (messageBuffer, "%s_tpl_0001.dummy", textBuff);
    fprintf (midiQcLogPtr, "PRO.REC2.RAW2.NAME  \"%s\" \n", messageBuffer);

    /*  Scan parameters */
    fprintf (midiQcLogPtr, "QC.SCN.PROCESSED %d \n", format->numOfScansProcessed);
    fprintf (midiQcLogPtr, "QC.SCN.REJECTED  %d \n", format->numOfScansRejected);

    /* Add keyword to QC log file */
    fprintf (midiQcLogPtr, "QC.FREQ.MIN         %f \n", filterInfo->optFreqLo);
    fprintf (midiQcLogPtr, "QC.FREQ.MAX         %f \n", filterInfo->optFreqHi);

    /*  Transfer functions */
    if (trfFunction->exists)
    {
        if (strcmp (format->obsCatg, "CALIB") == 0)
        {
            /* Add QC log */
            fprintf (midiQcLogPtr, "QC.THEOVIS          %f \n", calibrator->calibVisSqrd);
            fprintf (midiQcLogPtr, "QC.DTHEOVIS         %f \n", calibrator->calibVisSqrdErr);
            fprintf (midiQcLogPtr, "QC.T0               %f \n", trfFunction->trf[3]);
            fprintf (midiQcLogPtr, "QC.T1               %f \n", trfFunction->trf[4]);
            fprintf (midiQcLogPtr, "QC.T2               %f \n", trfFunction->trf[5]);
            fprintf (midiQcLogPtr, "QC.DT0              %f \n", trfFunction->trfErr[3]);
            fprintf (midiQcLogPtr, "QC.DT1              %f \n", trfFunction->trfErr[4]);
            fprintf (midiQcLogPtr, "QC.DT2              %f \n", trfFunction->trfErr[5]);
            fprintf (midiQcLogPtr, "QC.COT0             %f \n", 1.0/trfFunction->trf[3]);
            fprintf (midiQcLogPtr, "QC.COT1             %f \n", 1.0/trfFunction->trf[4]);
            fprintf (midiQcLogPtr, "QC.COT2             %f \n", 1.0/trfFunction->trf[5]);
            fprintf (midiQcLogPtr, "QC.DCOT0            %f \n", 1.0/trfFunction->trfErr[3]);
            fprintf (midiQcLogPtr, "QC.DCOT1            %f \n", 1.0/trfFunction->trfErr[4]);
            fprintf (midiQcLogPtr, "QC.DCOT2            %f \n", 1.0/trfFunction->trfErr[5]);
        }
        else
        {
            /* Add QC log */
            fprintf (midiQcLogPtr, "QC.AT0              %f \n", trfFunction->trf[3]);
            fprintf (midiQcLogPtr, "QC.AT1              %f \n", trfFunction->trf[4]);
            fprintf (midiQcLogPtr, "QC.AT2              %f \n", trfFunction->trf[5]);
            fprintf (midiQcLogPtr, "QC.DAT0             %f \n", trfFunction->trfErr[3]);
            fprintf (midiQcLogPtr, "QC.DAT1             %f \n", trfFunction->trfErr[4]);
            fprintf (midiQcLogPtr, "QC.DAT2             %f \n", trfFunction->trfErr[5]);
        }
    }

    /*  Visibilities */
    fprintf (midiQcLogPtr, "QC.UNCALV0          %f \n", rawVis->visSqrd);
    fprintf (midiQcLogPtr, "QC.UNCALV1          %f \n", rawVis->visSqrd1);
    fprintf (midiQcLogPtr, "QC.UNCALV2          %f \n", rawVis->visSqrd2);
    fprintf (midiQcLogPtr, "QC.DUNCALV0         %f \n", rawVis->visSqrdErr);
    fprintf (midiQcLogPtr, "QC.DUNCALV1         %f \n", rawVis->visSqrd1Err);
    fprintf (midiQcLogPtr, "QC.DUNCALV2         %f \n", rawVis->visSqrd2Err);
    if (calibVis->exists)
    {
        fprintf (midiQcLogPtr, "QC.CALV0            %f \n", calibVis->visSqrd[0]);
        fprintf (midiQcLogPtr, "QC.CALV1            %f \n", calibVis->visSqrd[1]);
        fprintf (midiQcLogPtr, "QC.CALV2            %f \n", calibVis->visSqrd[2]);
        fprintf (midiQcLogPtr, "QC.DCALV0           %f \n", calibVis->visSqrdErr[0]);
        fprintf (midiQcLogPtr, "QC.DCALV1           %f \n", calibVis->visSqrdErr[1]);
        fprintf (midiQcLogPtr, "QC.DCALV2           %f \n", calibVis->visSqrdErr[2]);
    }

    /*  Photometry parameters */
    fprintf (midiQcLogPtr, "QC.PHOTA0           %f \n", photom->photomA[0]);
    fprintf (midiQcLogPtr, "QC.DPHOTA0          %f \n", photom->photomAErr[0]);
    fprintf (midiQcLogPtr, "QC.PHOTA1           %f \n", photom->photomA[1]);
    fprintf (midiQcLogPtr, "QC.DPHOTA1          %f \n", photom->photomAErr[1]);
    fprintf (midiQcLogPtr, "QC.PHOTB0           %f \n", photom->photomB[0]);
    fprintf (midiQcLogPtr, "QC.DPHOTB0          %f \n", photom->photomBErr[0]);
    fprintf (midiQcLogPtr, "QC.PHOTB1           %f \n", photom->photomB[1]);
    fprintf (midiQcLogPtr, "QC.DPHOTB1          %f \n", photom->photomBErr[1]);
    fprintf (midiQcLogPtr, "QC.PHOTA01          %f \n", photom->ratioPhotomA0toA1);
    fprintf (midiQcLogPtr, "QC.PHOTB01          %f \n", photom->ratioPhotomB0toB1);
    fprintf (midiQcLogPtr, "QC.RMS.CHOPA0       %f \n", photom->chop2ChopAErr[0]);
    fprintf (midiQcLogPtr, "QC.RMS.CHOPA1       %f \n", photom->chop2ChopAErr[1]);
    fprintf (midiQcLogPtr, "QC.RMS.CHOPB0       %f \n", photom->chop2ChopBErr[0]);
    fprintf (midiQcLogPtr, "QC.RMS.CHOPB1       %f \n", photom->chop2ChopBErr[1]);

    /*  Add Calibrator keywords */
    if (strcmp (format->obsCatg, "CALIB") == 0)
    {
        fprintf (midiQcLogPtr, "QC.CALIB.POS        \"UNKNOWN\" \n");
        fprintf (midiQcLogPtr, "QC.CALIB.NAME       \"%s\" \n", calibrator->calibName);
        fprintf (midiQcLogPtr, "QC.CALIB.RA         %f \n", calibrator->calibRA);
        fprintf (midiQcLogPtr, "QC.CALIB.DEC        %f \n", calibrator->calibDEC);
        fprintf (midiQcLogPtr, "QC.CALIB.DIST       %f \n", RAD_TO_ARCSEC * calibrator->calibDistance);
        fprintf (midiQcLogPtr, "QC.CALIB.DIAM       %f \n", calibrator->calibDiameter);
        fprintf (midiQcLogPtr, "QC.CALIB.DDIAM      %f \n", calibrator->calibDiameterErr);
        fprintf (midiQcLogPtr, "QC.CALIB.FLAG       \"UNKNOWN\" \n");
        fprintf (midiQcLogPtr, "QC.CALIB.VIS        %f \n", calibrator->calibVisSqrd);
        fprintf (midiQcLogPtr, "QC.CALIB.DVIS       %f \n", calibrator->calibVisSqrdErr);
    }

    /*  Add Target keywords */
    fprintf (midiQcLogPtr, "QC.TARG.DIAM        \"UNKNOWN\" \n");
    fprintf (midiQcLogPtr, "QC.TARG.DDIAM       \"UNKNOWN\" \n");

    /*  Add UVW. The values of uvw will be added during pipeline execution */
    fprintf (midiQcLogPtr, "QC.BL.UVW1    %f \n", uvw->uCoord);
    fprintf (midiQcLogPtr, "QC.BL.UVW2    %f \n", uvw->vCoord);
    fprintf (midiQcLogPtr, "QC.BL.UVW3    %f \n", uvw->wCoord);

    /*  Baseline parameters */
    if (trfFunction->exists)
    {
        if (strcmp (format->obsCatg, "CALIB") == 0)
        {
            fprintf (midiQcLogPtr, "QC.BL.LENGTH  %f \n", calibrator->calibPblAverage);
            fprintf (midiQcLogPtr, "QC.BL.ANGLE   %f \n", calibrator->calibParangAverage);
        }
    }

    /*  release memory */
    free (messageBuffer);
    free (currentTime);
    free (textBuff);
    free (cleanString);
    free (stringTemp);
    free (classification);

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  loadFrgOutputDataUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Loads MIDI product data into appropriate data structures
*
*
* History:
* 04-Nov-03     (csabet) Created
******************************************************************************/
void loadFrgOutputDataUndisp (
    char            *fileName,
    FilterData        *filterInfo,
    float            *visAmp,
    float            *visSqrd,
    OiArray            *array,
    OiTarget        *targets,
    OiWavelength    *wave,
    OiVis            *vis,
    OiVis2            *vis2,
    UVW                *uvw,    //    Ou: Coordinates of the uvw
    int                *error)
{
    //    Local Declarations
    //    ------------------
    const char      routine[] = "loadFrgOutputDataUndisp";
    int                i, irec, iwave, itarg, mjdObs, *selection, extNumber;
    float           effectiveWave, effectiveBand, equinox;
    char            *stringQfits, *emptyString, *buffer, *observationData, *arrayName,
                    *instName, *cleanString;
    double          raEp0, decEp0, integrationTime;
    qfits_table     *arrayGeometry;
    char               *qfitsRow;
    short           *qfitsRow_int, targetId = 1;
    float           *qfitsRow_flt, nullFloat;
    double          *qfitsRow_dbl, utcTime;

    //    Algorithm
    //    ---------
    *error = 0;
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    emptyString = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    buffer = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    observationData = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    arrayName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    instName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Initialize values
    strcpy (emptyString, "NULL");
    set_fnan (&nullFloat);

    //    Get general parameters from the Header of the input FITS file
    //    -------------------------------------------------------------
    stringQfits = qfits_query_hdr (fileName, "DATE-OBS");
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", observationData);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", observationData);
    }


    stringQfits = qfits_query_hdr (fileName, "EXPTIME");
    if (stringQfits == NULL) integrationTime = nullFloat;
    else sscanf (stringQfits, "%lf", &integrationTime);

    extNumber = getFitsExtensionNumber (fileName, "ARRAY_GEOMETRY", error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot get ARRAY_GEOMETRY extension number from input FITS file");
        free (instName);
        free (arrayName);
        free (observationData);
        free (emptyString);
        free (buffer);
        free (cleanString);
        return;
    }
    
    stringQfits = qfits_query_ext (fileName, "ARRNAME", extNumber);
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", arrayName);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", arrayName);
    }

    stringQfits = qfits_query_hdr (fileName, "INSTRUME");
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", instName);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", instName);
    }

    stringQfits = qfits_query_hdr (fileName, "RA");
    if (stringQfits == NULL) raEp0 = nullFloat;
    else sscanf (stringQfits, "%lf", &raEp0);

    stringQfits = qfits_query_hdr (fileName, "DEC");
    if (stringQfits == NULL) decEp0 = nullFloat;
    else sscanf (stringQfits, "%lf", &decEp0);

    stringQfits = qfits_query_hdr (fileName, "EQUINOX");
    if (stringQfits == NULL) equinox = nullFloat;
    else sscanf (stringQfits, "%f", &equinox);

    stringQfits = qfits_query_hdr (fileName, "MJD-OBS");
    if (stringQfits == NULL) mjdObs = nullFloat;
    else sscanf (stringQfits, "%d", &mjdObs);

    stringQfits = qfits_query_hdr (fileName, "UTC");
    if (stringQfits == NULL) utcTime = nullFloat;
    else sscanf (stringQfits, "%lf", &utcTime);

    //    Load info for OI_ARRAY table
    //    ----------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading oi_array data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading oi_array data ... \n");

    //    Get the required parameters from the ARRAY_GEOMETRY extension of the input FITS file
    sscanf (arrayName, "%s", array->arrname);

    //    Get extension number
    extNumber = getFitsExtensionNumber (fileName, "ARRAY_GEOMETRY", error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot get ARRAY_GEOMETRY extension number from input FITS file");
        free (instName);
        free (arrayName);
        free (observationData);
        free (emptyString);
        free (buffer);
        free (cleanString);
        return;
    }
    
    stringQfits = qfits_query_ext (fileName, "FRAME", extNumber);
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", array->frame);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", array->frame);
    }
    
    stringQfits = qfits_query_ext (fileName, "ARRAYX", extNumber);
    if (stringQfits == NULL) array->arrayx = nullFloat;
    else sscanf (stringQfits, "%lf", &(array->arrayx));

    stringQfits = qfits_query_ext (fileName, "ARRAYY", extNumber);
    if (stringQfits == NULL) array->arrayy = nullFloat;
    else sscanf (stringQfits, "%lf", &(array->arrayy));

    stringQfits = qfits_query_ext (fileName, "ARRAYZ", extNumber);
    if (stringQfits == NULL) array->arrayz = nullFloat;
    else sscanf (stringQfits, "%lf", &(array->arrayz));

    //    Open the table
    arrayGeometry = qfits_table_open (fileName, extNumber);

    //    Load info for OI_ARRAY table
    //    ----------------------------
    selection = (int *) calloc (array->nelement, sizeof (int));
    for (i = 0; i < array->nelement; i++)
    {
        selection[i] = 1;
        qfitsRow = (char *) (qfits_query_column (arrayGeometry, 0, selection));
        sprintf (array->elem[i].tel_name, "%s", qfitsRow);
        free (qfitsRow);

        qfitsRow = (char *) (qfits_query_column (arrayGeometry, 1, selection));
        sprintf (array->elem[i].sta_name, "%s", qfitsRow);
        free (qfitsRow);

        qfitsRow_int = (short *) (qfits_query_column (arrayGeometry, 2, selection));
        array->elem[i].sta_index = qfitsRow_int[0];
        free (qfitsRow_int);

        qfitsRow_flt = (float *) (qfits_query_column (arrayGeometry, 3, selection));
        array->elem[i].diameter = qfitsRow_flt[0];
        free (qfitsRow_flt);

        qfitsRow_dbl = (double *) (qfits_query_column (arrayGeometry, 4, selection));
        array->elem[i].staxyz[0] = qfitsRow_dbl[0];
        array->elem[i].staxyz[1] = qfitsRow_dbl[1];
        array->elem[i].staxyz[2] = qfitsRow_dbl[2];
        free (qfitsRow_dbl);

        selection[i] = 0;
    }
    sprintf (array->revision, "%s", IAUEXCHANGE_VERSION);
    free (selection);
    qfits_table_close (arrayGeometry);


    //    Load info for OI_TARGET table
    //    -----------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading oi_target data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading oi_target data ... \n");

    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO OBS TARG NAME");
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", buffer);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", buffer);
    }

    for (itarg = 0; itarg < targets->ntarget; itarg++)
    {
        targets->targ[itarg].target_id = targetId;
        sscanf (buffer, "%s", targets->targ[itarg].target);
        targets->targ[itarg].raep0 = raEp0;
        targets->targ[itarg].decep0 = decEp0;
        targets->targ[itarg].equinox = equinox;
        targets->targ[itarg].ra_err = nullFloat;
        targets->targ[itarg].dec_err = nullFloat;
        targets->targ[itarg].sysvel = nullFloat;
        sscanf (emptyString, "%s", targets->targ[itarg].veltyp);
        sscanf (emptyString, "%s", targets->targ[itarg].veldef);
        targets->targ[itarg].pmra = nullFloat;
        targets->targ[itarg].pmdec = nullFloat;
        targets->targ[itarg].pmra_err = nullFloat;
        targets->targ[itarg].pmdec_err = nullFloat;
        targets->targ[itarg].parallax = nullFloat;
        targets->targ[itarg].para_err = nullFloat;
        sscanf (emptyString, "%s", targets->targ[itarg].spectyp);
    }
    sprintf (targets->revision, "%s", IAUEXCHANGE_VERSION);


    //    Load info for OI_WAVELENGTH table
    //    ---------------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading oi_wavelength data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading oi_wavelength data ... \n");

    //    Compute the parameters
    effectiveWave = 1.e-6 * (SPEED_OF_LIGHT /
        (0.5 * (filterInfo->optFreqLo + filterInfo->optFreqHi)));
    effectiveBand = 1.e-6 * (SPEED_OF_LIGHT /
        (filterInfo->optFreqHi - filterInfo->optFreqLo));

    sscanf (instName, "%s", wave->insname);
    for (i = 0; i < wave->nwave; i++)
    {
        wave->eff_wave[i] = effectiveWave;
        wave->eff_band[i] = effectiveBand;
    }
    sprintf (wave->revision, "%s", IAUEXCHANGE_VERSION);

    //    Get the uvw parameters
    computeUVW (array->elem[0].staxyz[0], array->elem[0].staxyz[1], array->elem[0].staxyz[2],
        array->elem[1].staxyz[0], array->elem[1].staxyz[1], array->elem[1].staxyz[2],
        mjdObs, raEp0, decEp0, uvw);

    //    Load info for OI_VIS table
    //    --------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading oi_vis data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading oi_vis data ... \n");

    //    Get the required parameters from the FITS file
    sscanf (observationData, "%s", vis->date_obs);
    sscanf (arrayName, "%s", vis->arrname);
    sscanf (instName, "%s", vis->insname);
    for (irec = 0; irec < vis->numrec; irec++)
    {
        vis->record[irec].target_id = targetId;
        vis->record[irec].time = utcTime;
        vis->record[irec].mjd = mjdObs;
        vis->record[irec].int_time = integrationTime;
        vis->record[irec].ucoord = uvw->uCoord;
        vis->record[irec].vcoord = uvw->vCoord;
        vis->record[irec].sta_index[0] = array->elem[0].sta_index;
        vis->record[irec].sta_index[1] = array->elem[1].sta_index;

        for (iwave = 0; iwave < wave->nwave; iwave++)
        {
            vis->record[irec].visamp[iwave] = visAmp[iwave];
            vis->record[irec].visamperr[iwave] = nullFloat;
            vis->record[irec].visphi[iwave] = nullFloat;
            vis->record[irec].visphierr[iwave] = nullFloat;
        }
        sprintf (vis->record[irec].flag, "%s", "FALSE");
    }
    sprintf (vis->revision, "%s", IAUEXCHANGE_VERSION);
    vis->nwave = wave->nwave;


    //    Load info for OI_VIS2 table
    //    ---------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading oi_vis2 data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading oi_vis2 data ... \n");

    //    Get the required parameters from the FITS file
    sscanf (observationData, "%s", vis2->date_obs);
    sscanf (arrayName, "%s", vis2->arrname);
    sscanf (instName, "%s", vis2->insname);
    for(irec = 0; irec < vis2->numrec; irec++)
    {
        vis2->record[irec].target_id = targetId;
        vis2->record[irec].time = utcTime;
        vis2->record[irec].mjd = mjdObs;
        vis2->record[irec].int_time = integrationTime;
        vis2->record[irec].ucoord = uvw->uCoord;
        vis2->record[irec].vcoord = uvw->vCoord;
        vis2->record[irec].sta_index[0] = array->elem[0].sta_index;;
        vis2->record[irec].sta_index[1] = array->elem[1].sta_index;;

        for(iwave = 0; iwave < wave->nwave; iwave++)
        {
            vis2->record[irec].vis2data[iwave] = visSqrd[iwave];
            vis2->record[irec].vis2err[iwave] = nullFloat;
        }
        sprintf (vis2->record[irec].flag, "%s", "FALSE");
    }
    sprintf (vis2->revision, "%s", IAUEXCHANGE_VERSION);
    vis2->nwave = wave->nwave;


    //    Release memory
    free (instName);
    free (arrayName);
    free (observationData);
    free (emptyString);
    free (buffer);
    free (cleanString);

    return;

}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  writeFrgFitsFileUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Writes and creates appropriate tables into a new fits file in
*               accordance with the IAU format
*
* History:
* 02-May-03     (csabet) Created
******************************************************************************/
void writeFrgFitsFileUndisp (
    char            *outFitsName,
    OiArray            *array,
    OiTarget        *targets,
    OiWavelength    *wavelength,
    OiVis            *vis,
    OiVis2            *vis2,
    int                *error)
{
    /*  Local Declarations
    --------------------*/
    const char  routine[] = "writeFrgFitsFileUndisp";

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    if (diagnostic)cpl_msg_info(cpl_func,"Writing data into the output FITS file \n\n");
    if (diagnostic) fprintf (midiReportPtr, "Writing data into the output FITS file \n\n");

    /*  Reset parameters */
    *error = 0;

    /*  Write OiArray table */
    writeOiArray (outFitsName, array, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot write OI_ARRAY in the output FITS file");
        return;
    }

    /*  Write OiTarget table */
    writeOiTarget (outFitsName, targets, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot write OI_TARGET in the output FITS file");
        return;
    }

    /*  Write OiWavelength table */
    writeOiWavelength (outFitsName, wavelength, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot write OI_WAVELENGTH in the output FITS file");
        return;
    }

    /*  Write OiVis table */
    writeOiVis (outFitsName, vis, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot write OI_VIS in the output FITS file");
        return;
    }

    /*  Write OiVis2 table */
    writeOiVis2 (outFitsName, vis2, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot write OI_VIS2 in the output FITS file");
        return;
    }

    return;

}
/*****************************************************************************/

