
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createProdKappa.c
* Description:  Contains routines for the product files handling
*
* History:      
* 07-Dec-05     (csabet)
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/stat.h>
#include <stdio.h>
#include <cpl.h>
#include <time.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "errorHandling.h"
#include "midiFitsUtility.h"
#include "fileHandling.h"
#include "createProdKappa.h"
#include "qfits.h"

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createKappaProd
* Input/Output: See function arguments to avoid duplication
* Description:  Creates MIDI product files
*
* History:      
* 15-Jun-04     (csabet) Created
******************************************************************************/
void createKappaProd (
    int                update,        // In: Indicates local or database
    MidiFiles       *fileNames,    // In: Pointer to the file structure
    ImageFormat     *size,        // In: Pointer to the image format structure
    TransferRatios    *trRatios,    // In: Transfer Ratios
    int                *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "createKappaProd";
    float        *spkImage;
    qfitsdumper    qdSpk;
   
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating Product files for SCI_PHOT_KAPPA from batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"---------------------------------------------------- \n");
    fprintf (midiReportPtr, "\nCreating Product files for SCI_PHOT_KAPPA from batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "---------------------------------------------------- \n");
    
    //    Create the Transfer Ratios files. Do not create is there has been an error
    if (!(*error))
        createTransferRatiosFile (update, size, fileNames, trRatios, error);

    //    Reset parameters
    *error = 0;

    //    Create the primary header extension
    createKappaPrimHead (fileNames, size, trRatios, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Primary Header extension");
        return;
    }
    
    //    Create a dummy image
    spkImage = (float *) calloc (size->iXWidth * size->iYWidth, sizeof (float));
    qdSpk.filename  = fileNames->outFitsName;
    qdSpk.npix      = size->iXWidth * size->iYWidth;
    qdSpk.ptype     = PTYPE_FLOAT;
    qdSpk.fbuf      = spkImage;
    qdSpk.out_ptype = BPP_IEEE_FLOAT;
    qfits_pixdump (&qdSpk);
    free (spkImage);
    
    //    Copy keywords from the raw FITS file into the QC log
    createKappaQcLog (fileNames->inFitsName, error);
    if (*error) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot copy keywords to QC log");

    //    Add output product to QC log
    addProdInfoToKappaQcLog (size, fileNames, trRatios, error);
 
    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createKappaPrimHead
* Input/Output: See function arguments to avoid duplication
* Description:  Creates the primary header
*
*
* History:
* 08-Dec-05     (csabet) Created
******************************************************************************/
void createKappaPrimHead (
    MidiFiles        *fileNames,
    ImageFormat        *size,
    TransferRatios    *trRatios,    // In: Transfer Ratios
    int                *error)
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "createKappaPrimHead";
    qfits_header    *outFitsHeader;
    FILE            *inFitsBatchPtr=NULL, *outFitsPtr;
    int             i;
    char            *textBuff, *stringQfits, *messageBuffer, *currentTime, *cleanString,
                    *stringTemp, *classification, *sWidthX, *sWidthY;
    time_t          now;
    struct tm       *newTime;
    struct stat     buf;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset parameters
    *error = 0;

    //    Open list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        *error = 1;
        return;
    }

    //    Allocate memory
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthX = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthY = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Get current time/date
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    //    Copy primary header from the raw file to the output header
    outFitsHeader = qfits_header_read (fileNames->inFitsName);
    if (outFitsHeader == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load header from the input FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        free (sWidthX);
        free (sWidthY);        
        return;
    }

    //    Write all NAXIS values as character strings
    sprintf (sWidthY, "%d", size->iYWidth);
    sprintf (sWidthX, "%d", size->iXWidth);

    //    Updates header for the Transfer Ratios output file
    qfits_header_mod (outFitsHeader, "BITPIX", "-32", "number of bits per pixel"); 
    qfits_header_mod (outFitsHeader, "NAXIS", "2", "number of data axes"); 
    qfits_header_add (outFitsHeader, "NAXIS1", sWidthX, "", NULL);
    qfits_header_add (outFitsHeader, "NAXIS2", sWidthY, "", NULL);
    qfits_header_mod (outFitsHeader, "INSTRUME", "MIDI", "MIDI Raw Data Display FITS created by DRS pipeline" );  

    //    Now write all the required product keaywords to the output header
    //    -----------------------------------------------------------------
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TYPE", size->obsType, "MIDI pipeline product type", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO CATG", "KAPPA_MATRIX", "Pipeline product category", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO ARCFILE", fileNames->archFileName, "Arcfile name of first raw file", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO PIPEDATE", currentTime, "Pipeline run date", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO VERSION", MIDI_PIPE_VERSION, "Pipeline version", NULL);
    qfits_header_add (outFitsHeader, "PIPEFILE", fileNames->pipeFileName, "Pipeline product file name", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO DID", MIDI_QC_DIC_VERSION, "QC dictionary version", NULL);

    //    Rewind the list of files
    rewind (inFitsBatchPtr);
    
    //    Loop through the list of files and write into the primary header
    i = 1;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        //    If classification is not given get it from the raw file
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL)
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d NAME", i);
        qfits_header_add (outFitsHeader, textBuff, messageBuffer, "FITS file name", "");
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d CATG", i++);
        qfits_header_add (outFitsHeader, textBuff, classification, "Observation Categoty", "");
    }
    fclose (inFitsBatchPtr);

    //    Add the required QC keywords
    sprintf (messageBuffer, "%d", trRatios->exists);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC KAPPA_EXISTS", messageBuffer, "1 = Transfer Ratios exist", "");
    if (trRatios->exists)
    {
        for (i = 0; i < size->iXWidth; i++)
        {
            if (badChannelList[i])
            {
                sprintf (messageBuffer, "%s", UNAV);
                sprintf (textBuff, "HIERARCH ESO QC KA1 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio", "");
                sprintf (messageBuffer, "%s", UNAV);
                sprintf (textBuff, "HIERARCH ESO QC SIGA1 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio Error", "");
        
                sprintf (messageBuffer, "%s", UNAV);
                sprintf (textBuff, "HIERARCH ESO QC KA2 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio", "");
                sprintf (messageBuffer, "%s", UNAV);
                sprintf (textBuff, "HIERARCH ESO QC SIGA2 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio Error", "");

                sprintf (messageBuffer, "%s", UNAV);
                sprintf (textBuff, "HIERARCH ESO QC KB1 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio", "");
                sprintf (messageBuffer, "%s", UNAV);
                sprintf (textBuff, "HIERARCH ESO QC SIGB1 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio Error", "");
    
                sprintf (messageBuffer, "%s", UNAV);
                sprintf (textBuff, "HIERARCH ESO QC KB2 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio", "");
                sprintf (messageBuffer, "%s", UNAV);
                sprintf (textBuff, "HIERARCH ESO QC SIGB2 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio Error", "");
            }
            else
            {
                sprintf (messageBuffer, "%f", trRatios->ka1[i]);
                sprintf (textBuff, "HIERARCH ESO QC KA1 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio", "");
                sprintf (messageBuffer, "%f", trRatios->siga1[i]);
                sprintf (textBuff, "HIERARCH ESO QC SIGA1 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio Error", "");
        
                sprintf (messageBuffer, "%f", trRatios->ka2[i]);
                sprintf (textBuff, "HIERARCH ESO QC KA2 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio", "");
                sprintf (messageBuffer, "%f", trRatios->siga2[i]);
                sprintf (textBuff, "HIERARCH ESO QC SIGA2 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio Error", "");

                sprintf (messageBuffer, "%f", trRatios->kb1[i]);
                sprintf (textBuff, "HIERARCH ESO QC KB1 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio", "");
                sprintf (messageBuffer, "%f", trRatios->sigb1[i]);
                sprintf (textBuff, "HIERARCH ESO QC SIGB1 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio Error", "");
    
                sprintf (messageBuffer, "%f", trRatios->kb2[i]);
                sprintf (textBuff, "HIERARCH ESO QC KB2 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio", "");
                sprintf (messageBuffer, "%f", trRatios->sigb2[i]);
                sprintf (textBuff, "HIERARCH ESO QC SIGB2 CH%d", i);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Transfer Ratio Error", "");
            }
        }
    }
    
    //    Place Holders
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH1", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH2", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH3", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH4", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH5", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH6", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH7", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH8", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH9", "TBD", "TBD", "");

    //    Create the output fits file. If the file exist delete it
    if (stat (fileNames->outFitsName, &buf) == 0) remove (fileNames->outFitsName);
    outFitsPtr = fopen (fileNames->outFitsName, "w");
    if (!outFitsPtr)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create output FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        free (sWidthX);
        free (sWidthY);        
        return;
    }

   cpl_msg_info(cpl_func,"Created Product FITS file: %s \n", fileNames->outFitsName);
    fprintf (midiReportPtr, "Created Product FITS file: %s \n", fileNames->outFitsName);

    //    Write header into the output file
    qfits_header_sort (&outFitsHeader);
    qfits_header_dump (outFitsHeader, outFitsPtr);
    fclose (outFitsPtr);
    qfits_header_destroy (outFitsHeader);

    //    release memory
    free (messageBuffer);
    free (currentTime);
    free (cleanString);
    free (stringTemp);
    free (classification);
    free (textBuff);
    free (sWidthX);    
    free (sWidthY);        

    return;
}
/*****************************************************************************/




/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    createKappaQcLog
* Input/Output:    See function arguments to avoid duplication
* Description:    Copies keywords from the raw input FITS primary header to the 
*               QC log and adds additional product keywords
*
* History:        
* 08-Dec-05     (csabet) Created
******************************************************************************/
void createKappaQcLog (
    char    *inFitsName,
    int        *error)
    
{

    /*    Local Declarations
    --------------------*/
    const char         routine[] = "createKappaQcLog";
    char            *qfitsString, *emptyString, *cleanString;

    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset status */
    *error = 0;

    /* Allocate memory */
    emptyString = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*    Initialise empty string */
    strcpy (emptyString, "\"UNKNOWN\"");

    /* Get all the keywords */
    qfitsString = qfits_query_hdr (inFitsName, "ARCFILE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ARCFILE             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ARCFILE             \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "TELESCOP");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TELESCOP            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TELESCOP            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "INSTRUME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INSTRUME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INSTRUME            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "MJD-OBS");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "MJD-OBS             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "MJD-OBS             \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "DATE-OBS");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DATE-OBS            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DATE-OBS            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "UTC");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "UTC                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "UTC                 %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "LST");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "LST                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "LST                 %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU GUID DEC");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "COU.GUID.DEC        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "COU.GUID.DEC        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU GUID RA");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "COU.GUID.RA         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "COU.GUID.RA         %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU GUID MAG");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "COU.GUID.MAG        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "COU.GUID.MAG        %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 STREHL_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.STREHL_MEAN %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.STREHL_MEAN %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 STREHL_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.STREHL_MEAN %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.STREHL_MEAN %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 STREHL_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.STREHL_RMS  %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.STREHL_RMS  %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 STREHL_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.STREHL_RMS  %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.STREHL_RMS  %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 WFE_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.WFE_MEAN    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.WFE_MEAN    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 WFE_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.WFE_MEAN    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.WFE_MEAN    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 WFE_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.WFE_RMS     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.WFE_RMS     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 WFE_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.WFE_RMS     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.WFE_RMS     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 ENC_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.ENC_MEAN    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.ENC_MEAN    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 ENC_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.ENC_MEAN    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.ENC_MEAN    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 ENC_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.ENC_RMS     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.ENC_RMS     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 ENC_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.ENC_RMS     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.ENC_RMS     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 FWHM_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.FWHM_MEAN   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.FWHM_MEAN   %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 FWHM_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.FWHM_MEAN   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.FWHM_MEAN   %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 FWHM_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.FWHM_RMS    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.FWHM_RMS    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 FWHM_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.FWHM_RMS    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.FWHM_RMS    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 L0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.L0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.L0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 L0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.L0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.L0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 L0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.L0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.L0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 L0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.L0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.L0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 R0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.R0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.R0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 R0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.R0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.R0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 R0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.R0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.R0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 R0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.R0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.R0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 T0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.T0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.T0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 T0_MEAN");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.T0_MEAN     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.T0_MEAN     %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO1 T0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO1.T0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO1.T0_RMS      %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO COU AO2 T0_RMS");
    if (qfitsString == NULL)
        fprintf (midiQcLogPtr, "COU.AO2.T0_RMS      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString);
        fprintf (midiQcLogPtr, "COU.AO2.T0_RMS      %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DIT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DITDELAY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET INT MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.INT.MODE        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.INT.MODE        \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NDIT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NRTS MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR CATG");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.CATG            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.CATG            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TECH");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.TECH            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.TECH            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TYPE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.TYPE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.TYPE            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS CAM NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.CAM.NAME        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.CAM.NAME        \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS FILT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.FILT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.FILT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS GRIS NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.MODE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.MODE            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 TYPE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SHUT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.SHUT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.SHUT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SLIT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AIRM END");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AIRM.END        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AIRM.END        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AIRM START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AIRM.START      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AIRM.START      %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS ALT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.ALT             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.ALT             %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI FWHM END");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.FWHM.END   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.FWHM.END   %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI FWHM START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.FWHM.START %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.FWHM.START %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI PRES");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.PRES       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.PRES       %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI RHUM");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.RHUM       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.RHUM       %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI TAU0 END");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.TAU0.END   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.TAU0.END   %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI TAU0 START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.TAU0.START %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.TAU0.START %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI TEMP");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.TEMP       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.TEMP       %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI WINDDIR");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.WINDDIR    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.WINDDIR    %s \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AMBI WINDSP");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AMBI.WINDSP     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AMBI.WINDSP     %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS AZ");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.AZ              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.AZ              %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF STATION1");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.CONF.STATION1   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.CONF.STATION1   \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF STATION2");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.CONF.STATION2   %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.CONF.STATION2   \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T1X");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.CONF.T1X        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.CONF.T1X        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T1Y");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.CONF.T1Y        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.CONF.T1Y        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T1Z");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.CONF.T1Z        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.CONF.T1Z        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T2X");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.CONF.T2X        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.CONF.T2X        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T2Y");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.CONF.T2Y        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.CONF.T2Y        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS CONF T2Z");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.CONF.T2Z        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.CONF.T2Z        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS PARANG END");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.PARANG.END      %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.PARANG.END      %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS PARANG START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.PARANG.START    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.PARANG.START    %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS PBL12 END");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.PBL12.END       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.PBL12.END       %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO ISS PBL12 START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ISS.PBL12.START     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ISS.PBL12.START     %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.ID              \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.NAME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.NAME            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PROG ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PROG.ID         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PROG.ID         \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.START           \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS TARG NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OCS EXPO1 FNAME1");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TPL.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TPL.ID              \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TPL.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TPL.START           \"%s\" \n", cleanString);}
    
    
    /* Release memory */
    free (emptyString);
    free (cleanString);
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    addProdInfoToKappaQcLog
* Input/Output:    See function arguments to avoid duplication
* Description:    Adds product information to QC log
*
*
* History:        
* 08-Dec-05     (csabet) Created
******************************************************************************/
void addProdInfoToKappaQcLog (
    ImageFormat        *format,    // In: Image format
    MidiFiles        *fileNames,
    TransferRatios    *trRatios,    // In: Transfer Ratios
    int                *error)
{

    /*    Local Declarations
    --------------------*/
    const char         routine[] = "addProdInfoToKappaQcLog";
    int             i;
    FILE            *inFitsBatchPtr;
    char            *stringQfits, *messageBuffer, *currentTime, *textBuff, *cleanString,
                    *stringTemp, *classification;
    time_t          now;
    struct tm        *newTime;
    
    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset parameters */
    *error = 0;

    /*    Allocate memory */
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    
    /*    Get current time/date */
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Add keyword to QC log file
    fprintf (midiQcLogPtr, "PRO.TYPE            \"%s\" \n", format->obsType);
    fprintf (midiQcLogPtr, "PRO.CATG            \"KAPPA_MATRIX\" \n");
    fprintf (midiQcLogPtr, "PRO.ARCFILE         \"%s\" \n", fileNames->archFileName);
    fprintf (midiQcLogPtr, "PRO.PIPEDATE        \"%s\" \n", currentTime);
    fprintf (midiQcLogPtr, "PRO.VERSION         \"%s\" \n", MIDI_PIPE_VERSION);
    fprintf (midiQcLogPtr, "PRO.PIPEFILE        \"%s\" \n", fileNames->pipeFileName);
    fprintf (midiQcLogPtr, "PRO.DID             \"%s\" \n", MIDI_QC_DIC_VERSION);

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    fprintf (midiQcLogPtr, "QC.KAPPA_EXISTS         %d \n", trRatios->exists);
    if (trRatios->exists)
    {
        for (i = 0; i < format->iXWidth; i++)
        {
            if (badChannelList[i])
            {
                fprintf (midiQcLogPtr, "QC.KA1.CH%d         %s \n", i, UNAV);
                fprintf (midiQcLogPtr, "QC.SIGA1.CH%d       %s \n", i, UNAV);
                fprintf (midiQcLogPtr, "QC.KA2.CH%d         %s \n", i, UNAV);
                fprintf (midiQcLogPtr, "QC.SIGA2.CH%d       %s \n", i, UNAV);
                fprintf (midiQcLogPtr, "QC.KB1.CH%d         %s \n", i, UNAV);
                fprintf (midiQcLogPtr, "QC.SIGB1.CH%d       %s \n", i, UNAV);
                fprintf (midiQcLogPtr, "QC.KB2.CH%d         %s \n", i, UNAV);
                fprintf (midiQcLogPtr, "QC.SIGB2.CH%d       %s \n", i, UNAV);
            }
            else
            {
                fprintf (midiQcLogPtr, "QC.KA1.CH%d         %f \n", i, trRatios->ka1[i]);
                fprintf (midiQcLogPtr, "QC.SIGA1.CH%d       %f \n", i, trRatios->siga1[i]);
                fprintf (midiQcLogPtr, "QC.KA2.CH%d         %f \n", i, trRatios->ka2[i]);
                fprintf (midiQcLogPtr, "QC.SIGA2.CH%d       %f \n", i, trRatios->siga2[i]);
                fprintf (midiQcLogPtr, "QC.KB1.CH%d         %f \n", i, trRatios->kb1[i]);
                fprintf (midiQcLogPtr, "QC.SIGB1.CH%d       %f \n", i, trRatios->sigb1[i]);
                fprintf (midiQcLogPtr, "QC.KB2.CH%d         %f \n", i, trRatios->kb2[i]);
                fprintf (midiQcLogPtr, "QC.SIGB2.CH%d       %f \n", i, trRatios->sigb2[i]);
            }
        }
    }
    
    /* Loop through the list of files and write into the QC log */
    i = 1;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);
        
        /* If classification is not given get it from the raw file */
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL) 
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.NAME  \"%s\" \n", i, messageBuffer);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.CATG  \"%s\" \n", i++, classification);
    }
    fclose (inFitsBatchPtr);

    /*    release memory */
    free (messageBuffer);
    free (currentTime);
    free (textBuff);
    free (cleanString);
    free (stringTemp);
    free (classification);
        
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createTransferRatiosFile
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a file for the transfer Matrix in the current directory
*                If update = 2, this process also creates or updates the 
*                database
*
* History:
* 07-Dec-05     (csabet) Created
******************************************************************************/
void createTransferRatiosFile (
    int                update,            // In: Indicates local or database
    ImageFormat        *format,    // In: Image format
    MidiFiles        *fileNames,        // In: Pointer to midi files structure
    TransferRatios    *trRatios,        // In: Transfer Ratios
    int                *error)            // Ou: Status
{
 
    //  Local Declarations
    //    ------------------
    const char    routine[] = "createTransferRatiosFile";
    FILE        *trrPtr=NULL, *trrHistoryPtr=NULL;
    char        *fileString;
    int            x;
    struct stat    buf;
 
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
 
    //  Reset status
    *error = 0;

    if (!(trRatios->exists))
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Transfer Ratios file");
        *error = 1;
        return;
    }
    
    //    Check if transfer matrix file exist. If so delete it
    if (stat (fileNames->trrNameWrite, &buf) == 0)
        remove (fileNames->trrNameWrite);

    //    Open the transfer matrix file for writing
    if ((trrPtr = fopen (fileNames->trrNameWrite, "w")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open file to store transfer matrix");
        *error = 1;
        return;
    }
        
    //    Open a file for Transfer Ratios history
    trrHistoryPtr = fopen (fileNames->trrHistoryName, "a");
    fprintf (trrHistoryPtr, "Transfer Ratios for batch %d \n", batchNumber);
    fprintf (trrHistoryPtr, "------------------------- \n");
    fprintf (trrHistoryPtr, "Channel  ka1 siga1         ka2 siga2         kb1 sigb1         kb2 sigb2 \n");
    
    //    Record the number of channels
    fprintf (trrPtr, "%d \n", format->iXWidth);
    for (x = 0; x < format->iXWidth; x++)
    {
        fprintf (trrPtr, "%f %f %f %f %f %f %f %f \n", trRatios->ka1[x], trRatios->siga1[x], 
            trRatios->ka2[x], trRatios->siga2[x], trRatios->kb1[x], trRatios->sigb1[x], 
            trRatios->kb2[x], trRatios->sigb2[x]);
        fprintf (trrHistoryPtr, "%3d        %6.2f %6.2f      %6.2f %6.2f      %6.2f %6.2f      %6.2f %6.2f \n", x, 
            trRatios->ka1[x], trRatios->siga1[x], trRatios->ka2[x], trRatios->siga2[x], 
            trRatios->kb1[x], trRatios->sigb1[x], trRatios->kb2[x], trRatios->sigb2[x]);
    }
    
    fclose (trrPtr);
    fclose (trrHistoryPtr);
   cpl_msg_info(cpl_func,"Created Transfer Ratios file   %s\n", fileNames->trrNameWrite);
    fprintf (midiReportPtr, "Created Transfer Ratios file   %s\n", fileNames->trrNameWrite);

    //    If update = 2, create or update the static database
    if (update == 2)
    {
        //    Open the transfer matrix file for writing
        fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        sprintf (fileString , "%s_%s.dat", fileNames->transferRatiosName, format->grismId);
        if ((trrPtr = fopen (fileString, "w")) == NULL)
        {
            sprintf (midiMessage, "Cannot open Transfer Ratios database file %s", fileString);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            *error = 1;
            free (fileString);
            return;
        }
        
        //    Record the number of channels    
        fprintf (trrPtr, "%d \n", format->iXWidth);
        for (x = 0; x < format->iXWidth; x++)
        {
            fprintf (trrPtr, "%f %f %f %f %f %f %f %f \n", trRatios->ka1[x], trRatios->siga1[x], 
                trRatios->ka2[x], trRatios->siga2[x], trRatios->kb1[x], trRatios->sigb1[x], 
                trRatios->kb2[x], trRatios->sigb2[x]);
        }
        
       cpl_msg_info(cpl_func,"Updated Transfer Ratios database \n");
        fprintf (midiReportPtr, "Updated Transfer Ratios database \n");
        fclose (trrPtr);
        free (fileString);
    }

    return;
}
/*****************************************************************************/

