/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  estimationFrg.c
* Description:  Contains routines for all data preprocessing algorithms
*
* History:
* 12-Jul-03     (csabet) Created. Derived from (jmeisner 03-Feb-03)
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "midiGlobal.h"
#include "fft.h"
#include "complex_midi.h"
#include "statistics.h"
#include "diagnostics.h"
#include "memoryHandling.h"
#include "errorHandling.h"
#include "midiLib.h"
#include "estimationFrg.h"
#include "photometry.h"
#include "visibility.h"
#include "calibration.h"

/**********************************************************
*   Constant definitions
**********************************************************/
 
/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  estimateFringe
* Input/Output: See function arguments to avoid duplication
* Description:    Computes DISPERSED fringe and Unnormalized visibility
*
*
* History:
* 16-Jul-04     (csabet) Added Interface specification. Code and further
*                        decomposion shall be carried out by Jeff
* 17-Feb-04     (csabet) Created template
* 22-Jan-05     (csabet) Cleaned up and modified data structures
******************************************************************************/
void estimateFringe (
    MidiFiles        *fileNames,            // In: Pointer to file names
    FilterData        *filterInfo,        // In: Filter information
    CompressedData    *compressed,        // In: Compressed interferometry data
    ImageFormat        *format,            // In: Points to the image format
    float           *freqCal,            // IO: If (freqCal[0][0] == 1.0F) then it still needs to be filled in by this function
    float            *dispRawVis,        // Ou: Raw unnormalized visibility
    float            *dispRawVisErr,        // Ou: Error of Raw unnormalized visibility
    int                *error)
{

    /*  Local Declarations
    --------------------*/
    const     char  routine[] = "estimateFringe";
    double    accum = 0.0;
    int        region, frame, X, n;   
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s'\n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s'\n", routine);

   cpl_msg_info(cpl_func,"\nComputing Raw visibilities from batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------- \n");
    fprintf(midiReportPtr, "\nComputing Raw visibilities from batch  %d \n", batchNumber);
    fprintf(midiReportPtr, "------------------------------------- \n");

    //    Reset
    *error = 0;
    
    //    Obtain the visibilities^2, and plot the spectra for each freq channel:
    ProcessSpectrumEachChannel (fileNames, filterInfo, compressed, format, dispRawVis, 
        dispRawVisErr, freqCal, error);
    if (*error) return;

    if (plotFile && diagnostic)
    {
        midiCreatePlotFile2D ("RawVisCombined", "Combined Raw Uncalibrated Visibility", 
            "Channel", "Visibility", 0, dispRawVis, 0, format->iXWidth, 1, 0);
                
        midiCreatePlotFile2D ("RawVisErrCombined", "Combined Raw Uncalibrated Visibility Error", 
            "Channel", "Visibility Error", 0, dispRawVisErr, 0, format->iXWidth, 1, 0);
    }
    
    //    Print results
   cpl_msg_info(cpl_func,"\nAverage values of DISPERSED Fringe \n"
        "---------------------------------- \n"
        "Region     Average values \n");
    fprintf (midiReportPtr, "\nAverage values of DISPERSED Fringe \n"
        "---------------------------------- \n"
        "Region     Average values \n");

    for (region = 0; region < format->numOfRegionsToProcess; region++)
    {
        accum=0.0;
        n = 0;
        for (frame = 0; frame < format->numOfFrames; frame++)
        {
            for (X = 0; X < format->iXWidth; X++)
            {
                if (badChannelList[X])
                    continue;
                    
                accum += (((compressed->iDispFringe)[region])[X])[frame];
                n++;
            }
        }
        
        accum /= n;
       cpl_msg_info(cpl_func,"%2d      %12.2f \n", region, accum);
        fprintf (midiReportPtr, "%2d      %12.2f \n", region, accum);
    }

    return;
}
/*****************************************************************************/



 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  ProcessSpectrumEachChannel
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 16-Jul-04     (JM) provided the code
* 20-Jan-05     (csabet) Cleaned up and integrated
******************************************************************************/
void ProcessSpectrumEachChannel (
    MidiFiles        *fileNames,                // In: Pointer to file names
    FilterData        *filterInfo,            // In: Filter information
    CompressedData    *compressed,            // In: Compressed interferometry data
    ImageFormat        *format,                   // In: Points to the image format
    float            *dispRawVis,            // Ou: 
    float            *dispRawVisErr,            // Ou: 
    float           *freqCal,                // Ou: Frequency calibration
    int                *error)
{

    //    Local Declarations
    //    ------------------
    const    char  routine[] = "ProcessSpectrumEachChannel";
    int     i, j, X, lowFreqIndex, highFreqIndex, loF, hiF,
            freqCalExists, localError, numOfSpectrums;
    float     *spectrum, peakval, peakch;
    float     calib, **waveCal=NULL;
    char    *string, *title, *dataFilePath, *cmdFileName, *argument;
    FILE    *cmdFilePtr=NULL, *dataFilePtr=NULL;
    float    zero=0.0;
    FILE    *wclHistoryPtr=NULL;
    float **PeakMatrix;    
    int   scan, index;
    float *PeakVectorMean;    
    char            *fileNamePeak, *titlePeak;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s'\n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s'\n", routine);

   cpl_msg_info(cpl_func,"\nProcessing each Channel Spectrum from batch %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------------- \n");
    fprintf(midiReportPtr, "\nProcessing each Channel Spectrum from batch %d \n", batchNumber);
    fprintf(midiReportPtr, "------------------------------------------- \n");
    
    //    Initialize
    *error = 0;
    freqCalExists = 0;

    //    Recalculate FFT size and the frequency indices
    if (diagnostic)cpl_msg_info(cpl_func,"FFT size was = %d \n", format->fftsize);
    fprintf (midiReportPtr, "FFT size was = %d \n", format->fftsize);
    SetFFTsize (format->fftsize);
    if (diagnostic)cpl_msg_info(cpl_func,"FFT size is now = %d \n", format->fftsize);
    fprintf (midiReportPtr, "FFT size is now = %d \n", format->fftsize);
    calib = format->optFreqCal / ((float) format->fftsize);
    lowFreqIndex = (int) (filterInfo->optFreqLo / calib);    // Only look at the relevant frequencies
    highFreqIndex = (int) (filterInfo->optFreqHi / calib);    // Only look at the relevant frequencies
    if (highFreqIndex > (format->fftsize/2)) highFreqIndex = (format->fftsize/2);

    //    Get calibrated channel frequencies
    waveCal = callocWaveCalibration (format->numOfDetectorRegions, format);
    getCalibratedChWave (fileNames,    format,    waveCal, error);
    if (!(*error))
    {
        freqCalExists = 1;

        //    Open a file for wavelength calibration history
        wclHistoryPtr = fopen (fileNames->wclHistoryName, "a");
        fprintf (wclHistoryPtr, "Wavelength Calibration \n");
        fprintf (wclHistoryPtr, "Grism ID          = %s \n", format->grismId);
        fprintf (wclHistoryPtr, "Number of Records = %d \n", format->iXWidth);
        fprintf (wclHistoryPtr, "--------------------------------------- \n");
        
        if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
        {
            for (X = 0; X < format->iXWidth; X++) fprintf (wclHistoryPtr, "%f %f %f %f\n", 
                waveCal[0][X], waveCal[1][X], waveCal[2][X], waveCal[3][X]);
            fclose (wclHistoryPtr);

            //    Combine channels. Only regions 2 and 3 (Interferometry is used)
            for (i = 0; i < format->iXWidth; i++)
                freqCal[i] = (waveCal[1][i] + waveCal[2][i]) * 0.5F; // Average them 
        }
        else
        {
            for (X = 0; X < format->iXWidth; X++) fprintf (wclHistoryPtr, "%f %f\n", 
                waveCal[0][X], waveCal[1][X]);
            fclose (wclHistoryPtr);

            //    Combine channels
            for (i = 0; i < format->iXWidth; i++)
                freqCal[i] = (waveCal[0][i] + waveCal[1][i]) * 0.5F; // Average them 
        }

        //    Create plot
            if (plotFile) midiCreatePlotFile2D ("WaveCalibCombined", "Combined Wavelength Claibration", 
                "Channel", "Wavelength (micron)", 0, freqCal, 0, format->iXWidth, 1, 0);
    }
    else
    {
        //    Readjust
        *error = 0;
        if (!(highFreqIndex - lowFreqIndex))
        {    
            *error = 1;
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No channels to process");
            return;
        }
    }
    freeWaveCalibration (format->numOfDetectorRegions, waveCal);
    
    if (diagnostic)cpl_msg_info(cpl_func,"\nOptical frequency calibration = %6.2f THz, calibration per FFT resolution = %6.2f THz \n", 
        format->optFreqCal, calib);
    if (diagnostic)cpl_msg_info(cpl_func,"Low frequency index (for %f THz) = %d \n", filterInfo->optFreqLo, lowFreqIndex);
    if (diagnostic)cpl_msg_info(cpl_func,"High frequency index (for %f THz) = %d \n", filterInfo->optFreqHi, highFreqIndex);
    if (diagnostic)cpl_msg_info(cpl_func,"With %d frames, and %d frames / scan \n",  format->numOfFrames, format->framesPerScan);
    if (diagnostic)cpl_msg_info(cpl_func,"\nProcessing Combined visibility for each channel \n");
    
    fprintf (midiReportPtr, "\nOptical frequency calibration = %6.2f THz, calibration per fft resolution = %6.2f THz QCLOG \n", 
        format->optFreqCal, calib);
    fprintf (midiReportPtr, "Low frequency index (for %f THz) = %d QCLOG \n", filterInfo->optFreqLo, lowFreqIndex);
    fprintf (midiReportPtr, "High frequency index (for %f THz) = %d QCLOG \n", filterInfo->optFreqHi, highFreqIndex);
    fprintf (midiReportPtr, "With %d frames, and %d frames / scan QCLOG \n",  format->numOfFrames, format->framesPerScan);
    fprintf (midiReportPtr, "\nProcessing Combined visibility for each channel QCLOG \n");

   cpl_msg_info(cpl_func,"We now carry out Frame, Scan and Channel rejection based on Signal/Noise \n\n");
    fprintf(midiReportPtr, "We now carry out Frame, Scan and Channel rejection based on Signal/Noise \n\n");

    if (diagnostic) 
       cpl_msg_info(cpl_func,"\nIncoherent visibility results:\n\nChannel   Wavelength (micron)   Log slope   Offset   White noise           Raw vis       Raw vis Error\n");
    fprintf (midiReportPtr, 
        "\nIncoherent visibility results:\n\nChannel   Wavelength (micron)   Log slope   Offset   White noise           Raw vis       Raw vis Error QCLOG\n");

    //    Prepare header for plots
    if (plotFile)
    {
        //    Allocate memory
        dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        cmdFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        string = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                
        //    Create the plot file
        sprintf (string, "3dPowerSpectrumCombinedInterf");
        sprintf (title, "Combined Interferometry Power Spectrum");
        sprintf (dataFilePath, "%s%s.%s.plt", outFileDir, outRootName, string);
        dataFilePtr = fopen (dataFilePath, "w");
    }

    //    Allocate memory
    spectrum = (float *) calloc (format->fftsize/2, sizeof(float)); 

    //Allocate memory for PeakMatrix:
    
    PeakMatrix=calloc( format->iXWidth,sizeof (float*) ); 
    for(i=0;i<format->iXWidth;i++){
       PeakMatrix[i]=calloc(format->numOfScans,sizeof(float));      
    }
    

    //    Process valid channels only
    for (X = 0; X < format->iXWidth; X++)
    {
        localError = 0;
        if (diagnostic)cpl_msg_info(cpl_func,"%3d",  X);
        fprintf (midiReportPtr, "%3d",  X);

        if (badChannelList[X])
        {
            if (diagnostic)cpl_msg_info(cpl_func,"          %5.2f               <-- %s --> \n", freqCal[X], UNAV);
            fprintf (midiReportPtr, "          %5.2f               <-- %s -->                     QCLOG \n", freqCal[X], UNAV);
            
            if (plotFile)
            {
                //    Fill in the empty values
                for (j = 0; j < format->fftsize/2; j++)
                    fprintf (dataFilePtr, "%f ", zero);
                fprintf (dataFilePtr, "\n");
            }
            continue;
        }
        
        //    Reset mmory
        memset (spectrum, 0, (sizeof(float)) * format->fftsize/2);

        //    Set low and high frequancy index for this channel
        if (!freqCalExists)
        {
            loF = lowFreqIndex - 10;    //    Not sure about this yet. But it's not a likely situation
            hiF = highFreqIndex;
        }
        else                         
        {
            loF = (int) (((SPEED_OF_LIGHT / freqCal[X])/calib) - 10);
            hiF = (int) (((SPEED_OF_LIGHT / freqCal[X])/calib) + 10);
        }            
        if (loF < 0) loF = 0;
        if (hiF > format->fftsize/2) hiF = format->fftsize/2;


        
        //    Get number of spectrums
        numOfSpectrums = getChannelSpectrum (X, loF, hiF, compressed->rejectList[X], 
                                             (((compressed->iDispFringe)[0])[X]), (((compressed->iDispFringe)[1])[X]), format, spectrum, PeakMatrix[X]);


        

        //    Check if any spectrums
        if (!numOfSpectrums)
        {
            badChannelList[X] |= BSL_DATA_ERROR;
            if (plotFile)
            {
                for (j = 0; j < format->fftsize/2; j++)
                    fprintf (dataFilePtr, "%f ", zero);
                fprintf (dataFilePtr, "\n");
            }
            if (diagnostic)cpl_msg_info(cpl_func,"          %5.2f               <-- %s --> \n", freqCal[X], UNAV);
            fprintf (midiReportPtr, "          %5.2f               <-- %s -->                     QCLOG \n", freqCal[X], UNAV);
            continue;
        }
        
        //    Reject bad spectrums
        localError = 0;
        for (j = 0; j < format->fftsize/2; j++)
        {
            if (isnan (spectrum[j]))
            {
                localError = 1;
                badChannelList[X] |= BSL_DATA_ERROR;
                break;
            }
        }
        if (localError)
        {
            if (plotFile)
            {
                for (j = 0; j < format->fftsize/2; j++)
                    fprintf (dataFilePtr, "%f ", zero);
                fprintf (dataFilePtr, "\n");
            }
            if (diagnostic)cpl_msg_info(cpl_func,"          %5.2f               <-- %s --> \n", freqCal[X], UNAV);
            fprintf (midiReportPtr, "          %5.2f               <-- %s -->                     QCLOG \n", freqCal[X], UNAV);
            continue;
        }

        //    Write into the plot file
        if (plotFile)
        {
            for (j = 0; j < format->fftsize/2; j++)
                fprintf (dataFilePtr, "%f ", spectrum[j]);
            fprintf (dataFilePtr, "\n");
        }

        if (!freqCalExists)
        {
            findSpectralPeak (spectrum, lowFreqIndex, highFreqIndex, &peakch, &peakval);
            freqCal[X] = peakch*calib;
        }
        else                         
        {
            if (freqCal[X] == 0.0) peakch = 0.0;
            else peakch = (SPEED_OF_LIGHT / freqCal[X])/calib;    // lookup from table inputted to this function
            peakval = spectrum[(int)(peakch+0.5F)];    // Read out the peak value directly (was for plotting. Not used);
        }            
            
        if (diagnostic)cpl_msg_info(cpl_func,"          %5.2f  ", freqCal[X]);
        fprintf (midiReportPtr, "          %5.2f  ", freqCal[X]);
 
        //    Compute unnormalized visibility estimate from spectrum
        if (peakch)    
            dispRawVis[X] = analyseSpectrum (format, peakch, spectrum, calib, &(dispRawVisErr[X]));
        else 
        {
            dispRawVis[X] = 0.F;
            dispRawVisErr[X] = 0.F;
        }
        
        if (diagnostic)cpl_msg_info(cpl_func,"        %11.4f", dispRawVis[X]);
        if (diagnostic)cpl_msg_info(cpl_func,"        %9.4f", dispRawVisErr[X]);
        fprintf (midiReportPtr, "        %11.4f", dispRawVis[X]);
        fprintf (midiReportPtr, "        %9.4f", dispRawVisErr[X]);
            
        if (diagnostic)cpl_msg_info(cpl_func,"\n");
        fprintf (midiReportPtr,    "   QCLOG\n");
    }
    //    Release memory
    free (spectrum);


    //Calculate the mean values of the PeakMatrix

    PeakVectorMean=calloc(format->numOfScans,sizeof (float) ); 
    

    for (scan = 0; scan < format->numOfScans; scan++)
    {
       index=0;
       for (X = 0; X < format->iXWidth; X++)
       {
          if(PeakMatrix[X][scan]>0){
             PeakVectorMean[scan]+=PeakMatrix[X][scan];
             index++;
          }
       }
       PeakVectorMean[scan]=PeakVectorMean[scan]/(float)index;
/*        cpl_msg_info(cpl_func,"PeakVectorMean: %f",PeakVectorMean[scan]); */
/*        cpl_msg_info(cpl_func,"from %d Chanels\n",index);  */
    }
    
    //Write the mean values (averaged over the channel) in a file
    fileNamePeak = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    titlePeak = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sprintf (fileNamePeak, "PeakScanChannelaveraged");
    sprintf (titlePeak, "PeakScanChannelaveraged");

    midiCreatePlotFile2D (fileNamePeak, titlePeak, "Scan", "Peak", 0, PeakVectorMean, 0, format->numOfScans, 1, 0);
    free (fileNamePeak);
    free (titlePeak);
    
    
    //Free memory 

    free(PeakVectorMean);

    for(i=0;i<format->iXWidth;i++){
       free (PeakMatrix[i]);      
    }
    free (PeakMatrix);

    
    //    Fill in the empty values and close dataFilePtr
    if (plotFile)
        fclose (dataFilePtr);

    //    Display plot file
    if (plotFile > 1)
    {
        //    Create the command file
        sprintf (cmdFileName, "plotCmdFile.grb");
        cmdFilePtr = fopen (cmdFileName, "w");
        fprintf (cmdFilePtr, "set title \'%s\' \n", title);
        fprintf (cmdFilePtr, "set xlabel 'Frequency Bin' \n");
        fprintf (cmdFilePtr, "set ylabel 'Channel' \n");
        fprintf (cmdFilePtr, "set zlabel 'Power Spectrum' \n");
        fprintf (cmdFilePtr, "set xrange [0:%d] \n", format->fftsize/2);
        fprintf (cmdFilePtr, "set yrange [0:%d] \n", format->iXWidth);
        fprintf (cmdFilePtr, "set ticslevel 0 \n");
        fprintf (cmdFilePtr, "splot \'%s\' matrix with lines 0 \n", dataFilePath);
        if (plotDuration == -1)
        {
           cpl_msg_info(cpl_func,"\n");
            fprintf (cmdFilePtr, "pause %d \"          ***** Hit Return to continue *****\" \n", plotDuration);
        }
        else
            fprintf (cmdFilePtr, "pause %d \n", plotDuration);
        
        fclose (cmdFilePtr);

        //    Run the command file
        sprintf (argument, "gnuplot -title \"MIDI DRS\" -geometry 900x900+0+0 %s", cmdFileName);
        int ret = system (argument);
        if (ret != 0) {
             cpl_msg_warning(cpl_func,"Failed to call the system command");
         }

        //    Remove temporary files
        remove (cmdFileName);
    }

    if (plotFile)
    {
        if (plotFile == 3) remove (dataFilePath);    

        //    Release memory
        free (dataFilePath);
        free (cmdFileName);
        free (argument);
        free (string);
        free (title);
    }

    //    Report
    if (!freqCalExists)
    {
       cpl_msg_info(cpl_func,"\nWavelength Calibration Calculated from Interferometry data \n");
       cpl_msg_info(cpl_func,"---------------------------------------------------------- \n");
        fprintf (midiReportPtr, "\nWavelength Calibration Calculated from Interferometry data \n");
        fprintf (midiReportPtr, "---------------------------------------------------------- \n");
        for (X = 0; X < format->iXWidth; X++)
        {
            if (freqCal[X] == 0.0)
            {
               cpl_msg_info(cpl_func,"%3d   %7.4f \n", X, zero);
                fprintf (midiReportPtr, "%3d   %7.4f \n", X, zero);
            }
            else
            {
                freqCal[X] = SPEED_OF_LIGHT/freqCal[X];
               cpl_msg_info(cpl_func,"%3d   %7.4f \n", X, freqCal[X]);
                fprintf (midiReportPtr, "%3d   %7.4f \n", X, freqCal[X]);
            }
        }
       cpl_msg_info(cpl_func,"\n");
        fprintf (midiReportPtr, "\n");

        //    Open a file for wavelength calibration history
        wclHistoryPtr = fopen (fileNames->wclHistoryName, "a");
        fprintf (wclHistoryPtr, "Wavelength Calibration from Interferometry data \n");
        fprintf (wclHistoryPtr, "Grism ID          = %s \n", format->grismId);
        fprintf (wclHistoryPtr, "Number of Records = %d \n", format->iXWidth);
        fprintf (wclHistoryPtr, "--------------------------------------- \n");
        for (X = 0; X < format->iXWidth; X++) fprintf (wclHistoryPtr, "%f \n", freqCal[X]);
        fclose (wclHistoryPtr);

        //    Create plot
        if (plotFile) midiCreatePlotFile2D ("WaveCalibInterfData", "Wavelength Claibration from Interferometry data", 
            "Channel", "Wavelength (micron)", 0, freqCal, 0, format->iXWidth, 1, 0);

        //    Readjust valid channels
        for (X = 0; X < format->iXWidth; X++)
        {
            if (freqCal[X] <= 0.F)
                badChannelList[X] |= BSL_DATA_ERROR;
        }
    }
    
    return;
}
/*****************************************************************************/


 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getChannelSpectrum
* Input/Output: See function arguments to avoid duplication
* Description:    This function ALLOCATES fftsize/2 elements for a PS, and returns a 
*                pointer to it
*
* History:
* 16-Jul-04     (JM) provided the initial idea
* 20-Jan-05     (csabet) Cleaned up and extensively modified
******************************************************************************/
int getChannelSpectrum (
    int            X,                // In: Channel number
    int            lowFreqIndex,    // In: First index to consider
    int            highFreqIndex,    // In: Last index to consider + 1
    int            *rejectList,    // In: Pointer to the begining of the bad Frame List for one channel
    float        *inData1,        // In: Pointer to position of frame 0
    float        *inData2,        // In: Pointer to position of frame 0 of SECOND region. Null if not used
    ImageFormat    *format,        // In: Points to the image format.
    float        *output,        // Ou: Output spectrum
    float        *Peakvector)        // Ou: Output spectrum

{

    //    Local Declarations
    //    ------------------
    int                i, k, scan, numOfPs, *badFrame, minFrames, fftSizeHalf, 
                    scanFailed, peakIndex;
    float            accum, dclevel, scale, lengthAve, *data1, *data2, *spectrum, 
                    median, peak, ratio, *spectrumPtr;
    struct Complex    *input2ft;
    char            *fileName, *title;
/*     char            *titlePeak; */
/*     char            *fileNamePeak; */
    
    //    Algorithm
    //    ---------
    minFrames = MIN_FRAMES_PER_SCAN * format->framesPerScan;
    fftSizeHalf = (format->fftsize >> 1);
    
    //    Allocate memory
    FFTarray = (struct Complex *) calloc(MAXFFTSIZE, sizeof(struct Complex));
    input2ft = (struct Complex *) calloc(format->fftsize, sizeof(struct Complex));
    spectrum = (float *) calloc (fftSizeHalf, sizeof (float));

    //    Compute spectrum
    numOfPs = 0;
    lengthAve = 0.0;

/*     // Open file for plotting the Peak and the Median as a function of the scan */

/*     fileNamePeak = (char *) calloc (MAX_STRING_LENGTH, sizeof (char)); */
/*     titlePeak = (char *) calloc (MAX_STRING_LENGTH, sizeof (char)); */
/*     sprintf (fileNamePeak, "PeakSpectrumScanChannel%d", X); */
/*     sprintf (titlePeak, "PeakChannel %d", X); */


    for (scan = 0; scan < format->numOfScans; scan++)
    {
        scanFailed = 0;
        memset (input2ft, 0, (sizeof(struct Complex)) * format->fftsize);
        badFrame = rejectList + scan * format->framesPerScan;    // Set to beginning of this scan
        data1 = inData1 + scan * format->framesPerScan;            // Set to beginning of this scan
        data2 = inData2 + scan * format->framesPerScan;            // Set to beginning of this scan
        
        //  Accumulate to the DC level here:
        accum =  0.F;
        k = 0;
        for (i = 0; i < format->framesPerScan; i++)
        {
            //    Accumulate good frames only
            if (!(*badFrame))
            {
                accum += *data1 - *data2;

                //    Also copy to fft input array
                input2ft[k].r = *data1 - *data2;
                k++;
            }
            data1++;
            data2++;
            badFrame++;
        }
        lengthAve += (float) k;

        //    Process this scan if there were enough good frames
        if (k > minFrames)
        {
            // DC level of this scan
            dclevel = accum / ((float) k);
        
            //    Now subtract the dc level from the values we just inputted to the fft input array:
            for (i = 0; i < k; i++)
                input2ft[i].r -= dclevel;
            
            //    Now apply the window function
            input2ft[--i].r *= 0.145F;    // Should point to last nonzero element.
            input2ft[0].r *= 0.145F;
            input2ft[--i].r *= 0.5F;
            input2ft[1].r *= 0.5F;
            input2ft[--i].r *= 0.855F;
            input2ft[2].r *= 0.855F;
 
            //     Now do the FFT:
            FFT (input2ft, FFTlevel);    // Output is waiting in FFTarray

            //    Analyse the scan spectrum
            spectrumPtr = spectrum;
            for (i = 0; i < fftSizeHalf; i++)
                *spectrumPtr++ = Cmag2(FFTarray[i]);
            
            peak = signalPeak (spectrum, 0, fftSizeHalf, &peakIndex);
            median = signalMedian (spectrum, 0, fftSizeHalf);
            ratio = peak/median;

            Peakvector[scan]=peak;            


            //    Check validity of the peak power
            if ((ratio > 10.0) && (peakIndex > lowFreqIndex) && (peakIndex < highFreqIndex))
            {
                //     Now add the (sqrd mag) of the fft result to the power spectrum output:
                for (i = 0; i < fftSizeHalf; i++)
                    output[i] += spectrum[i];
            
                //    Increment number of power spectrums
                numOfPs++;
            }
            else
            {
                scanFailed = 1;
                //    Transfer bad scans to rejection list
                for (i = 0; i < format->framesPerScan; i++)
                    rejectList[i + scan * format->framesPerScan] |= BSL_SNR_ERROR;
            }

            //    Plot
            if (plotFile && diagnostic > 2)
            {
                if (scanFailed)cpl_msg_info(cpl_func,"\n    <<ERROR>>: Scan %3d rejected \n\n", scan);
               cpl_msg_info(cpl_func,"    Expected low frequency index  = %3d \n", lowFreqIndex);
               cpl_msg_info(cpl_func,"    Expected high frequency index = %3d \n", highFreqIndex);
               cpl_msg_info(cpl_func,"    Found peak index at           = %3d \n", peakIndex);
               cpl_msg_info(cpl_func,"    peak/median                   = %f \n", ratio);
                if (scanFailed) fprintf (midiReportPtr, "\n    <<ERROR>>: Scan %3d rejected \n\n", scan);
                fprintf (midiReportPtr, "    Expected low frequency index  = %3d \n", lowFreqIndex);
                fprintf (midiReportPtr, "    Expected high frequency index = %3d \n", highFreqIndex);
                fprintf (midiReportPtr, "    Found peak index at           = %3d \n", peakIndex);
                fprintf (midiReportPtr, "    peak/median                   = %f \n", ratio);
        
                fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                sprintf (fileName, "SpectrumScan%dChannel%d", scan, X);
                sprintf (title, "Power Spectrum. Scan %d, Channel %d", scan, X);
                midiCreatePlotFile2D (fileName, title, "Frequancy bin", "Power Spectrum", 
                    1, spectrum, 0, fftSizeHalf, 1, 0);
                free (fileName);
                free (title);
            }           

            
        }
    }
 
       
/*     midiCreatePlotFile2D (fileNamePeak, titlePeak, "Scan", "Peak", 0, Peakvector, 0, format->numOfScans, 1, 0); */
/*     free (fileNamePeak); */
/*     free (titlePeak); */
    
    

    //    All done accumulating. Now normalize:
    if (numOfPs > 0)
    {
        //    Take into account energy in scan after windowed
        lengthAve /= ((float) numOfPs);
        scale = 4.F * ((float) format->fftsize) / (((float) numOfPs) * (lengthAve - 3.996));    
        for (i = 0; i < fftSizeHalf; i++)
            output[i] *= scale;
    }

    //    Release memory
    free (input2ft);
    free (spectrum);
    free (FFTarray);

    return (numOfPs);
}
/********************************************************************************/
 
 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  findSpectralPeak
* Input/Output: See function arguments to avoid duplication
* Description:    Performs a 3-pt interpolation
*
* History:
* 21-Jul-03     (JM)
* 20-Jan-05        (csabet) Integrated
******************************************************************************/
void findSpectralPeak (
    float    *data,            // In: Spectrum input
    int        lowFreqIndex,    // In: First index to consider
    int        highFreqIndex,    // In: Last index to consider + 1
    float    *peakch,        // Ou: Interpolated channel number
    float    *peakinterp)    // Ou: Interpolated value at that position
{

    /*  Local Declarations
    --------------------*/
    int        i, j = lowFreqIndex;
    float    peakval = 0.F;


    /*  Algorithm
    -----------*/
    for (i = lowFreqIndex; i < highFreqIndex; i++)
    {
        if (data[i] > peakval)
        {
            j = i;
            peakval = data[i];
        }
    }

    if ((j == lowFreqIndex) || (j == (highFreqIndex-1)))
    {    // basically that means it failed:
        *peakch = 0.F;
        *peakinterp = 0.F;
        return;
    }

    //    Do 3-pt interp and return results:     
    *peakch = ((float) j) + (data[j-1]-data[j+1])/(2.F*(data[j-1]+data[j+1]-2.F*data[j]));
    if (*peakch<0.F)  *peakch = 0.F;    // just in case.....
    if (*peakch > ((float) highFreqIndex)) *peakch = ((float) highFreqIndex);
    *peakinterp  = data[j] - 0.125F*(data[j+1] - data[j-1]) * (data[j+1] - data[j-1]) / 
        (data[j+1] + data[j-1] - 2.F*data[j]);
    
    return;
}
/*****************************************************************************/
 
 
 
 
 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  analyseSpectrum
* Input/Output: See function arguments to avoid duplication
* Description:    Computes unnormalized visibility estimate from spectrum
*
* History:
* 21-Jul-03     (JM)
* 20-Jan-05        (csabet) Cleaned up, added error calculation and Integrated
******************************************************************************/
float analyseSpectrum (                // Ou: Unnormalized visibility
    ImageFormat    *format,            // In: Points to the image format
    float        fringeFreq,            // In: Expected optical frequency of fringe, in units of 
                                    //     *spectrum (i.e. actualfreq = fringeFreq * optfreqcal/fftsize)
    float        *spectrum,            // In: Main input
    float        calib,                // In: Optical freq cal, per fft freq point
    float        *fringePowerErr)    // Ou: Error of Unnormalized visibility
{

    /*  Local Declarations
    --------------------*/
    float    fringePower, fringeWidth, slope, offset;
    int        fringeLo1,fringeLo2, tenTHz, fringeHi1,fringeHi2, i;
    float    *noiseSpec, loPwr, hiPwr;
    int     lopt, hipt;
    float      whitenoise = 0.F;
    int     pass;

    /*  Algorithm
    -----------*/
 
    //    Allocate memory
    noiseSpec = (float *) calloc (format->fftsize/2, sizeof(float));
 
    //    Very crude estimate of the half width of the fringe. (Need to increase when atmospheric OPD velocity is important)
    fringeWidth = 1.5 * ((float) format->fftsize) / 
        (((float) format->framesPerScan) - 4.F);    // -4 to take windowing into account
 
    fringeHi1 = (int)(0.5F + fringeFreq + fringeWidth);
    fringeHi2 = (int)(0.5F + fringeFreq + 3.F * fringeWidth);
    if (fringeHi2 >= (format->fftsize/2 - 5))
        fringeHi2 = (format->fftsize/2 - 6);
    if (fringeHi1 >= (fringeHi2-1))    // very unlikely
        fringeHi1 = fringeHi2-2;
 
    fringeLo1 = (int)(0.5F + fringeFreq - 3.F*fringeWidth);
    fringeLo2 = (int)(0.5F + fringeFreq - fringeWidth);

    //    NEW: Do NOT include anything below 10 THz:
    tenTHz = (int) (0.5F + 10.F/calib) ;
    if (fringeLo1 < tenTHz)
        fringeLo1 = tenTHz;
    if (fringeLo1 >= (fringeLo2-1))    // JUST in case
        fringeLo1 = fringeLo2-2;
 
    //    NEW: Do 5 passes, each with updated estimate of whitenoise:
    for (pass = 0; pass < 5; pass++) 
    {
        //    Make est of whitenoise from very highest freqs. First time around, noiseSpec==0
        whitenoise = 0.F;    // to accum
        for (i = fringeHi2; i < format->fftsize/2; i++)
            whitenoise += (spectrum[i] -  noiseSpec[i]);
             
        whitenoise /= (float) (format->fftsize/2 - fringeHi2);    // Estimate of whitenoise level. Acual noiselevel = noiseSpec[i] + whitenoise
              
        loPwr = 0.0;    // To accum
        for (i = fringeLo1; i < fringeLo2; i++)
        {
            if (spectrum[i] > (1.05F*whitenoise))
                loPwr += log((double) (spectrum[i] - whitenoise));
            else
                loPwr += log((double) (.05F*whitenoise));
        }
 
        loPwr /= (double) (fringeLo2 - fringeLo1);    // Get average

        hiPwr = 0.0;    // To accum
        for (i = fringeHi1; i < fringeHi2; i++)
        {
            if (spectrum[i] > (1.05F*whitenoise))            
                hiPwr += log((double) (spectrum[i] - whitenoise));
            else
                hiPwr += log((double) (.05F*whitenoise));
        }
 
        hiPwr /= (double) (fringeHi2 - fringeHi1);    // Get average
 
        //    Now fit a line to it!
        slope = (hiPwr-loPwr)/(log((fringeHi2+fringeHi1-1)*0.5F) - log((fringeLo2 + fringeLo1-1)*0.5F));       
        offset = loPwr - slope*log((fringeLo2 + fringeLo1-1)*0.5F);
 
        //    Now fill in the estimated "noise spectrum":
        for (i = 1; i < (format->fftsize/2); i++)
            noiseSpec[i] = exp(offset + slope*log(i));
        
        noiseSpec[0] = noiseSpec[1];
    }

    if (diagnostic)cpl_msg_info(cpl_func,"           %6.3f       %7.2f    %8.2f ", slope, offset, whitenoise);
    fprintf (midiReportPtr, "           %6.3f       %7.2f    %8.2f ", slope, offset, whitenoise);
 
    fringePower = 0.F;        // to accum
    *fringePowerErr = 0.F;    // Added, csabet 21-Feb-05
    lopt = (int)(fringeFreq - fringeWidth);
    if (lopt < 0) lopt = 0;
        hipt = (int)(fringeFreq + fringeWidth + 1.F);
    
    if (hipt >= format->fftsize/2) hipt = format->fftsize/2 - 1;
    
    for (i = lopt; i <= hipt; i++)
    {
        fringePower += (spectrum[i] - noiseSpec[i] - whitenoise);
        *fringePowerErr += (noiseSpec[i] + whitenoise);
    }
        
    //    Release memory
    free (noiseSpec);

    return (fringePower);
}
/*****************************************************************************/

