/******************************************************************************
*******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software
*
* Module name:    iauWrite.c
* Description:    Routines to write MIDI product data in accordance with the IAU 
*                Exchange format 
*
* History:        
* 01-Jun-06        (csabet) Removed static memory allocation
* 10-Jul-03        (csabet) Created. Derived from John Young <jsy1001@cam.ac.uk>
*******************************************************************************
******************************************************************************/

/******************************************************************************
*    Compiler directives
******************************************************************************/

/******************************************************************************
*    Include files
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <cpl.h>
#include <string.h>
#include "iauExchange.h"
#include "iauWrite.h"
#include "midiGlobal.h"
#include "qfits.h"
#include "qfits_header.h"

/******************************************************************************
*    Global Variables 
******************************************************************************/

/******************************************************************************
*    Local Prototypes 
******************************************************************************/

/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    writeOiArray
* Input/Output:    See function arguments to avoid duplication
* Description:    Writes OI_ARRAY fits binary table
*
*
* History:        
* 15-Apr-04        (csabet) Converted to qfits
* 10-Jul-03        (csabet) Created. Derived from John Young <jsy1001@cam.ac.uk>
******************************************************************************/
void writeOiArray (
    char    *outFitsName,     // In: Name of output FITS file
    OiArray    *array,        // In:    Pointer to the OiArray structure
    int     *status)    // IO:    Error status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "writeOiArray";
    FILE        *outFitsPtr;
    qfits_table    *oiArrayTable;
    int        tableWidth, numOfColumns, numOfRows, numOfElemStaIndex, 
            numOfElemStaName, numOfElemTelName, numOfElemDiameter, numOfElemStaxyz;
    qfits_col    *colPtr;
    qfits_header    *oiArrayHeader;
    const void    **outArrayPtr;
    char        *telName, *staName, *telPtr, *staPtr;
    char        *tempString;
    int        i, j, k, arrayStatus;
    float        *diameter;
    double        *staxyz;
    short        *staIndex;
    
    
    //    Algorithm
    //    ---------

    //    Reset status and set table parameters
    *status = 0;
    arrayStatus = 0;
    numOfRows = array->nelement;
    numOfColumns = 5;
    tableWidth = -1;
    numOfElemTelName = 16;
    numOfElemStaName = 16;
    numOfElemStaIndex = 1;
    numOfElemDiameter = 1;
    numOfElemStaxyz = 3;

    //    Allocate memory
    tempString = (char *) calloc ( MAX_STRING_LENGTH, sizeof (char));
    
    //    Open the output FITS file for writing
    outFitsPtr = fopen (outFitsName, "a");
    if (outFitsPtr == NULL)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot open output FITS file for writing %s\n", routine, outFitsName);
        free (tempString);
        return;
    }

    //    Create a new table
    oiArrayTable = qfits_table_new (outFitsName, QFITS_BINTABLE, tableWidth, numOfColumns, numOfRows);
    if (!oiArrayTable)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot create OI_ARRAY table \n", routine);
        free (tempString);
        fclose (outFitsPtr);
        remove (outFitsName);
        return;
    }

    //    Fill in the column information
    colPtr = oiArrayTable->col;
    qfits_col_fill (colPtr, numOfElemTelName, 0, sizeof(char), TFITS_BIN_TYPE_A, "TEL_NAME", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemStaName, 0, sizeof(char), TFITS_BIN_TYPE_A, "STA_NAME", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemStaIndex, 0, sizeof(short), TFITS_BIN_TYPE_I, "STA_INDEX", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemDiameter, 0, sizeof(float), TFITS_BIN_TYPE_E, "DIAMETER", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemStaxyz, 0, sizeof(double), TFITS_BIN_TYPE_D, "STAXYZ", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);

    //    Create OI_ARRAY header
    oiArrayHeader = qfits_table_ext_header_default (oiArrayTable);
    qfits_header_add (oiArrayHeader, "EXTNAME", "OI_ARRAY", "Name of this binary table extension", "");      
    qfits_header_add (oiArrayHeader, "OI_REVN", array->revision, "Revision number of the table definition", "");
    qfits_header_add (oiArrayHeader, "ARRNAME", array->arrname, "Array name", "");
    qfits_header_add (oiArrayHeader, "FRAME", array->frame, "Coordinate frame", "");
    sprintf (tempString, "%f", array->arrayx);
    qfits_header_add (oiArrayHeader, "ARRAYX", tempString, "Array centre x coordinate", "");
    sprintf (tempString, "%f", array->arrayy);
    qfits_header_add (oiArrayHeader, "ARRAYY", tempString, "Array centre y coordinate", "");
    sprintf (tempString, "%f", array->arrayz);
    qfits_header_add (oiArrayHeader, "ARRAYZ", tempString, "Array centre z coordinate", "");

    //    Diagnostics
    if (diagnostic > 2) qfits_header_dump (oiArrayHeader, midiReportPtr);

    //    Allocate space for the array of data
    outArrayPtr = (const void **) calloc (numOfColumns, sizeof(const void *));
    telName = (char *) calloc (numOfRows *  MAX_STRING_LENGTH, sizeof(char));
    staName = (char *) calloc (numOfRows *  MAX_STRING_LENGTH, sizeof(char));
    staIndex = (short *) calloc (numOfRows, sizeof(short));
    diameter = (float *) calloc (numOfRows, sizeof(float));
    staxyz = (double *) calloc (numOfRows * numOfElemStaxyz, sizeof(double));
    
    //    Load buffer
    telPtr = telName;
    for (i = 0; i < numOfRows; i++)
    {
        sprintf (telPtr, "%s", array->elem[i].tel_name);
        telPtr += numOfElemTelName;
    }
    
    staPtr = staName;
    for (i = 0; i < numOfRows; i++)
    {
        sprintf (staPtr, "%s", array->elem[i].sta_name);
        staPtr += numOfElemStaName;
    }

    k = 0;
    for (i = 0; i < numOfRows; i++)
    {
        staIndex[i] = array->elem[i].sta_index;
        diameter[i] = array->elem[i].diameter;
        for (j = 0; j < numOfElemStaxyz; j++)
        {
            staxyz[k++] = array->elem[i].staxyz[j];
        }
    }

    //    Point to the correct elements
    outArrayPtr[0] = (void *) telName;
    outArrayPtr[1] = (void *) staName;
    outArrayPtr[2] = (void *) staIndex;
    outArrayPtr[3] = (void *) diameter;
    outArrayPtr[4] = (void *) staxyz;

    arrayStatus = qfits_table_append_xtension_hdr (outFitsPtr, oiArrayTable, outArrayPtr, oiArrayHeader);
    if (arrayStatus == - 1)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot write into OI_ARRAY table \n", routine);
        remove (outFitsName);
    }
    
    //    Close files and release memory
    qfits_header_destroy (oiArrayHeader);
    qfits_table_close (oiArrayTable);
    free (tempString);
    free (outArrayPtr);
    fclose (outFitsPtr);
    free (telName);
    free (staName);
    free (staIndex);
    free (diameter);
    free (staxyz);
        
    return;
}
/*****************************************************************************/

/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    writeOiTarget
* Input/Output:    See function arguments to avoid duplication
* Description:    Write OI_TARGET fits binary table
*
*
* History:        
* 13-May-04        (csabet) Converted to qfits
* 10-Jul-03        (csabet) Created. Derived from John Young <jsy1001@cam.ac.uk>
******************************************************************************/
void writeOiTarget (
    char        *outFitsName,     // In: Name of output FITS file
    OiTarget    *targets,    // In:    Pointer to OiTarget
    int        *status)     // IO:    Error status
{

    //    Local Declaration
    //    -----------------
    const char    routine[] = "writeOiTarget";
    FILE        *outFitsPtr;
    qfits_table    *oiTargetTable;
    int        tableWidth, numOfColumns, numOfRows, numOfElements, numOfElemTargetName,
            numOfElemVeltyp, numOfElemVeldef, numOfElemSpectyp, arrayStatus;
    qfits_col    *colPtr;
    qfits_header    *oiTargetHeader;
    const void    **outTargetPtr;
    char        *tempString;
    
    //    Algorithm
    //    ---------

    //    Reset status and set table parameters
    *status = 0;
    arrayStatus = 0;
    numOfRows = targets->ntarget;
    numOfColumns = 17;
    tableWidth = -1;
    numOfElements = 1;
    numOfElemTargetName = 16;
    numOfElemVeltyp = 8;
    numOfElemVeldef    = 8;
    numOfElemSpectyp = 16;
    
    //    Allocate memory
    tempString = (char *) calloc ( MAX_STRING_LENGTH, sizeof (char));
    
    //    Open the output FITS file for writing
    outFitsPtr = fopen (outFitsName, "a");
    if (outFitsPtr == NULL)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot open output FITS file for writing %s\n", routine, outFitsName);
        free (tempString);
        return;
    }

    //    Create a new table
    oiTargetTable = qfits_table_new (outFitsName, QFITS_BINTABLE, tableWidth, numOfColumns, numOfRows);
    if (!oiTargetTable)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot create OI_TARGET table \n", routine);
        free (tempString);
        fclose (outFitsPtr);
        remove (outFitsName);
        return;
    }

    //    Fill in the column information
    colPtr = oiTargetTable->col;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(short), TFITS_BIN_TYPE_I, "TARGET_ID", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemTargetName, 0, sizeof(char), TFITS_BIN_TYPE_A, "TARGET", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "RAEP0", "deg", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "DECEP0", "deg", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(float), TFITS_BIN_TYPE_E, "EQUINOX", "year", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "RA_ERR", "deg", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "DEC_ERR", "deg", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "SYSVEL", "m/s", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVeltyp, 0, sizeof(char), TFITS_BIN_TYPE_A, "VELTYP", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVeldef, 0, sizeof(char), TFITS_BIN_TYPE_A, "VELDEF", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "PMRA", "deg/year", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "PMDEC", "deg/year", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "PMRA_ERR", "deg/year", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "PMDEC_ERR", "deg/year", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(float), TFITS_BIN_TYPE_E, "PARALLAX", "deg", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(float), TFITS_BIN_TYPE_E, "PARA_ERR", "deg", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemSpectyp, 0, sizeof(char), TFITS_BIN_TYPE_A, "SPECTYP", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);

    //    Create OI_RARGET header
    oiTargetHeader = qfits_table_ext_header_default (oiTargetTable);
    qfits_header_add (oiTargetHeader, "EXTNAME", "OI_TARGET", "Name of this binary table extension", "");      
    qfits_header_add (oiTargetHeader, "OI_REVN", targets->revision, "Revision number of the table definition", "");

    //    Diagnostics
    if (diagnostic > 2) qfits_header_dump (oiTargetHeader, midiReportPtr);

    //    Allocate space for the array of data
    outTargetPtr = (const void **) calloc (numOfColumns, sizeof(const void *));

    //    Point to the correct elements
    outTargetPtr[0] = (void *) &(targets->targ->target_id);
    outTargetPtr[1] = (void *) targets->targ->target;
    outTargetPtr[2] = (void *) &(targets->targ->raep0);
    outTargetPtr[3] = (void *) &(targets->targ->decep0);
    outTargetPtr[4] = (void *) &(targets->targ->equinox);
    outTargetPtr[5] = (void *) &(targets->targ->ra_err);
    outTargetPtr[6] = (void *) &(targets->targ->dec_err);
    outTargetPtr[7] = (void *) &(targets->targ->sysvel);
    outTargetPtr[8] = (void *) targets->targ->veltyp;
    outTargetPtr[9] = (void *) targets->targ->veldef;
    outTargetPtr[10] = (void *) &(targets->targ->pmra);
    outTargetPtr[11] = (void *) &(targets->targ->pmdec);
    outTargetPtr[12] = (void *) &(targets->targ->pmra_err);
    outTargetPtr[13] = (void *) &(targets->targ->pmdec_err);
    outTargetPtr[14] = (void *) &(targets->targ->parallax);
    outTargetPtr[15] = (void *) &(targets->targ->para_err);
    outTargetPtr[16] = (void *) targets->targ->spectyp;

    arrayStatus = qfits_table_append_xtension_hdr (outFitsPtr, oiTargetTable, outTargetPtr, oiTargetHeader);
    if (arrayStatus == - 1)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot write into OI_TARGET table \n", routine);
        remove (outFitsName);
    }
    
    //    Close files and release memory
    qfits_header_destroy (oiTargetHeader);
    qfits_table_close (oiTargetTable);
    free (tempString);
    free (outTargetPtr);
    fclose (outFitsPtr);
        
    return;
}
/*****************************************************************************/

/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    writeOiWavelength
* Input/Output:    See function arguments to avoid duplication
* Description:    Write OI_WAVELENGTH fits binary table
*
*
* History:        
* 13-May-04        (csabet) Converted to qfits
* 10-Jul-03        (csabet) Created. Derived from John Young <jsy1001@cam.ac.uk>
******************************************************************************/
void writeOiWavelength (
    char            *outFitsName,     // In: Name of output FITS file
    OiWavelength    *wave,             // In:    Pointer to OiWavelength
    int                *status)         // IO:    Status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "writeOiWavelength";
    FILE            *outFitsPtr;
    qfits_table        *oiWaveTable;
    int                i, tableWidth, numOfColumns, numOfRows, numOfElements, arrayStatus;
    qfits_col        *colPtr;
    qfits_header    *oiWaveHeader;
    const void            **outWavePtr;
    char            *tempString;
    float            *effWave, *effBand;
    
    //    Algorithm
    //    ---------

    //    Reset status and set table parameters
    *status = 0;
    arrayStatus = 0;
    numOfRows = wave->nwave;
    numOfColumns = 2;
    tableWidth = -1;
    numOfElements = 1;
    
    //    Allocate memory
    tempString = (char *) calloc ( MAX_STRING_LENGTH, sizeof (char));
    
    //    Open the output FITS file for writing
    outFitsPtr = fopen (outFitsName, "a");
    if (outFitsPtr == NULL)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot open output FITS file for writing %s\n", routine, outFitsName);
        free (tempString);
        return;
    }

    //    Create a new table
    oiWaveTable = qfits_table_new (outFitsName, QFITS_BINTABLE, tableWidth, numOfColumns, numOfRows);
    if (!oiWaveTable)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot create OI_WAVELENGTH table \n", routine);
        free (tempString);
        fclose (outFitsPtr);
        remove (outFitsName);
        return;
    }

    //    Fill in the column information
    colPtr = oiWaveTable->col;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(float), TFITS_BIN_TYPE_E, "EFF_WAVE", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(float), TFITS_BIN_TYPE_E, "EFF_BAND", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);

    //    Create OI_WAVELENGTH header
    oiWaveHeader = qfits_table_ext_header_default (oiWaveTable);
    qfits_header_add (oiWaveHeader, "EXTNAME", "OI_WAVELENGTH", "Name of this binary table extension", "");      
    qfits_header_add (oiWaveHeader, "OI_REVN", wave->revision, "Revision number of the table definition", "");
    qfits_header_add (oiWaveHeader, "INSNAME", wave->insname, "Detector name", "");

    //    Diagnostics
    if (diagnostic > 2) qfits_header_dump (oiWaveHeader, midiReportPtr);

    //    Allocate space for the array of data
    outWavePtr = (const void **) calloc (numOfColumns, sizeof(const void *));
    effWave = (float *) calloc (numOfRows * numOfElements, sizeof(float));
    effBand = (float *) calloc (numOfRows * numOfElements, sizeof(float));
    
    //    Load data
    for (i = 0; i < numOfRows; i++)
    {
        effWave[i] = wave->eff_wave[i];
        effBand[i] = wave->eff_band[i];
    }
    
    //    Point to the correct elements
    outWavePtr[0] = (void *) effWave;
    outWavePtr[1] = (void *) effBand;

    arrayStatus = qfits_table_append_xtension_hdr (outFitsPtr, oiWaveTable, outWavePtr, oiWaveHeader);
    if (arrayStatus == - 1)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot write into OI_TARGET table \n", routine);
        remove (outFitsName);
    }
    
    //    Close files and release memory
    qfits_header_destroy (oiWaveHeader);
    qfits_table_close (oiWaveTable);
    free (tempString);
    free (effWave);
    free (effBand);
    free (outWavePtr);
    fclose (outFitsPtr);
        
    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    writeOiVis
* Input/Output:    See function arguments to avoid duplication
* Description:    Write OI_VIS fits binary table
*
*
* History:        
* 14-May-04        (csabet) Converted to qfits
* 10-Jul-03        (csabet) Created. Derived from John Young <jsy1001@cam.ac.uk>
******************************************************************************/
void writeOiVis (                // Ou: On error, returns non-zero cfitsio error code, and sets status
    char        *outFitsName,     // In: Name of output FITS file
    OiVis         *vis,             // In: Pointer to OiVis structure
    int         *status)         // IO: Status
{

    //    Local Declarations
    //    ------------------
    const char     routine[] = "writeOiVis";
    FILE            *outFitsPtr;
    qfits_table        *oiVisTable;
    int                tableWidth, numOfColumns, numOfRows, numOfElements, numOfElemStaIndex, numOfElemVis;
    qfits_col        *colPtr;
    qfits_header    *oiVisHeader;
    const void            **outVisPtr;
    char            *tempString, *flag, *stringPtr;
    int                i, j, k, l, m, *targetId, arrayStatus;
    double            *time, *mjd, *integTime, *visAmp, *visAmapErr, *visPhi, *visPhiErr,
                    *uCoord, *vCoord; 
    short *staIndex;

    //    Algorithm
    //    ---------

    //    Reset status and set table parameters
    *status = 0;
    arrayStatus = 0;
    numOfRows = vis->numrec;
    numOfColumns = 12;
    tableWidth = -1;
    numOfElements = 1;
    numOfElemStaIndex = 2;
    numOfElemVis = vis->nwave;
    
    //    Allocate memory
    tempString = (char *) calloc ( MAX_STRING_LENGTH, sizeof (char));
    
    //    Open the output FITS file for writing
    outFitsPtr = fopen (outFitsName, "a");
    if (outFitsPtr == NULL)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot open output FITS file for writing %s\n", routine, outFitsName);
        free (tempString);
        return;
    }

    //    Create a new table
    oiVisTable = qfits_table_new (outFitsName, QFITS_BINTABLE, tableWidth, numOfColumns, numOfRows);
    if (!oiVisTable)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot create OI_VIS table \n", routine);
        free (tempString);
        fclose (outFitsPtr);
        remove (outFitsName);
        return;
    }

    //    Fill in the column information
    colPtr = oiVisTable->col;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(short), TFITS_BIN_TYPE_I, "TARGET_ID", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "TIME", "s", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "MJD", "day", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "INT_TIME", "s", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVis, 0, sizeof(double), TFITS_BIN_TYPE_D, "VISAMP", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVis, 0, sizeof(double), TFITS_BIN_TYPE_D, "VISAMPERR", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVis, 0, sizeof(double), TFITS_BIN_TYPE_D, "VISPHI", "deg", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVis, 0, sizeof(double), TFITS_BIN_TYPE_D, "VISPHIERR", "deg", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "UCOORD", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "VCOORD", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemStaIndex, 0, sizeof(short), TFITS_BIN_TYPE_I, "STA_INDEX", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVis, 0, sizeof(char), TFITS_BIN_TYPE_L, "FLAG", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;

    //    Create OI_VIS header
    oiVisHeader = qfits_table_ext_header_default (oiVisTable);
    qfits_header_add (oiVisHeader, "EXTNAME", "OI_VIS", "Name of this binary table extension", "");
    qfits_header_add (oiVisHeader, "OI_REVN", vis->revision, "Revision number of the table definition", "");
    qfits_header_add (oiVisHeader, "DATE-OBS", vis->date_obs, "UTC start date of observations", "");
    if (strlen(vis->arrname) > 0)
        qfits_header_add (oiVisHeader, "ARRNAME", vis->arrname, "Array name", "");
  
    qfits_header_add (oiVisHeader, "INSNAME", vis->insname, "Detector name", "");

    //    Diagnostics
    if (diagnostic > 2) qfits_header_dump (oiVisHeader, midiReportPtr);

    //    Allocate space for the array of data
    outVisPtr = (const void **) calloc (numOfColumns, sizeof(const void *));
    targetId = (int *) calloc (numOfRows * numOfElements, sizeof (int));
    time = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    mjd = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    integTime = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    visAmp = (double *) calloc (numOfRows * numOfElemVis, sizeof (double));
    visAmapErr = (double *) calloc (numOfRows * numOfElemVis, sizeof (double));
    visPhi = (double *) calloc (numOfRows * numOfElemVis, sizeof (double));
    visPhiErr = (double *) calloc (numOfRows * numOfElemVis, sizeof (double));
    uCoord = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    vCoord = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    staIndex = (short *) calloc (numOfRows * numOfElemStaIndex, sizeof (short));
    flag = (char *) calloc (numOfRows * numOfElemVis+1, sizeof (char));

    //    Load the data
    k = 0;
    l = 0;
    m = 0;
    stringPtr = flag;
    for (i = 0; i < numOfRows; i++)
    {
        for (j = 0; j < numOfElements; j++)
        {
            targetId[k] = vis->record[j].target_id;
            time[k]  = vis->record[j].time;
            mjd[k]  = vis->record[j].mjd;
            integTime[k]  = vis->record[j].int_time;
            uCoord[k] = vis->record[j].ucoord;
            vCoord[k++] = vis->record[j].vcoord;
        }



        for (j = 0; j < numOfElemVis; j++)
        {
            visAmp[l] = vis->record[i].visamp[j];
            visAmapErr[l] = vis->record[i].visamperr[j];
            visPhi[l] = vis->record[i].visphi[j];
            visPhiErr[l++] = vis->record[i].visphierr[j];
            
            if (strcmp (vis->record[i].flag, "FALSE") == 0)
               sprintf (stringPtr, "%s", "F");
            else
               sprintf (stringPtr, "%s", "T");
            stringPtr += numOfElements;
            
        }
/*         stringPtr += numOfElements; */

        for (j = 0; j < numOfElemStaIndex; j++)
        {
            staIndex[m++] = vis->record[i].sta_index[j];
        }
    }
    
    //    Point to the correct elements
    outVisPtr[0] = (void *) targetId;
    outVisPtr[1] = (void *) time;
    outVisPtr[2] = (void *) mjd;
    outVisPtr[3] = (void *) integTime;
    outVisPtr[4] = (void *) visAmp;
    outVisPtr[5] = (void *) visAmapErr;
    outVisPtr[6] = (void *) visPhi;
    outVisPtr[7] = (void *) visPhiErr;
    outVisPtr[8] = (void *) uCoord;
    outVisPtr[9] = (void *) vCoord;
    outVisPtr[10] = (void *) staIndex;
    outVisPtr[11] = (void *) flag;

    arrayStatus = qfits_table_append_xtension_hdr (outFitsPtr, oiVisTable, outVisPtr, oiVisHeader);
    if (arrayStatus == - 1)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot write into OI_VIS table \n", routine);
        remove (outFitsName);
    }
    
    //    Close files and release memory
    qfits_header_destroy (oiVisHeader);
    qfits_table_close (oiVisTable);
    free (tempString);
    free (outVisPtr);
    fclose (outFitsPtr);
    free (targetId);
    free (time);
    free (mjd);
    free (integTime);
    free (visAmp);
    free (visAmapErr);
    free (visPhi);
    free (visPhiErr);
    free (uCoord);
    free (vCoord);
    free (staIndex);
    free (flag);
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    writeOiVis2
* Input/Output:    See function arguments to avoid duplication
* Description:    Write OI_VIS2 fits binary table
*
*
* History:        
* 14-May-04        (csabet) Converted to qfits
* 10-Jul-03        (csabet) Created. Derived from John Young <jsy1001@cam.ac.uk>
******************************************************************************/
void writeOiVis2 (            // Ou: On error, returns non-zero cfitsio error code, and sets status
    char    *outFitsName,     // In: Name of output FITS file
    OiVis2     *vis2,             // In: Pointer to OiVis2 structure
    int     *status)          // IO: Status
{


    //    Local Declarations
    //    ------------------
    const char     routine[] = "writeOiVis2";
    FILE            *outFitsPtr;
    qfits_table        *oiVis2Table;
    int                tableWidth, numOfColumns, numOfRows, numOfElements, numOfElemStaIndex, numOfElemVis2;
    qfits_col        *colPtr;
    qfits_header    *oiVis2Header;
    const void            **outVis2Ptr;
    char            *tempString, *flag, *stringPtr;
    int                i, j, k, l, m, *targetId, arrayStatus;
    double            *time, *mjd, *integTime, *vis2Data, *vis2Err, *uCoord, *vCoord; 
    short *staIndex;

    //    Algorithm
    //    ---------

    //    Reset status and set table parameters
    *status = 0;
    arrayStatus = 0;
    numOfRows = vis2->numrec;
    numOfColumns = 10;
    tableWidth = -1;
    numOfElements = 1;
    numOfElemStaIndex = 2;
    numOfElemVis2 = vis2->nwave;
    
    //    Allocate memory
    tempString = (char *) calloc ( MAX_STRING_LENGTH, sizeof (char));
    
    //    Open the output FITS file for writing
    outFitsPtr = fopen (outFitsName, "a");
    if (outFitsPtr == NULL)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot open output FITS file for writing %s\n", routine, outFitsName);
        free (tempString);
        return;
    }

    //    Create a new table
    oiVis2Table = qfits_table_new (outFitsName, QFITS_BINTABLE, tableWidth, numOfColumns, numOfRows);
    if (!oiVis2Table)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot create OI_VIS2 table \n", routine);
        free (tempString);
        fclose (outFitsPtr);
        remove (outFitsName);
        return;
    }

    //    Fill in the column information
    colPtr = oiVis2Table->col;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(short), TFITS_BIN_TYPE_I, "TARGET_ID", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "TIME", "s", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "MJD", "day", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "INT_TIME", "s", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVis2, 0, sizeof(double), TFITS_BIN_TYPE_D, "VIS2DATA", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVis2, 0, sizeof(double), TFITS_BIN_TYPE_D, "VIS2ERR", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "UCOORD", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "VCOORD", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemStaIndex, 0, sizeof(short), TFITS_BIN_TYPE_I, "STA_INDEX", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemVis2, 0, sizeof(char), TFITS_BIN_TYPE_L, "FLAG", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;

    //    Create OI_VIS header
    oiVis2Header = qfits_table_ext_header_default (oiVis2Table);
    qfits_header_add (oiVis2Header, "EXTNAME", "OI_VIS2", "Name of this binary table extension", "");
    qfits_header_add (oiVis2Header, "OI_REVN", vis2->revision, "Revision number of the table definition", "");
    qfits_header_add (oiVis2Header, "DATE-OBS", vis2->date_obs, "UTC start date of observations", "");
    if (strlen(vis2->arrname) > 0)
        qfits_header_add (oiVis2Header, "ARRNAME", vis2->arrname, "Array name", "");
  
    qfits_header_add (oiVis2Header, "INSNAME", vis2->insname, "Detector name", "");

    //    Diagnostics. Comment out later
    if (diagnostic > 2) qfits_header_dump (oiVis2Header, midiReportPtr);

    //    Allocate space for the array of data
    outVis2Ptr = (const void **) calloc (numOfColumns, sizeof(const void *));
    targetId = (int *) calloc (numOfRows * numOfElements, sizeof (int));
    time = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    mjd = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    integTime = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    vis2Data = (double *) calloc (numOfRows * numOfElemVis2, sizeof (double));
    vis2Err = (double *) calloc (numOfRows * numOfElemVis2, sizeof (double));
    uCoord = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    vCoord = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    staIndex = (short *) calloc (numOfRows * numOfElemStaIndex, sizeof (short));
    flag = (char *) calloc (numOfRows *  numOfElemVis2+1, sizeof (char));

    //    Load the data
    k = 0;
    l = 0;
    m = 0;
    stringPtr = flag;
    for (i = 0; i < numOfRows; i++)
    {
        for (j = 0; j < numOfElements; j++)
        {
            targetId[k] = vis2->record[j].target_id;
            time[k]  = vis2->record[j].time;
            mjd[k]  = vis2->record[j].mjd;
            integTime[k]  = vis2->record[j].int_time;
            uCoord[k] = vis2->record[j].ucoord;
            vCoord[k++] = vis2->record[j].vcoord;
        }
            

        for (j = 0; j < numOfElemVis2; j++)
        {
            vis2Data[l] = vis2->record[i].vis2data[j];
            vis2Err[l++] = vis2->record[i].vis2err[j];

        if (strcmp (vis2->record[i].flag, "FALSE") == 0)
            sprintf (stringPtr, "%s", "F");
        else
            sprintf (stringPtr, "%s", "T");
        stringPtr += numOfElements;

        }

        for (j = 0; j < numOfElemStaIndex; j++)
        {
            staIndex[m++] = vis2->record[i].sta_index[j];
        }
    }
    
    //    Point to the correct elements
    outVis2Ptr[0] = (void *) targetId;
    outVis2Ptr[1] = (void *) time;
    outVis2Ptr[2] = (void *) mjd;
    outVis2Ptr[3] = (void *) integTime;
    outVis2Ptr[4] = (void *) vis2Data;
    outVis2Ptr[5] = (void *) vis2Err;
    outVis2Ptr[6] = (void *) uCoord;
    outVis2Ptr[7] = (void *) vCoord;
    outVis2Ptr[8] = (void *) staIndex;
    outVis2Ptr[9] = (void *) flag;

    arrayStatus = qfits_table_append_xtension_hdr (outFitsPtr, oiVis2Table, outVis2Ptr, oiVis2Header);
    if (arrayStatus == - 1)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot write into OI_VIS2 table \n", routine);
        remove (outFitsName);
    }
    
    //    Close files and release memory
    qfits_header_destroy (oiVis2Header);
    qfits_table_close (oiVis2Table);
    free (tempString);
    free (outVis2Ptr);
    fclose (outFitsPtr);
    free (targetId);
    free (time);
    free (mjd);
    free (integTime);
    free (vis2Data);
    free (vis2Err);
    free (uCoord);
    free (vCoord);
    free (staIndex);
    free (flag);
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    writeOiT3
* Input/Output:    See function arguments to avoid duplication
* Description:    Write OI_T3 fits binary table
*
*                Warning: This routine is not tested as it is not used in the MIDI
* History:        
* 14-May-04        (csabet) Converted to qfits
* 10-Jul-03        (csabet) Created. Derived from John Young <jsy1001@cam.ac.uk>
******************************************************************************/
void writeOiT3 (            // Ou: On error, returns non-zero cfitsio error code, and sets status
    char    *outFitsName,     // In: Name of output FITS file
    OiT3     *t3,             // In: Pointer to OiT3 structure
    int     *status)          // IO: Status
{

    //    Local Declarations
    //    ------------------
    const char     routine[] = "writeOiT3";
    FILE            *outFitsPtr;
    qfits_table        *oiT3Table;
    int                tableWidth, numOfColumns, numOfRows, numOfElements, numOfElemStaIndex, numOfElemT3;
    qfits_col        *colPtr;
    qfits_header    *oiT3Header;
    const void            **outT3Ptr;
    char            *tempString, *flag, *stringPtr;
    int                i, j, k, l, m, *targetId, *staIndex;
    double            *time, *mjd, *integTime, *t3Amp, *t3AmpErr, *t3Phi, *t3PhiErr,
                    *u1Coord, *v1Coord, *u2Coord, *v2Coord;

    //    Algorithm
    //    ---------

    //    Reset status and set table parameters
    *status = 0;
    numOfRows = t3->numrec;
    numOfColumns = 14;
    tableWidth = -1;
    numOfElements = 1;
    numOfElemStaIndex = 1;
    numOfElemT3 = t3->nwave;
    
    //    Allocate memory
    tempString = (char *) calloc ( MAX_STRING_LENGTH, sizeof (char));
    
    //    Open the output FITS file for writing
    outFitsPtr = fopen (outFitsName, "a");
    if (outFitsPtr == NULL)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot open output FITS file for writing %s\n", routine, outFitsName);
        free (tempString);
        return;
    }

    //    Create a new table
    oiT3Table = qfits_table_new (outFitsName, QFITS_BINTABLE, tableWidth, numOfColumns, numOfRows);
    if (!oiT3Table)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot create OI_T3 table \n", routine);
        free (tempString);
        fclose (outFitsPtr);
        remove (outFitsName);
        return;
    }

    //    Fill in the column information
    colPtr = oiT3Table->col;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(short), TFITS_BIN_TYPE_I, "TARGET_ID", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "TIME", "s", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "MJD", "day", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "INT_TIME", "s", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemT3, 0, sizeof(double), TFITS_BIN_TYPE_D, "T3AMP", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemT3, 0, sizeof(double), TFITS_BIN_TYPE_D, "T3AMPERR", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemT3, 0, sizeof(double), TFITS_BIN_TYPE_D, "T3PHI", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemT3, 0, sizeof(double), TFITS_BIN_TYPE_D, "T3PHIERR", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "U1COORD", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "V1COORD", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "U2COORD", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(double), TFITS_BIN_TYPE_D, "V2COORD", "m", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElemStaIndex, 0, sizeof(short), TFITS_BIN_TYPE_I, "STA_INDEX", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;
    qfits_col_fill (colPtr, numOfElements, 0, sizeof(char), TFITS_BIN_TYPE_A, "FLAG", " ", " ", " ", 
        0, 0.0, 0, 0.0, 0);
    colPtr++;

    //    Create OI_VIS header
    oiT3Header = qfits_table_ext_header_default (oiT3Table);
    qfits_header_add (oiT3Header, "EXTNAME", "OI_T3", "Name of this binary table extension", "");
    qfits_header_add (oiT3Header, "OI_REVN", t3->revision, "Revision number of the table definition", "");
    qfits_header_add (oiT3Header, "DATE-OBS", t3->date_obs, "UTC start date of observations", "");
    if (strlen(t3->arrname) > 0)
        qfits_header_add (oiT3Header, "ARRNAME", t3->arrname, "Array name", "");
  
    qfits_header_add (oiT3Header, "INSNAME", t3->insname, "Detector name", "");

    //    Diagnostics. Comment out later
    if (diagnostic > 2) qfits_header_dump (oiT3Header, midiReportPtr);

    //    Allocate space for the array of data
    outT3Ptr = (const void **) calloc (numOfColumns, sizeof(const void *));
    targetId = (int *) calloc (numOfRows * numOfElements, sizeof (int));
    time = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    mjd = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    integTime = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    t3Amp = (double *) calloc (numOfRows * numOfElemT3, sizeof (double));
    t3AmpErr = (double *) calloc (numOfRows * numOfElemT3, sizeof (double));
    t3Phi = (double *) calloc (numOfRows * numOfElemT3, sizeof (double));
    t3PhiErr = (double *) calloc (numOfRows * numOfElemT3, sizeof (double));
    u1Coord = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    v1Coord = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    u2Coord = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    v2Coord = (double *) calloc (numOfRows * numOfElements, sizeof (double));
    staIndex = (int *) calloc (numOfRows * numOfElemStaIndex, sizeof (int));
    flag = (char *) calloc (numOfRows *  MAX_STRING_LENGTH, sizeof (char));

    //    Load the data
    k = 0;
    l = 0;
    m = 0;
    stringPtr = flag;
    for (i = 0; i < numOfRows; i++)
    {
        for (j = 0; j < numOfElements; j++)
        {
            targetId[k] = t3->record[j].target_id;
            time[k]  = t3->record[j].time;
            mjd[k]  = t3->record[j].mjd;
            integTime[k]  = t3->record[j].int_time;
            u1Coord[k] = t3->record[j].u1coord;
            v1Coord[k++] = t3->record[j].v1coord;
            u2Coord[k] = t3->record[j].u2coord;
            v2Coord[k++] = t3->record[j].v2coord;
        }
            
       cpl_msg_info(cpl_func,"flag = %s\n", t3->record[i].flag);
        if (strcmp (t3->record[i].flag, "FALSE") == 0)
            sprintf (stringPtr, "%s", "F");
        else
            sprintf (stringPtr, "%s", "T");
        stringPtr += numOfElements;

        for (j = 0; j < numOfElemT3; j++)
        {
            t3Amp[l] = t3->record[i].t3amp[j];
            t3AmpErr[l] = t3->record[i].t3amperr[j];
            t3Phi[l] = t3->record[i].t3phi[j];
            t3PhiErr[l] = t3->record[i].t3phierr[j];
        }

        for (j = 0; j < numOfElemStaIndex; j++)
        {
            staIndex[m++] = t3->record[i].sta_index[j];
        }
    }
    
    //    Point to the correct elements
    outT3Ptr[0] = (void *) targetId;
    outT3Ptr[1] = (void *) time;
    outT3Ptr[2] = (void *) mjd;
    outT3Ptr[3] = (void *) integTime;
    outT3Ptr[4] = (void *) t3Amp;
    outT3Ptr[5] = (void *) t3AmpErr;
    outT3Ptr[6] = (void *) t3Phi;
    outT3Ptr[7] = (void *) t3PhiErr;
    outT3Ptr[8] = (void *) u1Coord;
    outT3Ptr[9] = (void *) v1Coord;
    outT3Ptr[10] = (void *) u2Coord;
    outT3Ptr[11] = (void *) v2Coord;
    outT3Ptr[12] = (void *) staIndex;
    outT3Ptr[13] = (void *) flag;

    if (qfits_table_append_xtension_hdr (outFitsPtr, oiT3Table, outT3Ptr, oiT3Header) == -1)
    {
        *status = 1;
       cpl_msg_info(cpl_func,"<<Warning>> from '%s'. Cannot write into OI_T3 table \n", routine);
        remove (outFitsName);
    }
    
    //    Close files and release memory
    qfits_header_destroy (oiT3Header);
    qfits_table_close (oiT3Table);
    free (tempString);
    free (outT3Ptr);
    fclose (outFitsPtr);
    free (targetId);
    free (time);
    free (mjd);
    free (integTime);
    free (t3Amp);
    free (t3AmpErr);
    free (t3Phi);
    free (t3PhiErr);
    free (u1Coord);
    free (v1Coord);
    free (u2Coord);
    free (v2Coord);
    free (staIndex);
    free (flag);

    return;
}
/*****************************************************************************/
