/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiFitsUtility.c
* Description:  Contains general routines for FITS handling
*
* History:
* 25-Apr-03     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/stat.h>
#include <stdio.h>
#include <cpl.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "midiFitsUtility.h"
#include "diagnostics.h"
#include "memoryHandling.h"
#include "errorHandling.h"
#include "fileHandling.h"
#include "qfits.h"
#include "qfits_rw.h"

/**********************************************************
*   Global Variables
**********************************************************/

/**********************************************************
*   Constant definitions
**********************************************************/

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software 
*
* Module name:  getImageFormat
* Input/Output: See function arguments to avoid duplication
* Description:  This routine obtains all the image formats for later processing
*
* History:      
* 14-Aug-03     (csabet) Created
******************************************************************************/
void getImageFormat (
    char        *inFitsFile,                // In: Name of the input FITS file
    int         extNumOfImagingDataFile,    // In: Extension number of the IMAGING_DATA in input file
    ImageFormat *format,                     // Ou: Pointer to format structure
    int            *error)                        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "getImageFormat";
    char        *sTemp, *cleanString, *stringQfits, *emptyString;
    int            numOfExtensions;
    struct stat buf;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    format->numOfPinholes = 0;
    
    //    Check if the file exist
    if (stat (inFitsFile, &buf) == -1)
    {
        
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__,     "Cannot find requested FITS file");
        *error = 1;
        return;
    }
    else if ((qfits_is_fits (inFitsFile)) != 1)  // Check if this is a FITS file
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Non-FITS file");
        *error = 1;
        return;
    }

    //    Check if the file has data tables
    numOfExtensions = qfits_query_n_ext (inFitsFile);
    if (numOfExtensions < 1)
    {
        format->hasData = 0;
        return;
    }
    else format->hasData = 1;

    //    Allocate memory
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    emptyString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    strcpy (emptyString, "UNKNOWN");

    //    Get data Observation Category
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO DPR CATG");
    if (stringQfits == NULL)
    {
        sprintf (format->obsCatg, "%s", emptyString);
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'DPR CATG'");
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->obsCatg, "%s", cleanString);
    }

    //    Get data Observation Technique
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO DPR TECH");
    if (stringQfits == NULL)
    {
        sprintf (format->obsTech, "%s", emptyString);
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'DPR TECH'");
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->obsTech, "%s", cleanString);
    }

    //    Get data Observation Type
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO DPR TYPE");
    if (stringQfits == NULL)
    {
        sprintf (format->obsType, "%s", emptyString);
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'DPR TYPE'");
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->obsType, "%s", cleanString);
    }

    //    Get Shutter ID
    stringQfits = qfits_query_ext (inFitsFile, "HIERARCH ESO INS SHUT ID", 0);
    if (stringQfits == NULL) 
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'SHUT ID'");
        sprintf (format->shutterId, "%s", emptyString);    
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->shutterId, "%s", cleanString);
    }

    //    Get beam combiner
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO INS OPT1 ID");
    if (stringQfits == NULL) 
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'OPT1 ID'");
        sprintf (format->beamCombiner, "%s", emptyString);
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->beamCombiner, "%s", cleanString);
    }

    //    Determine if data is dispesed using the beam combiner
    if (strcmp (format->beamCombiner, "HIGH_SENS") == 0)
        format->isDisp = 1;
    else if (strcmp (cleanString, "SCI_PHOT") == 0)
        format->isDisp = 1;
    else if (strcmp (cleanString, "OPEN") == 0)
        format->isDisp = 0;
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Beam Combiner 'OPT1 ID'");
        sprintf (format->beamCombiner, "%s", emptyString);
    }

    //    Get target name
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO OBS TARG NAME");
    if (stringQfits == NULL)
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'TARG NAME'");
        sprintf (format->targetName, "%s", emptyString);
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->targetName, "%s", cleanString);
    }

    //    Get Grism ID
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO INS GRIS NAME");
    if (stringQfits == NULL)
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'GRIS NAME'");
        sprintf (format->grismId, "%s", emptyString);
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->grismId, "%s", cleanString);
    }

    //    Get Camera Name
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO INS CAM ID");
    if (stringQfits == NULL) 
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Camera ID");
        sprintf (format->cameraId, "%s", emptyString);
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->cameraId, "%s", cleanString);
    }

    //    Get Filter ID
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO INS FILT NAME");
    if (stringQfits == NULL) 
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Filter Name");
        sprintf (format->filterName, "%s", emptyString);
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->filterName, "%s", cleanString);
    }

    //    Get TPL Start
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO TPL START");
    if (stringQfits == NULL)
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'TPL START'");
        sprintf (format->tplStart, "%s", emptyString);
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->tplStart, "%s", cleanString);
    }

    //    Get TPL Name
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO TPL NAME");
    if (stringQfits == NULL)
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'TPL NAME'");
        sprintf (format->tplName, "%s", emptyString);
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->tplName, "%s", cleanString);
    }

    //    Get MIDI subwindow format from the IMAGING_DATA
    sTemp = qfits_query_ext (inFitsFile, "TDIM14", extNumOfImagingDataFile);
    if (sTemp == NULL) sTemp = qfits_query_ext (inFitsFile, "TDIM17", extNumOfImagingDataFile);
    if (sTemp == NULL) sTemp = qfits_query_ext (inFitsFile, "TDIM20", extNumOfImagingDataFile);
    if (sTemp == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get TDIM14, TDIM17 or TDIM20");
        *error = 1;
    }
    else sscanf (sTemp, "'(%d,%d) '", &(format->iXWidth), &(format->iYWidth));

    //    Get number of frames
    sTemp = qfits_query_ext (inFitsFile, "NAXIS2", extNumOfImagingDataFile);
    if ((sTemp == NULL) || (!sscanf (sTemp, "%d", &(format->numOfFrames))))
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get NAXIS2");
        *error = 1;
    }
    
    //    Get frames per scan
    sTemp = qfits_query_ext (inFitsFile, "MAXSTEP", extNumOfImagingDataFile);
    if ((sTemp == NULL) || (!sscanf (sTemp, "%d", &(format->framesPerScan))))
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get MAXSTEP");
        *error = 1;
    }

    //    Compute sub-image size
    format->subWindowSize = format->iXWidth * format->iYWidth;


    //    Get number of pinholes. Get SLIT Name
    stringQfits = qfits_query_hdr (inFitsFile, "HIERARCH ESO INS SLIT NAME");
    if (stringQfits == NULL)
    {
        if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get 'SLIT NAME'");
        cleanUpString (stringQfits, emptyString);
    }
    else
    {
        cleanUpString (stringQfits, cleanString);
        sprintf (format->slitName, "%s", cleanString);
        if (strcmp (cleanString, "SLIT_0.05") == 0) format->numOfPinholes = 1;        // size = 0.05
        else if (strcmp (cleanString, "SLIT_0.1") == 0) format->numOfPinholes = 1;    // size = 0.1
        else if (strcmp (cleanString, "SLIT_0.2") == 0) format->numOfPinholes = 1;    // size = 0.2
        else if (strcmp (cleanString, "S_0.4C") == 0) format->numOfPinholes = 1;    // size = 0.4
        else if (strcmp (cleanString, "T_0.07C") == 0) format->numOfPinholes = 3;    // size = 0.07    centred
        else if (strcmp (cleanString, "T_0.11C") == 0) format->numOfPinholes = 3;    // size = 0.11    centred
        else if (strcmp (cleanString, "T_0.17C") == 0) format->numOfPinholes = 3;    // size = 0.17    centred
        else if (strcmp (cleanString, "OPEN") == 0) format->numOfPinholes = 0;
        else if (strcmp (cleanString, "FullField") == 0) format->numOfPinholes = 0;
        else
        {
            if (diagnostic) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown 'SLIT NAME'");
            cleanUpString (stringQfits, emptyString);
        }
    }
    
    //    Compute number of detector regions
    format->numOfRegionsToProcess = 2;
    if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
        format->numOfDetectorRegions = 4;
    else
        format->numOfDetectorRegions = 2;

    //    Log results
    if (diagnostic) 
    {
       cpl_msg_info(cpl_func,"FORMAT OF FILE %s \n", inFitsFile);
       cpl_msg_info(cpl_func,"-------------- \n");
       cpl_msg_info(cpl_func,"Batch Template Name          = %s \n", batchTemplate);
       cpl_msg_info(cpl_func,"FITS TPL NAME                = %s \n", format->tplName);
       cpl_msg_info(cpl_func,"Observation Category         = %s \n", format->obsCatg);
       cpl_msg_info(cpl_func,"Observation Technique        = %s \n", format->obsTech);
       cpl_msg_info(cpl_func,"Observation Type             = %s \n", format->obsType);
       cpl_msg_info(cpl_func,"Beam Combiner                = %s \n", format->beamCombiner);
       cpl_msg_info(cpl_func,"Filter Name                  = %s \n", format->filterName);
       cpl_msg_info(cpl_func,"Shutter ID                   = %s \n", format->shutterId);
       cpl_msg_info(cpl_func,"Grism ID                     = %s \n", format->grismId);
       cpl_msg_info(cpl_func,"Target Name                  = %s \n", format->targetName);
       cpl_msg_info(cpl_func,"Slit Name                    = %s \n", format->slitName);
       cpl_msg_info(cpl_func,"TPL Start                    = %s \n", format->tplStart);
       cpl_msg_info(cpl_func,"Number of Frames             = %d \n", format->numOfFrames);
        if (strcmp (format->shutterId, "ABOPEN") == 0)
           cpl_msg_info(cpl_func,"Frames per scan              = %d \n", format->framesPerScan);
       cpl_msg_info(cpl_func,"X, Column dimension          = %d \n", format->iXWidth);
       cpl_msg_info(cpl_func,"Y, Column dimension          = %d \n", format->iYWidth);
       cpl_msg_info(cpl_func,"Sub-window size              = %d \n", format->subWindowSize);
       cpl_msg_info(cpl_func,"Number of Pinholes           = %d \n", format->numOfPinholes);
       cpl_msg_info(cpl_func,"Number of Detector Regions   = %d \n", format->numOfDetectorRegions);
        if (strcmp(format->beamCombiner, "SCI_PHOT") == 0)
        {
           cpl_msg_info(cpl_func,"Number of Interf Regions     = %d \n", format->numOfRegionsToProcess);
           cpl_msg_info(cpl_func,"Number of Photom A Regions   = %d \n", format->numOfRegionsToProcess-1);
           cpl_msg_info(cpl_func,"Number of Photom B Regions   = %d \n", format->numOfRegionsToProcess-1);
        }
        else
        {
           cpl_msg_info(cpl_func,"Number of Regions to process = %d \n", format->numOfRegionsToProcess);
        }
       cpl_msg_info(cpl_func,"\n");
    }

    fprintf (midiReportPtr, "FORMAT OF FILE %s \n", inFitsFile);
    fprintf (midiReportPtr, "-------------- \n");
    fprintf (midiReportPtr, "Batch Template Name          = %s \n", batchTemplate);
    fprintf (midiReportPtr, "FITS TPL NAME                = %s \n", format->tplName);
    fprintf (midiReportPtr, "Observation Category         = %s \n", format->obsCatg);
    fprintf (midiReportPtr, "Observation Technique        = %s \n", format->obsTech);
    fprintf (midiReportPtr, "Observation Type             = %s \n", format->obsType);
    fprintf (midiReportPtr, "Beam Combiner                = %s \n", format->beamCombiner);
    fprintf (midiReportPtr, "Filter Name                  = %s \n", format->filterName);
    fprintf (midiReportPtr, "Shutter ID                   = %s \n", format->shutterId);
    fprintf (midiReportPtr, "Grism ID                     = %s \n", format->grismId);
    fprintf (midiReportPtr, "Target Name                  = %s \n", format->targetName);
    fprintf (midiReportPtr, "Slit Name                    = %s \n", format->slitName);
    fprintf (midiReportPtr, "TPL Start                    = %s \n", format->tplStart);
    fprintf (midiReportPtr, "Number of Frames             = %d \n", format->numOfFrames);
    if (strcmp (format->shutterId, "ABOPEN") == 0)
        fprintf (midiReportPtr, "Frames per scan              = %d \n", format->framesPerScan);
    fprintf (midiReportPtr, "X, Column dimension          = %d \n", format->iXWidth);
    fprintf (midiReportPtr, "Y, Column dimension          = %d \n", format->iYWidth);
    fprintf (midiReportPtr, "Sub-window size              = %d \n", format->subWindowSize);
    fprintf (midiReportPtr, "Number of Pinholes           = %d \n", format->numOfPinholes);
    fprintf (midiReportPtr, "Number of Detector Regions   = %d \n", format->numOfDetectorRegions);
    if (strcmp(format->beamCombiner, "SCI_PHOT") == 0)
    {
        fprintf (midiReportPtr, "Number of Interf Regions     = %d \n", format->numOfRegionsToProcess);
        fprintf (midiReportPtr, "Number of Photom A Regions   = %d \n", format->numOfRegionsToProcess-1);
        fprintf (midiReportPtr, "Number of Photom B Regions   = %d \n", format->numOfRegionsToProcess-1);
    }
    else
    {
        fprintf (midiReportPtr, "Number of Regions to process = %d \n", format->numOfRegionsToProcess);
    }
    fprintf (midiReportPtr, "\n");

    //    Check validity of the format
    if (format->iXWidth <= 0 || format->iYWidth <= 0 || format->numOfFrames <= 0 || format->framesPerScan <= 0) 
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Invalid file format");
        *error = 1;
    }

    /*    Release memory */
    free (cleanString);
    free (emptyString);
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getFilterData
* Input/Output: See function arguments to avoid duplication
* Description:
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void getFilterData (
    char        *fileName,    // In: Name of the input FITS file
    FilterData    *filterInfo,// IO: Pointer to the filter data structure
    int            *error)        // Ou: Error status
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "getFilterData";
    char        *tempStr, *cleanString;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //    Allocate memory
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    
    //    Set the default values
    filterInfo->optFreqLo = N_BAND_FILT_LONG;    // micron
    filterInfo->optFreqHi = N_BAND_FILT_SHORT;    // micron

    if (diagnostic)cpl_msg_info(cpl_func,"Extracting filter data from    %s \n", fileName);
    if (diagnostic) fprintf (midiReportPtr, "Extracting filter data from    %s \n", fileName);

    //    Get filter name
    tempStr = qfits_query_hdr (fileName, "HIERARCH ESO INS FILT NAME");
    if (tempStr == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Extracting filter name");
       cpl_msg_info(cpl_func,"Default Lower %f Higher and Lower %f micron are used \n", filterInfo->optFreqLo, filterInfo->optFreqHi);
        fprintf(midiReportPtr, "Default Higher %f micron and Lower %f micron are used \n", filterInfo->optFreqLo, filterInfo->optFreqHi);
        free (cleanString);
        return;
    }
    else
    {
        cleanUpString (tempStr, cleanString);
        sscanf (cleanString, "%s", filterInfo->filterName);
    }

    if (strcmp (filterInfo->filterName, "Nband") == 0)
    {
        filterInfo->optFreqLo = N_BAND_FILT_LONG;
        filterInfo->optFreqHi = N_BAND_FILT_SHORT;
    }
    else if (strcmp (filterInfo->filterName, "OPEN") == 0)
    {
        filterInfo->optFreqLo = N_BAND_FILT_LONG;
        filterInfo->optFreqHi = N_BAND_FILT_SHORT;
    }
    else if (strcmp (filterInfo->filterName, "SiC") == 0)
    {
        filterInfo->optFreqLo = SIC_FILT_LONG;
        filterInfo->optFreqHi = SIC_FILT_SHORT;
    }
    else if (strcmp (filterInfo->filterName, "N8.7") == 0)
    {
        filterInfo->optFreqLo = N8_7_FILT_LONG;
        filterInfo->optFreqHi = N8_7_FILT_SHORT;
    }
    else if (strcmp (filterInfo->filterName, "ArIII") == 0)
    {
        filterInfo->optFreqLo = ArIII_FILT_LONG;
        filterInfo->optFreqHi = ArIII_FILT_SHORT;
    }
    else if (strcmp (filterInfo->filterName, "SIV") == 0)
    {
        filterInfo->optFreqLo = SIV_FILT_LONG;
        filterInfo->optFreqHi = SIV_FILT_SHORT;
    }
    else if (strcmp (filterInfo->filterName, "N11.3") == 0)
    {
        filterInfo->optFreqLo = N11_3_FILT_LONG;
        filterInfo->optFreqHi = N11_3_FILT_SHORT;
    }
    else if (strcmp (filterInfo->filterName, "NeII") == 0)
    {
        filterInfo->optFreqLo = NEII_FILT_LONG;
        filterInfo->optFreqHi = NEII_FILT_SHORT;
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Filter ID");
       cpl_msg_info(cpl_func,"             Default Higher %f micron and Lower %f micron are used \n\n", filterInfo->optFreqLo, filterInfo->optFreqHi);
        fprintf(midiReportPtr, "             Default Higher %f micron and Lower %f micron are used \n\n", filterInfo->optFreqLo, filterInfo->optFreqHi);
        free (cleanString);
        return;
    }

    if (diagnostic)cpl_msg_info(cpl_func,"Filter ID                       = %s \n", filterInfo->filterName);
    if (diagnostic)cpl_msg_info(cpl_func,"Lower optical filter wavelength = %f micron  \n", filterInfo->optFreqHi);
    if (diagnostic)cpl_msg_info(cpl_func,"Upper optical filter wavelength = %f micron  \n\n", filterInfo->optFreqLo);
    fprintf (midiReportPtr, "Filter ID                       = %s \n", filterInfo->filterName);
    fprintf (midiReportPtr, "Lower optical filter wavelength = %f micron  \n", filterInfo->optFreqHi);
    fprintf (midiReportPtr, "Upper optical filter wavelength = %f micron  \n\n", filterInfo->optFreqLo);

    //    Convert to THz
    filterInfo->optFreqLo = SPEED_OF_LIGHT / filterInfo->optFreqLo;
    filterInfo->optFreqHi = SPEED_OF_LIGHT / filterInfo->optFreqHi;
    
    free (cleanString);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  getFitsExtensionNumber
* Input/Output: See function arguments to avoid duplication
* Description:  This routine returns the extension number of the given extension
*
* History:      
* 04-Oct-04     (csabet) Created
******************************************************************************/
int getFitsExtensionNumber (    // Ou: Extension number
    char    *fileName,            // In: Name of the input FITS file
    const char    *extName,            // In:    Given extension name
    int        *error)                // Ou:    Error status
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "getFitsExtensionNumber";
    int         i, numOfExtensions = 0, extNumber = -1, found = 0;
    char        *stringTemp, *cleanString;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;

    //  Allocate memory
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //  Get the total number of extensions
    numOfExtensions = qfits_query_n_ext (fileName);

    if (diagnostic)cpl_msg_info(cpl_func,"Number of extensions          = %d\n", numOfExtensions);
    if (diagnostic) fprintf(midiReportPtr, "Number of extensions          = %d\n", numOfExtensions);

    for (i = 0; i < numOfExtensions+1; i++)
    {
        stringTemp = qfits_query_ext (fileName, "EXTNAME", i);
        if (stringTemp != NULL)
        {
            cleanUpString (stringTemp, cleanString);

            if (strcmp (cleanString, extName) == 0) 
            {
                if (diagnostic)cpl_msg_info(cpl_func,"%s extension number = %d\n", extName, i);
                if (diagnostic) fprintf (midiReportPtr, "%s extension number = %d\n", extName, i);
                extNumber = i;
                found = 1;
                break;
            }
        }
    }

    if (found == 0)
    {
        sprintf (midiMessage, "Cannot find FITS   '%s'  extension", extName);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
    }

    //    Release memory
    free (cleanString);
    
    return (extNumber);
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createMidiMaskFile
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a mask file from an input photometry file
*
* History:
* 26-Nov-03 (csabet) Created
******************************************************************************/
void createMidiMaskFile (
    int            widthX,            // In: Column dimension
    int            widthY,            // In: Column dimension
    char        *inFileName,    // In: Name of the input data file
    short int    *inData1,        // In: Pointer to the input data
    short int    *inData2,        // In: Pointer to the input data
    int            numOfFrames,    // In: Number of frames
    int            subWindowSize,    // In: Sizeof the sub-window
    char        *maskFileName,    // In: Full path name of the mask file
    int            *error)            // Ou: Error status
{

    /*  Local Declarations
    --------------------*/
    const char      routine[] = "createMidiMaskFile";
    float           *outData1, *outData2;
    qfits_header    *primHeadMask;
    FILE            *maskFitsPtr;
    char            *dateTime;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /*  Reset status */
    *error = 0;

    if (diagnostic)cpl_msg_info(cpl_func,"Creating mask FITS file   %s \n", maskFileName);
    fprintf (midiReportPtr, "Creating mask FITS file   %s \n", maskFileName);

    /*  Extract Primary Header from the input file */
    if (diagnostic)cpl_msg_info(cpl_func,"Extracting primary header from input FITS file   %s \n", inFileName);
    if (diagnostic) fprintf(midiReportPtr, "Extracting primary header from input FITS file   %s \n", inFileName);
    primHeadMask = qfits_header_read (inFileName);
    if (primHeadMask == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot extract primary header from input FITS file");
       cpl_msg_info(cpl_func,"\nCannot Create mask FITS file for batch  %d \n", batchNumber);
        fprintf (midiReportPtr, "\nCannot Create mask FITS file for batch  %d \n", batchNumber);
        *error = 1;
        return;
    }

    /*  Update primary header for the Mask file */
    dateTime = qfits_get_datetime_iso8601 ();
    qfits_header_mod (primHeadMask, "DATE", dateTime, "Date and time this file was written");
    qfits_header_mod (primHeadMask, "INSTRUME", "MIDI", "MIDI Mask created by DRS pipeline" );
    qfits_header_add (primHeadMask, "EXTNAME", "Primary", "MIDI Mask primary header", "" );

    /*  Create the mask file and dump the primary header and close the file */
    maskFitsPtr = fopen (maskFileName, "w");
    qfits_header_dump (primHeadMask, maskFitsPtr);
    qfits_header_destroy (primHeadMask);
    fclose (maskFitsPtr);

    /*  Allocate memory */
    outData1 = (float *) calloc (subWindowSize, sizeof (float));
    outData2 = (float *) calloc (subWindowSize, sizeof (float));

    /*  Load data */
    loadMidiMaskData (numOfFrames, subWindowSize, inData1, inData2, outData1, outData2, error);

    /*  Create the tables and write the data */
    writeMidiMaskFile (widthX, widthY, maskFileName, subWindowSize, outData1, outData2, error);
    if (!(*error))
    {
       cpl_msg_info(cpl_func,"Created Mask FITS file: %s \n", maskFileName);
        fprintf (midiReportPtr, "Created Mask FITS file: %s \n", maskFileName);
    }

    /*  Release memory */
    free (outData1);
    free (outData2);

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  loadMidiMaskData
* Input/Output: See function arguments to avoid duplication
* Description:  Uses the input data to compute masks and loads MIDI mask data
*               into appropriate data structures
*
*
* History:
* 26-Nov-03     (csabet) Created
******************************************************************************/
void loadMidiMaskData (
    int        numOfFrames,    // In: Number of frames in the data
    int        subWindowSize,    // In: Size of sub-window
    short    *inData1,        // In: Pointer to input data
    short    *inData2,        // In: Pointer to input data
    float    *outData1,        // Ou: Pointer to output data
    float    *outData2,        // Ou: Pointer to output data
    int        *error)            // Ou: Status
{
    /*  Local Declarations
    --------------------*/
    const char      routine[] = "loadMidiMaskData";
    int             i, j, pixel;
    char            *tempFileName;
    FILE            *tempFilePtr;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /*  Reset status */
    *error = 0;

    /*  Create mask data */
    for (i = 0; i < subWindowSize; i++)
    {
        outData1[i] = 1.0;
        outData2[i] = 1.0;
    }

    if (plotFile && diagnostic)
    {
        /*  Allocate memory */
        tempFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        
        sprintf (tempFileName, "%s%s.AvePhotomData1.plt", outFileDir, outRootName);
        tempFilePtr = fopen (tempFileName, "w");
        for (j = 0; j < numOfFrames; j++)
        {
            for (pixel = 0; pixel < subWindowSize; pixel++)
            {
                i = j * subWindowSize + pixel;
                fprintf (tempFilePtr, "%d ", inData1[i]);
            }
            fprintf (tempFilePtr, "\n");
        }
        fclose (tempFilePtr);

        sprintf (tempFileName, "%s%s.AvePhotomData2.plt", outFileDir, outRootName);
        tempFilePtr = fopen (tempFileName, "w");
        for (j = 0; j < numOfFrames; j++)
        {
            for (pixel = 0; pixel < subWindowSize; pixel++)
            {
                i = j * subWindowSize + pixel;
                fprintf (tempFilePtr, "%d ", inData2[i]);
            }
            fprintf (tempFilePtr, "\n");
        }
        fclose (tempFilePtr);

        sprintf (tempFileName, "%s%s.maskData1.plt", outFileDir, outRootName);
        tempFilePtr = fopen (tempFileName, "w");
        for (i = 0; i < subWindowSize; i++)
            fprintf (tempFilePtr, "%5.4f\n", outData1[i]);
        fclose (tempFilePtr);

        sprintf (tempFileName, "%s%s.maskData2.plt", outFileDir, outRootName);
        tempFilePtr = fopen (tempFileName, "w");
        for (i = 0; i < subWindowSize; i++)
            fprintf (tempFilePtr, "%5.4f\n", outData2[i]);
        fclose (tempFilePtr);

        /*  release memory */
        free (tempFileName);
    }

    return;

}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  writeMidiMaskFile
* Input/Output: See function arguments to avoid duplication
* Description:  Writes and creates appropriate tables into a new mask fits file
*
* History:
* 26-Nov-03 (csabet) Created
******************************************************************************/
void writeMidiMaskFile (
    int        widthX,            // In: Column dimension
    int        widthY,            // In: Column dimension
    char    *maskFitsName,    // In: Name of the FITS file
    int        numOfElements,    // In: Number of elements
    float    *data1,            // In: Pointer to the data1
    float    *data2,            // In: Pointer to the data2
    int        *error)            // Ou: Status
{
    /*  Local Declarations
    --------------------*/
    const char  routine[] = "writeMidiMaskFile";
    int     numOfRowsImagData = 1, numOfElementsImagData;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /*  Reset status */
    *error = 0;

    if (diagnostic)cpl_msg_info(cpl_func,"Writing data into the mask FITS file \n\n");
    fprintf (midiReportPtr, "Writing data into the mask FITS file \n\n");

    /*  Write IMAGING_DATA extension */
    numOfElementsImagData = numOfElements;
    writeMidiMaskImagData (widthX, widthY, maskFitsName, numOfElementsImagData, numOfRowsImagData,
        data1, data2, error);

    /*  Add other extensions if needed */
    /*  TBD */

    return;

}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  writeMidiMaskImagData
* Input/Output: See function arguments to avoid duplication
* Description:  Writes IMAGING_DATA fits binary table in the MIDI mask file
*       with an appropriate header.
*
*
* History:
* 27-Nov-03 (csabet) Created.
******************************************************************************/
void writeMidiMaskImagData (
    int        iWidthX,        // In: Column dimension
    int        iWidthY,        // In: Column dimension
    char    *maskFitsName,    // In: Name of the FITS file
    int        numOfElements,    // In: Number of elements in data1 and data2
    int        numOfRows,        // In: Number of rows for data1 and data2
    float    *data1,            // In: Pointer to the data1
    float    *data2,            // In: Pointer to the data2
    int        *error)            // Ou: Status
{

    /*  Local Declarations
    --------------------*/
    const char      routine[] = "writeMidiMaskImagData";
    qfits_table     *maskTable;
    int             tableWidth = -1, numOfColumns = 2;
    float           **array;
    qfits_col       *col;
    qfits_header    *imagDataHeader;
    FILE            *maskFitsPtr;
    char            *tdim2, *tdim1;

    /*  Algorithm
    -----------*/
    tableWidth = -1;

    /*  Reset status */
    *error = 0;

    /*  Allocate memory */
    tdim1 = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    tdim2 = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*  Open the mask file for writing */
    maskFitsPtr = fopen (maskFitsName, "a");
    if (maskFitsPtr == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open mask file");
        *error = 1;
        free (tdim1);
        free (tdim2);
        return;
    }

    /*  Create a new table */
    maskTable = qfits_table_new (maskFitsName, QFITS_BINTABLE, tableWidth, numOfColumns, numOfRows);
    if (!maskTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create table");
        *error = 1;
        free (tdim1);
        free (tdim2);
        fclose (maskFitsPtr);
        return;
    }

    /*  Fill in the column information */
    col = maskTable->col;
    qfits_col_fill (col, numOfElements, 2, sizeof(float), TFITS_BIN_TYPE_E, "DATA1", " ", " ", " ",
        0, 0.0, 0, 1.0, 0);
    col++;
    qfits_col_fill (col, numOfElements, 2, sizeof(float), TFITS_BIN_TYPE_E, "DATA2", " ", " ", " ",
        0, 0.0, 0, 1.0, numOfElements* sizeof(float));

    /*  Create the imaging data header */
    imagDataHeader = qfits_table_ext_header_default(maskTable);
    qfits_header_add (imagDataHeader, "EXTNAME", "IMAGING_DATA", "MIDI Mask Extension name", "" );


    /*  Add dimensionality to the extension header */
    sprintf (tdim1, "'(%d,%d)'", iWidthX, iWidthY);
    sprintf (tdim2, "'(%d,%d)'", iWidthX, iWidthY);
    qfits_header_add (imagDataHeader, "TDIM1", tdim1, "Column dimensionality", "" );
    qfits_header_add (imagDataHeader, "TDIM2", tdim2, "Column dimensionality", "" );


//  qfits_header_consoledump (imagDataHeader);

    /*  Allocate space for the array of data */
    array = (float **) calloc (numOfColumns, sizeof(float *));
    array[0] = data1;
    array[1] = data2;

    if (qfits_table_append_xtension_hdr (maskFitsPtr, maskTable, (const void **)array, imagDataHeader) == -1)
        *error = 1;

    qfits_header_destroy (imagDataHeader);
    qfits_table_close (maskTable);
    free (array);
    fclose (maskFitsPtr);
    free (tdim1);
    free (tdim2);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  findImagingDataExtension
* Input/Output: See function arguments to avoid duplication
* Description:  This routine returns the extension number of the IMAGING_DATA
*
* History:
* 14-Aug-03     (csabet) Created. Derived from pballest July 2003
******************************************************************************/
int findImagingDataExtension (        // Ou: Extension number
    char    *fileName,              // In: Name of the input FITS file
    int        defaultExtensionNumber, // In: efault extension number
    int        *error)                    // Ou: Error status
{
    /*  Local Declarations
    --------------------*/
    const char  routine[] = "findImagingDataExtension";
    int         currentExtension    = 0;
    int         lastExtensionNumber = 0;
    char        *stringTemp, *cleanString;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /*  Reset status */
    *error = 0;

    /*  Allocate memory */
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    if (diagnostic)
    {
        sprintf (midiMessage, "Searching for IMAGING_DATA extension in data file %s", fileName);
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }

    /*  Get the total number of extensions */
    lastExtensionNumber = qfits_query_n_ext (fileName);

    if (diagnostic)
    {
        sprintf (midiMessage, "Number of extensions          = %d", lastExtensionNumber);
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }

    if (defaultExtensionNumber <= lastExtensionNumber)
    {
        stringTemp = qfits_query_ext (fileName, "EXTNAME", defaultExtensionNumber);
        cleanUpString (stringTemp, cleanString);

        if (strcmp (cleanString, "IMAGING_DATA") == 0)
        {
            currentExtension = defaultExtensionNumber;
            if (diagnostic)
            {
                sprintf (midiMessage, "IMAGING_DATA extension number = %d", currentExtension);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }

            /*  Found. return extension number */
            free (cleanString);
            return (currentExtension);
        }
    }

    /*  In all other cases we search through all extensions for the proper one */
    for (currentExtension = 1; currentExtension <= lastExtensionNumber; ++currentExtension)
    {
        stringTemp = qfits_query_ext (fileName, "EXTNAME", currentExtension);
        cleanUpString (stringTemp, cleanString);
        if (strcmp (cleanString, "IMAGING_DATA") == 0) break;
    }

    if (currentExtension <= lastExtensionNumber)
    {
        if (diagnostic)
        {
            sprintf (midiMessage, "IMAGING_DATA extension number = %d", currentExtension);
            midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        }
        free (cleanString);
        return (currentExtension);
    }
    else
    {
        if (diagnostic)
        {
            sprintf (midiMessage, "No IMAGING_DATA extension in %s", fileName);
            midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        }
        free (cleanString);
        return (-1);    // Indicating no data
    }

    if (diagnostic) 
    {
        sprintf (midiMessage, "IMAGING_DATA extension number = %d", currentExtension);
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
    
    /* Release memory */
    free (cleanString);

    return (currentExtension);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  selectMask
* Input/Output: See function arguments to avoid duplication
* Description:  Selects a mask file from the information given in the input
*               FITS file. If this Mask file does not exist or is not a FITS
*               file the program terminates.
*
*
* History:
* 14-Aug-03     (csabet) Created. Derived from pballest July 2003
******************************************************************************/
void selectMask (
    enum MaskMode    maskMode,    // In:  Method of choosing mask file
    MidiFiles        *fileNames,    // In:  Pointer to the file structure
    char            *maskFile,    // Ou:  Full name of the mask
    int                *error)        // Ou:  Error status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "selectMask";
    char          *qfitsTemp, *stringTemp;
    struct stat   buf;
    FILE          *temp1Ptr = NULL, *temp2Ptr = NULL;
    size_t        stringLength;
    unsigned int  i;
    char      nextChar, *cleanString;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf( midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //    Allocate memory
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char ));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char ));

    //    Determine where to look for the mask
    switch (maskMode)
    {
       case SPECIFIED:
          //    Remove prefix
/*             removePathName (fileNames->maskFileName, midiReportPtr); */
/*             sprintf (maskFile, "%s%s", fileNames->maskFileDir, fileNames->maskFileName); */
          sprintf (maskFile, "%s", fileNames->maskFileName); 
          cpl_msg_info(cpl_func,"maskFile: %s",maskFile);
          break;

        case SELECTED:
            qfitsTemp = qfits_query_hdr (fileNames->inFitsName, "HIERARCH ESO DET FITSTPL");
            if (qfitsTemp != NULL)
            {
                cleanUpString (qfitsTemp, cleanString);

                /*  Replace the prefix */
                stringLength = strlen (cleanString);
                temp1Ptr = fopen("temp1.grb", "w");
                fprintf (temp1Ptr, "%s\n", cleanString);
                fclose (temp1Ptr);
                temp1Ptr = fopen("temp1.grb", "r");
                temp2Ptr = fopen("temp2.grb", "w");
                for (i = 0; i < stringLength; i++)
                {
                    fscanf (temp1Ptr, "%c", &nextChar);
                    if (i > 7) fprintf (temp2Ptr, "%c", nextChar);
                }
                fclose (temp1Ptr);
                fclose (temp2Ptr);
                temp2Ptr = fopen("temp2.grb", "r");
                fscanf (temp2Ptr, "%s\n", stringTemp);
                fclose (temp2Ptr);

                remove ("temp1.grb");
                remove ("temp2.grb");
                sprintf (maskFile, "%s%s%s", fileNames->maskFileDir, "minrtsMask", stringTemp);
            }
            else
            {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read mask name");
                *error = 1;
            }
            break;

        case COMPUTED:
            sprintf (maskFile, "%s%s", fileNames->maskFileDir, fileNames->maskFileName);
            break;

        default:
            break;
    }

    /*  Rewrite the mask file for record */
    sprintf (fileNames->maskFileName, "%s", maskFile);

    /*  Check if the file exist */
    if (strcmp (maskFile, "") == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find a suitable mask file");
        *error = 1;
    }
    else if (stat (maskFile, &buf) == -1)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find mask file");
        *error = 1;
    }
    else if ((qfits_is_fits (maskFile)) != 1)    /*  Check if this is a FITS file */
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Found Non-FITS file");
        *error = 1;
    }

    /*  Release memory */
    free (stringTemp);
    free (cleanString);

    return;
}
/*****************************************************************************/

 
/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiInitFormat
* Input/Output: See function arguments to avoid duplication
* Description:  Initialises image format
*
* History:      
* 10-Jul-05     (csabet) created
******************************************************************************/
void midiInitFormat (
    ImageFormat *format)    // Ou: Pointer to the image format structure
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "midiInitFormat";
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    format->hasData = 0;
    format->numOfDetectorRegions = 0;
    format->numOfRegionsToProcess = 0;
    format->numOfPinholes = 0;
    format->numOfFrames = 0;
    format->framesPerScan = 0;
    format->numOfScans = 0;
    format->subWindowSize = 0;
    format->iXWidth = 0;
    format->iYWidth = 0;
     
    return;
}
/******************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiCopyFormat
* Input/Output: See function arguments to avoid duplication
* Description:  Copies image format
*
* History:      
* 10-Jul-05     (csabet) created
******************************************************************************/
void midiCopyFormat (
    ImageFormat *formatIn,    // In: Pointer to the image format structure
    ImageFormat *formatOut)    // Ou: Pointer to the image format structure
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "midiCopyFormat";
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    sprintf (formatOut->obsCatg, "%s", formatIn->obsCatg);
    sprintf (formatOut->obsTech, "%s", formatIn->obsTech);
    sprintf (formatOut->obsType, "%s", formatIn->obsType);
    sprintf (formatOut->cameraId, "%s", formatIn->cameraId);
    sprintf (formatOut->beamCombiner, "%s", formatIn->beamCombiner);
    sprintf (formatOut->filterName, "%s", formatIn->filterName);
    sprintf (formatOut->shutterId, "%s", formatIn->shutterId);
    sprintf (formatOut->grismId, "%s", formatIn->grismId);
    sprintf (formatOut->targetName, "%s", formatIn->targetName);
    sprintf (formatOut->slitName, "%s", formatIn->slitName);
    sprintf (formatOut->tplStart, "%s", formatIn->tplStart);
    sprintf (formatOut->tplName, "%s", formatIn->tplName);
    formatOut->numOfDetectorRegions = formatIn->numOfDetectorRegions;
    formatOut->numOfRegionsToProcess = formatIn->numOfRegionsToProcess;
    formatOut->numOfPinholes = formatIn->numOfPinholes;
    formatOut->framesPerScan = formatIn->framesPerScan;
    formatOut->subWindowSize = formatIn->subWindowSize;
    formatOut->iXWidth = formatIn->iXWidth;
    formatOut->iYWidth = formatIn->iYWidth;
     
    return;
}
/******************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiReportFormat
* Input/Output: See function arguments to avoid duplication
* Description:  Reports image format
*
* History:      
* 05-Dec-05     (csabet) created
******************************************************************************/
void midiReportFormat (
    const char        *type,        // In: Type of file
    int            isDisp,        // In: Whether DISPERSED or not
    ImageFormat *format)    // In: Pointer to the image format structure
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "midiReportFormat";
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func," \n");
   cpl_msg_info(cpl_func,"%s FORMAT \n", type);
   cpl_msg_info(cpl_func,"--------------------- \n");
   cpl_msg_info(cpl_func,"FITS Template Name           = %s \n", format->tplName);
   cpl_msg_info(cpl_func,"Batch Template Name          = %s \n", batchTemplate);
   cpl_msg_info(cpl_func,"Observation Category         = %s \n", format->obsCatg);
   cpl_msg_info(cpl_func,"Observation Technique        = %s \n", format->obsTech);
   cpl_msg_info(cpl_func,"Observation Type             = %s \n", format->obsType);
   cpl_msg_info(cpl_func,"Camera ID                    = %s \n", format->cameraId);
   cpl_msg_info(cpl_func,"Beam Combiner                = %s \n", format->beamCombiner);
   cpl_msg_info(cpl_func,"Filter ID                    = %s \n", format->filterName);
   cpl_msg_info(cpl_func,"Shutter ID                   = %s \n", format->shutterId);
   cpl_msg_info(cpl_func,"Grism ID                     = %s \n", format->grismId);
   cpl_msg_info(cpl_func,"Slit Name                    = %s \n", format->slitName);
   cpl_msg_info(cpl_func,"Target Name                  = %s \n", format->targetName);
   cpl_msg_info(cpl_func,"TPL Start                    = %s \n", format->tplStart);
    if (isDisp)
       cpl_msg_info(cpl_func,"Data is                      = DISPERSED \n");
    else
       cpl_msg_info(cpl_func,"Data is                      = UNDISPERSED \n");
   cpl_msg_info(cpl_func,"Number of Frames             = %d \n", format->numOfFrames);
    if ((strcmp(format->beamCombiner, "SCI_PHOT") == 0) || (strcmp (format->shutterId, "ABOPEN") == 0))
    {
       cpl_msg_info(cpl_func,"Frames per scan              = %d \n", format->framesPerScan);
       cpl_msg_info(cpl_func,"Number of Scans              = %d \n", format->numOfScans);
    }
   cpl_msg_info(cpl_func,"X, Column dimension          = %d \n", format->iXWidth);
   cpl_msg_info(cpl_func,"Y, Column dimension          = %d \n", format->iYWidth);
   cpl_msg_info(cpl_func,"Sub-window size              = %d \n", format->subWindowSize);
   cpl_msg_info(cpl_func,"Number of Pinholes           = %d \n", format->numOfPinholes);
   cpl_msg_info(cpl_func,"Number of Detector Regions   = %d \n", format->numOfDetectorRegions);
   cpl_msg_info(cpl_func,"Number of Regions to process = %d \n", format->numOfRegionsToProcess);
   cpl_msg_info(cpl_func,"\n");

    fprintf (midiReportPtr, " \n");
    fprintf (midiReportPtr, "%s FORMAT \n", type);
    fprintf (midiReportPtr, "--------------------- \n");
    fprintf (midiReportPtr, "FITS Template Name           = %s \n", format->tplName);
    fprintf (midiReportPtr, "Batch Template Name          = %s \n", batchTemplate);
    fprintf (midiReportPtr, "Observation Category         = %s \n", format->obsCatg);
    fprintf (midiReportPtr, "Observation Technique        = %s \n", format->obsTech);
    fprintf (midiReportPtr, "Observation Type             = %s \n", format->obsType);
    fprintf (midiReportPtr, "Camera ID                    = %s \n", format->cameraId);
    fprintf (midiReportPtr, "Beam Combiner                = %s \n", format->beamCombiner);
    fprintf (midiReportPtr, "Filter ID                    = %s \n", format->filterName);
    fprintf (midiReportPtr, "Shutter ID                   = %s \n", format->shutterId);
    fprintf (midiReportPtr, "Grism ID                     = %s \n", format->grismId);
    fprintf (midiReportPtr, "Slit Name                    = %s \n", format->slitName);
    fprintf (midiReportPtr, "Target Name                  = %s \n", format->targetName);
    fprintf (midiReportPtr, "TPL Start                    = %s \n", format->tplStart);
    if (isDisp)
        fprintf (midiReportPtr, "Data is                      = DISPERSED \n");
    else
        fprintf (midiReportPtr, "Data is                      = UNDISPERSED \n");
    fprintf (midiReportPtr, "Number of Frames             = %d \n", format->numOfFrames);
    if ((strcmp(format->beamCombiner, "SCI_PHOT") == 0) || (strcmp (format->shutterId, "ABOPEN") == 0))
    {
        fprintf (midiReportPtr, "Frames per scan              = %d \n", format->framesPerScan);
        fprintf (midiReportPtr, "Number of Scans              = %d \n", format->numOfScans);
    }
    fprintf (midiReportPtr, "X, Column dimension          = %d \n", format->iXWidth);
    fprintf (midiReportPtr, "Y, Column dimension          = %d \n", format->iYWidth);
    fprintf (midiReportPtr, "Sub-window size              = %d \n", format->subWindowSize);
    fprintf (midiReportPtr, "Number of Pinholes           = %d \n", format->numOfPinholes);
    fprintf (midiReportPtr, "Number of Detector Regions   = %d \n", format->numOfDetectorRegions);
    fprintf (midiReportPtr, "Number of Regions to process = %d \n", format->numOfRegionsToProcess);
    fprintf (midiReportPtr, "\n");

    return;
}
/******************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  selectChannels
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void selectChannels (
    int            start,        // In: Start region
    int            end,        // In: End region
    ImageFormat    *format,    // In: File format
    float        **mask)        // In: Mask file
{
    //    Local Declarations
    //    ------------------
    const char  routine[] = "selectChannels";
    int            i, x, y, r, masked;
    float        *array;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nSelecting active channels based on the given mask \n");
   cpl_msg_info(cpl_func,"------------------------------------------------- \n");
    fprintf(midiReportPtr, "\nSelecting active channels based on the given mask \n");
    fprintf(midiReportPtr, "------------------------------------------------- \n");

    //    Allocate memory
    array = (float *) calloc (format->iXWidth, sizeof(float));
    
    for (r = start; r < end; r++)
    {
        for (y = 0; y < format->iYWidth; y++)
        {
            for (x = 0; x < format->iXWidth; x++)
            {
                i = x + y * format->iXWidth;
                if (mask[r][i] > array[x]) array[x] = mask[r][i];
            }
        }
    }

    masked = 0;
    for (x = 0; x < format->iXWidth; x++)
    {
        if (array[x] == 0.0)
        {
            masked++;
            badChannelList[x] |= BSL_MASKED_CHANNEL;
        }
    }

   cpl_msg_info(cpl_func,"Number of active channels = %3d \n\n", (format->iXWidth - masked));
    fprintf(midiReportPtr, "Number of active channels = %3d QCLOG \n\n", (format->iXWidth - masked));

    //    Plot
    if (plotFile && diagnostic > 1)    midiCreatePlotFile2D ("SelectedChannels", "Selected Channels", 
        "Wavelength Channel", "Mask Profile", 0, array, 0, format->iXWidth, 1, 0);

    //    Release memory
    free (array);
    
    return;
}
/*****************************************************************************/

