/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  photometry.c
* Description:
*
* History:
* 03-Feb-03     (jmeisner) Created
* 03-Mar-03     (csabet) Added title, directives and included in the MIDI pipeline
* 21-Jan-05     (csabet) Major clean up and redesign
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include <math.h>
#include "midiGlobal.h"
#include <string.h>
#include "midiLib.h"
#include "statistics.h"
#include "diagnostics.h"
#include "photometry.h"
#include "errorHandling.h"


/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables
**********************************************************/

/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  estimatePhotom
* Input/Output: See function arguments to avoid duplication
* Description:  
*
* History:        
* 11-Aug-05        (csabet) Created
*****************************************************************************/
void estimatePhotom(
    char            key,            // In: Strig character indicating type
    CompressedData    *compressed,    // In: Compressed data
    ImageFormat        *format,        // In: Photometry size
    float            **photom,        // Ou: Photometry result
    float            **photomErr,    // Ou: Estimated photometry RMS error
    int                *error)
{
    
    //    Local Declarations
    //    ------------------
    const char    routine[] = "estimatePhotom";
    int            region, X, regionId;
    float         choppingRMS;        // Measure of chop-to-chop rms RELATIVE to full photometry, NOT USED;
    char        *string, *title;
    float        *arrayPlot;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"\nInvoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "\nInvoking      routine   '%s' \n", routine);

    //    Reset parameters
    *error = 0;
    
    //    Exception
    if (format->numOfFrames < 2)
    {
       cpl_msg_info(cpl_func,"Not enough data. Expected more than 1. Found %d", format->numOfFrames);
        fprintf (midiReportPtr, "Not enough data. Expected more than 1. Found %d", format->numOfFrames);
        *error = 1;
        return;
    }

    //    If not chopped exit
    if (!(format->chopped))
    {
        *error = 1;
        return;
    }

   cpl_msg_info(cpl_func,"\n");
    fprintf (midiReportPtr, "\n");

    for (region = 0; region < format->numOfRegionsToProcess; region++)
    {
        if (plotFile)
            arrayPlot = (float *) calloc (format->iXWidth, sizeof(float));

        if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
        {
            if (key == 'A') regionId = 1;
            else if (key == 'B') regionId = 4;
            else regionId = region+2;
        }
        else
            regionId = region+1;

        if (diagnostic)cpl_msg_info(cpl_func, "Running dispersed Photom %c, for region %d\n", key, regionId);
        fprintf (midiReportPtr, "Running dispersed Photom %c, for region %d\n", key, regionId);

        for (X = 0; X < format->iXWidth; X++)
        {
            if (badChannelList[X])
                continue;
            
            if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
            {
                photom[region][X] = getChopPhotomSP (X, (((compressed->iDispFringe)[region])[X]), 
                    format->numOfScans, format->framesPerScan, compressed->rejectList[X], &(photomErr[region][X]), 
                    &choppingRMS);
            }
            else
            {
                photom[region][X] = getChopPhotom (X, (((compressed->iDispFringe)[region])[X]), 
                    compressed->tarType, &(photomErr[region][X]), &choppingRMS, format->numOfFrames);
            }

            if (isnan(photom[region][X]))
                badChannelList[X] |= BSL_DATA_ERROR;

            if (plotFile)
            {
                if (isnan(photom[region][X]))
                    arrayPlot[X] = 0.0;
                else
                    arrayPlot[X] = photom[region][X];
            }
        }
        
        if (plotFile)
        {
            title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            string = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            sprintf (string, "Photom%cDATA%d", key, regionId);
            sprintf (title, "Photometry %c, for Region %d", key, regionId);
            midiCreatePlotFile2D (string, title, "Channel", "Photometry", 0, photom[region], 0, format->iXWidth, 1, 0);
            sprintf (string, "Photom%cErrDATA%d", key, regionId);
            sprintf (title, "Photometry %c Error, for Region %d", key, regionId);
            midiCreatePlotFile2D (string, title, "Channel", "Photometry Error", 0, photomErr[region], 0, format->iXWidth, 1, 0);
            free (string);
            free (title);
            free (arrayPlot);
        }
    }

    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  getChopPhotomSP
* Input/Output: See function arguments to avoid duplication
* Description:  
*
* History:        
* 14-Feb-06        (csabet) Created
******************************************************************************/
float getChopPhotomSP (        // Ou: Photometry
    int        X,                // In: Wavelength channel
    float    *compressed,    // In: Pointer to compressed data for one X and one R
    int     numOfScans,        // In: Number of frames
    int     framesPerScan,    // In: Number of frames per scan
    int        *rejectList,    // In: Bad Frame List
    float    *errRMS,        // Ou: rms error returned
    float    *choppingRMS)    // Ou: Measure of chop-to-chop rms RELATIVE to full photometry;
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "getChopPhotomSP";
    int            f, j, s, numOfAccum, length;
    float        accum, *results, photom, variance, rms;

    //    Algorithm
    //    ---------

    //    Allocate memory
    results = (float *) calloc (numOfScans, sizeof (float));
        
    //    Create an array of photometric results for 'T' only
    length = 0;
    for (s = 0; s < numOfScans; s++)
    {
        numOfAccum = 0;
        accum = 0.F;
        for (j = 0; j < framesPerScan; j++)
        {
            f = j + s * framesPerScan;
            if (!(rejectList[f]))
            {
                accum += (float) compressed[f];
                numOfAccum++;
            }
        }
        if (numOfAccum)
        {
            results[length] = accum / ((float) numOfAccum);
            length++;
        }
    }

    //    Compute statistics
    if (length)
    {
        photom = signalMean (results, 0, length);
        variance = signalVariance (results, 0, length, &rms);

        //    If zero variance, set rms to a small value
        if (variance == 0.0)
            rms = 0.1F;
    
        *errRMS = rms / sqrt((float)length);

        //    Return relative error per chop
        if (photom > 0.0)    
            *choppingRMS = rms / photom;
    }
    
    //    Exception
    if (photom <= 0.0 || !length)
    {
        photom = NOT_A_NUMBER;
        sprintf (midiMessage, "Cannot compute photometry for channel %3d", X);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        
        //    Reject this channel
        badChannelList[X] |= BSL_DATA_ERROR;
    }

    //    Release local memory
    free(results);
    
    return (photom);
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  getChopPhotom
* Input/Output: See function arguments to avoid duplication
* Description:  
*
* History:        
* 14-Feb-06        (csabet) Created from JM. Removed goto statements and restructured
******************************************************************************/
float getChopPhotom (
    int        X,                // In: Wavelength channel
    float    *collapsedData,    // In: Pointer to linear array of chopping data, sequential with time
    char    *targetType,    // In: Pointer to description array. 'S'=offsource, 'T'=on source, 'U'=???
    float    *errRMS,          // Ou: rms error returned
    float    *choppingRMS,    // Ou: Measure of chop-to-chop rms RELATIVE to full photometry;
    int     numData)        // In: Number of data points in the above arrays to use.
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "getChopPhotom";
    char        lastType;
    int            index, numOfaccum;
    float        accum, accum2, *results, photom, mean, variance, rms;

    //    Algorithm
    //    ---------

    //    Initialise
    lastType = 'U';
    accum = 0.F;
    numOfaccum = 0;

    //    Allocate memory
    results = (float *) calloc (numData, sizeof (float));
    
    //    Create an array of photometric results for 'T' only
    for (index = 0; index < numData; index++)
    {
        //    If state has changed then compute the average brightness in the previous state
        if ((targetType[index] != lastType) && numOfaccum)
        {
            if (targetType[index] == 'T')
            {
                results[index] = accum / ((float) numOfaccum);
            
                //    Reset parameters
                numOfaccum = 0;
                accum = 0.F;
            }
        }
        
        //    Now accumulate
        if (targetType[index] == 'T')
        {
            accum += (float) collapsedData[index];
            numOfaccum++;
        }
        
        //    Save current state
        lastType = targetType[index];
    }

    //    Alright, now results[] has a number of non-zero values. Compute statistics
    accum = 0.F;
    accum2 = 0.F;
    numOfaccum = 0;
    for (index = 0; index < numData; index++)
    {
        if (results[index] != 0.F)
        {
            accum += results[index];
            accum2 += results[index] * results[index];
            numOfaccum++;
        }
    }

    //    Compute mean and variance
    if (numOfaccum)
    {
        mean = accum / ((float) numOfaccum);
        variance = (accum2 - mean * mean * ((float) numOfaccum)) / ((float) (numOfaccum - 1));
    
        //    If zero variance, set rms to a small value
        if (variance > 0.0)
            rms = sqrt(variance);
        else
            rms = 0.1F;
    
        *errRMS = rms / sqrt((float)numOfaccum);
        photom = mean;

        //    Return relative error per chop
        if (photom > 0.0)    
            *choppingRMS = rms / photom;
    }

    //    Exception
    if (photom <= 0.0 || !numOfaccum)
    {
        photom = NOT_A_NUMBER;
        sprintf (midiMessage, "Cannot compute photometry for channel %3d", X);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        
        //    Reject this channel
        badChannelList[X] |= BSL_DATA_ERROR;
    }

    //    Release local memory
    free(results);
    
    return (photom);
}
/*****************************************************************************/




/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  estimatePhotomUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  For UNDISPERSED processing
*
* History:        
* 03-Mar-03        (csabet) Cleaned up and Integrated
******************************************************************************/
void estimatePhotomUndisp (
    ImageFormat        *format,
    CompressedData    *inData,        // In: Compressed photometry data
    float            *errs,            // Ou: array of rms error returned (size = numOfRegionsChopped)
    float            *choppingRMS,    // Ou: Measure of chop-to-chop rms RELATIVE to full photometry;
    float            *results)        // Ou: array of photometric results
{

    /*  Local Declarations
    --------------------*/
    int        region;
    double    *thisdata;

    /*  Algorithm
    -----------*/
    for (region = 0; region < format->numOfRegionsToProcess; region++)
    {
        if (region == 0)
            thisdata = inData->iFringe1;
        else
            thisdata = inData->iFringe2;

        results[region] =  getChopPhotomUndisp ((double *)thisdata, inData->tarType, errs+region, 
            choppingRMS+region, format->numOfFrames);

        if (diagnostic > 1)
        {
           cpl_msg_info(cpl_func,"Photometric measurement for region %d: %f \n", region+1, results[region]);
           cpl_msg_info(cpl_func,"Relative chop-to-chop rms:             %4.1f%% \n", 100.F*choppingRMS[region]);
            fprintf (midiReportPtr, "Photometric measurement for region %d: %f \n", region+1, results[region]);
            fprintf(midiReportPtr, "Relative chop-to-chop rms:              %4.1f%% \n", 100.F*choppingRMS[region]);
        }
    }
    
    return;
}
/*****************************************************************************/




/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  getChopPhotomDoubleUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  
*
* History:        
* 14-Feb-06        (csabet) Created from JM. Removed goto statements and restructured
******************************************************************************/
float getChopPhotomUndisp (
    double    *collapsedData,    // In: Pointer to linear array of chopping data, sequential with time
    char    *targetType,    // In: Pointer to description array. 'S'=offsource, 'T'=on source, 'U'=???
    float    *err,              // Ou: rms error returned
    float    *choppingRMS,    // Ou: Measure of chop-to-chop rms RELATIVE to full photometry;
    int     numData)        // In: Number of data points in the above arrays to use.
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "getChopPhotomUndisp";
    char        lastType;
    int            index, numOfaccum;
    float        accum, accum2, *results, photom, mean, variance, rms;

    //    Algorithm
    //    ---------

    //    Initialise
    lastType = 'U';
    accum = 0.F;
    numOfaccum = 0;

    //    Allocate memory
    results = (float *) calloc (numData, sizeof (float));
    
    //    Create an array of photometric results for 'T' only
    for (index = 0; index < numData; index++)
    {
        //    If state has changed then compute the average brightness in the previous state
        if ((targetType[index] != lastType) && numOfaccum)
        {
            if (targetType[index] == 'T')
            {
                results[index] = accum / ((float) numOfaccum);
            
                //    Reset parameters
                numOfaccum = 0;
                accum = 0.F;
            }
        }
        
        //    Now accumulate
        if (targetType[index] == 'T')
        {
            accum += (float) collapsedData[index];
            numOfaccum++;
        }
        
        //    Save current state
        lastType = targetType[index];
    }

    //    Alright, now results[] has a number of non-zero values. Compute statistics
    accum = 0.F;
    accum2 = 0.F;
    numOfaccum = 0;
    for (index = 0; index < numData; index++)
    {
        if (results[index] != 0.F)
        {
            accum += results[index];
            accum2 += results[index] * results[index];
            numOfaccum++;
        }
    }

    //    Compute mean and variance
    mean = accum / ((float) numOfaccum);
    variance = (accum2 - mean * mean * ((float) numOfaccum)) / ((float) (numOfaccum - 1));

    //    Compute rms. If zero variance, set rms to a small value
    if (variance > 0.0)
        rms = sqrt(variance);
    else
        rms = 0.1F;

    *err = rms / sqrt((float)numOfaccum);
    photom = mean;

    //    Singularity check
    if (photom > 0.0)    
        *choppingRMS = rms / photom;    //    Return relative err per chop.
    else
    {
        /*    In dispsersed mode, this will always happen on detector 
        columns with zero weighting. So don't want to get too upset about it */
        *choppingRMS = NOT_A_NUMBER;
        sprintf (midiMessage, "Singularity detected. Values in this batch are unreliable");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }

    //    Release local memory
    free(results);
    
    return (photom);
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  computeBinnedPhotom
* Input/Output: See function arguments to avoid duplication
* Description:  This function computes HIGH_SENS combined photometry levels of 
*                regions 1 and 2 at the follwing spectral channels:
*                1) Channels:    113 -> 121    (PRISM)
*                                052 -> 068    (GRISM) i.e. [ArIII] band
*
*                2) Channels:    088 -> 098    (PRISM)
*                                116 -> 136    (GRISM) i.e. [SIV] band
*
*                3) Channels:    036 -> 050    (PRISM)
*                                217 -> 245    (GRISM) i.e. [NeII] band
* History:        
* 21-Sep-06        (csabet) Created
******************************************************************************/
void computeBinnedPhotom (
    CompressedData    *photomA,    // In: Compressed data
    CompressedData    *photomB,    // In: Compressed data
    ImageFormat        *format,    // In: Photometry size
    float            *photomA1,    // Ou: Binned photometry
    float            *photomA2,    // Ou: Binned photometry
    float            *photomA3,    // Ou: Binned photometry
    float            *photomB1,    // Ou: Binned photometry
    float            *photomB2,    // Ou: Binned photometry
    float            *photomB3,    // Ou: Binned photometry
    int                *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "computeBinnedPhotom";
    int            i, j;

    //    Algorithm
    //    ---------

    //    Reset parameters
    *error = 0;
    *photomA1 = *photomA2 = *photomA3 = 0.0;
    *photomB1 = *photomB2 = *photomB3 = 0.0;

    if (strcmp(format->grismId, "PRISM") == 0)
    {
        for (i = 36; i < 51; i++)
        {
            if (badChannelList[i])
                continue;
                
            for (j = 0; j < format->numOfFrames; j++)
            {
                if (photomA->tarType[j] == 'T' && !(photomA->rejectList[i][j]))
                    *photomA3 += ( (((photomA->iDispFringe)[0])[i])[j] + (((photomA->iDispFringe)[1])[i])[j] );
                if (photomB->tarType[j] == 'T' && !(photomB->rejectList[i][j]))
                    *photomB3 += ( (((photomB->iDispFringe)[0])[i])[j] + (((photomB->iDispFringe)[1])[i])[j] );
            }
        }
        for (i = 88; i < 99; i++)
        {
            if (badChannelList[i])
                continue;
                
            for (j = 0; j < format->numOfFrames; j++)
            {
                if (photomA->tarType[j] == 'T' && !(photomA->rejectList[i][j]))
                    *photomA2 += ( (((photomA->iDispFringe)[0])[i])[j] + (((photomA->iDispFringe)[1])[i])[j] );
                if (photomB->tarType[j] == 'T' && !(photomB->rejectList[i][j]))
                    *photomB2 += ( (((photomB->iDispFringe)[0])[i])[j] + (((photomB->iDispFringe)[1])[i])[j] );
            }
        }
        for (i = 113; i < 122; i++)
        {
            if (badChannelList[i])
                continue;
                
            for (j = 0; j < format->numOfFrames; j++)
            {
                if (photomA->tarType[j] == 'T' && !(photomA->rejectList[i][j]))
                    *photomA1 += ( (((photomA->iDispFringe)[0])[i])[j] + (((photomA->iDispFringe)[1])[i])[j] );
                if (photomB->tarType[j] == 'T' && !(photomB->rejectList[i][j]))
                    *photomB1 += ( (((photomB->iDispFringe)[0])[i])[j] + (((photomB->iDispFringe)[1])[i])[j] );
            }
        }
    }
    else if (strcmp(format->grismId, "GRISM") == 0)
    {
        for (i = 52; i < 69; i++)
        {
            if (badChannelList[i])
                continue;
                
            for (j = 0; j < format->numOfFrames; j++)
            {
                if (photomA->tarType[j] == 'T' && !(photomA->rejectList[i][j]))
                    *photomA1 += ( (((photomA->iDispFringe)[0])[i])[j] + (((photomA->iDispFringe)[1])[i])[j] );
                if (photomB->tarType[j] == 'T' && !(photomB->rejectList[i][j]))
                    *photomB1 += ( (((photomB->iDispFringe)[0])[i])[j] + (((photomB->iDispFringe)[1])[i])[j] );
            }
        }
        for (i = 116; i < 137; i++)
        {
            if (badChannelList[i])
                continue;
                
            for (j = 0; j < format->numOfFrames; j++)
            {
                if (photomA->tarType[j] == 'T' && !(photomA->rejectList[i][j]))
                    *photomA2 += ( (((photomA->iDispFringe)[0])[i])[j] + (((photomA->iDispFringe)[1])[i])[j] );
                if (photomB->tarType[j] == 'T' && !(photomB->rejectList[i][j]))
                    *photomB2 += ( (((photomB->iDispFringe)[0])[i])[j] + (((photomB->iDispFringe)[1])[i])[j] );
            }
        }
        for (i = 217; i < 245; i++)
        {
            if (badChannelList[i])
                continue;
                
            for (j = 0; j < format->numOfFrames; j++)
            {
                if (photomA->tarType[j] == 'T' && !(photomA->rejectList[i][j]))
                    *photomA3 += ( (((photomA->iDispFringe)[0])[i])[j] + (((photomA->iDispFringe)[1])[i])[j] );
                if (photomB->tarType[j] == 'T' && !(photomB->rejectList[i][j]))
                    *photomB3 += ( (((photomB->iDispFringe)[0])[i])[j] + (((photomB->iDispFringe)[1])[i])[j] );
            }
        }
    }
    else
    {
        sprintf (midiMessage, "Unknown GRISM ID ... %s", format->grismId);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
        return;
    }

    return;
}
/*****************************************************************************/

