
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*       VLTI MIDI Data Reduction Image Qulaity Software
*
* Module name:  procAcq.c
* Description:  Contains routines for processing the Image Quality
*
* History:      
* 25-Apr-05     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "midiGlobal.h"
#include "qfits.h"
#include "midiLib.h"
#include "imageProcessing.h"
#include "midiFitsUtility.h"
#include "createProdAcq.h"
#include "diagnostics.h"
#include "memoryHandling.h"
#include "errorHandling.h"
#include "procAcq.h"

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  procAcq
* Input/Output: See function arguments to avoid duplication
* Description:  Performs complete post processing for the Image Quality mode
*
* History:
* 25-Apr-05     (csabet) Created
******************************************************************************/
void procAcq (
    MidiFiles    *fileNames,    // In: Pointer to file names
    int            *error)        // Ou: Status
{

    //  Local Declarations
    //    ------------------
    const char        routine[] = "procAcq";
    char            *fileTemp, *classification;
    FILE            *inFitsBatchPtr;
    ImageFormat        *format;
    int                extNumOfImagingDataFile;
    ImageQuality    *image=NULL;
    FILE            *signaturePtr=NULL;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nProcessing Image Quality for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"---------------------------------- \n");
    fprintf (midiReportPtr, "\nProcessing Image Quality for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "---------------------------------- \n");

    //    Write a signature
    signaturePtr = fopen ("MIDI_sig_acq.log", "w");
    fclose (signaturePtr);

    //    Allocate memory
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    format = callocImageFormat ();

    //    Reset status
    *error = 0;
    
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No data preprocessing has been carried out for this batch");
        freeImageFormat (format);
        free (fileTemp);
        free (classification);
        *error = 1;
        return;
    }

    //    Extract file name and its classification
    fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
    if (diagnostic)cpl_msg_info(cpl_func,"\n   Processing file   %s \n", fileNames->inFitsName);
    fprintf(midiReportPtr, "\n   Processing file   %s \n", fileNames->inFitsName);

    //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
    extNumOfImagingDataFile  = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
    if (*error || !extNumOfImagingDataFile) 
    {
        fclose (inFitsBatchPtr);
        freeImageFormat (format);
        free (fileTemp);
        free (classification);
        return;    
    }

    //    Get Image Format parameters
    getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, format, error);
    if (*error || !(format->hasData))
    {
        fclose (inFitsBatchPtr);
        freeImageFormat (format);
        free (fileTemp);
        free (classification);
        return;
    }

    //    Check Categ, Tech and Type and then compute the image size
    if (((strcmp (format->obsCatg, "CALIB") == 0) || 
        (strcmp (format->obsCatg, "SCIENCE") == 0)) &&
        ((strcmp (format->obsTech, "IMAGE,WINDOW") == 0) || 
        (strcmp (format->obsType, "COARSE,OBJECT") == 0)))
    {
        //    Compress Acquisition Image
        image = callocImageQuality (format);
        compressAcq (fileNames->inFitsName, extNumOfImagingDataFile, image, format, error);
        if (*error) 
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compress data in FITS file");
            fclose (inFitsBatchPtr);
            freeImageQuality (format, image);
            freeImageFormat (format);
            free (fileTemp);
            free (classification);
            return;
        }
            
        //    Evaluate Image quality
        assessImageQuality (fileNames->inFitsName, image, format, error);
        if (*error) 
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot assess image quality in FITS file");
            fclose (inFitsBatchPtr);
            freeImageQuality (format, image);
            freeImageFormat (format);
            free (fileTemp);
            free (classification);
            return;
        }

        //    Create Output products
        createAcqProd (fileNames, format, image, error);
        if (*error) 
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create product FITS file");
            fclose (inFitsBatchPtr);
            freeImageQuality (format, image);
            freeImageFormat (format);
            free (fileTemp);
            free (classification);
            return;
        }
        freeImageQuality (format, image);
    }
    else
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Catg, Type, or Tech");
    }

    //    Close the file list
    fclose (inFitsBatchPtr);
    
    //    Release memory
    freeImageFormat (format);
    free (fileTemp);
    free (classification);    

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  compressAcq
* Input/Output: See function arguments to avoid duplication
* Description:  Prepares Image data
*
* History:      
* 25-Apr-05     (csabet) Created
******************************************************************************/
void compressAcq (
    char            *inFitsFile,            // In: Name of the input FITS file
    int                extNumOfImagingDataFile,// In: Extension number of the IMAGING_DATA in input file
    ImageQuality    *compressed,            // In: Compressed data
    ImageFormat        *format,                // In: File format 
    int                *error)                    // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char        routine[] = "compressAcq";
    qfits_table        *pTable=NULL;
    short int        **inData;
    char            *inTarType=NULL, *tempStr, *dataName, lastTT;
    int                i, pixel, *foundData, scalingOffset, *indexData, numOfSkyFrames, numOfFramesUsed,
                    frame, region, indexTarType, foundTarType=0, transitions, numOfUndefinedFrames, numOfBadFrames,
                    tartypMult = 2;    // Signifies that there are 2 (instead of 1!) characters per tartyp value (only 
                                    // first is useful). In future, if the problem producing the MIDI files is solved, 
                                    // can be changed to 1 (only for newer files) or made variable (with some way to 
                                    // determine whether it is an "old" type file). Note: this problem really has 
                                    // nothing to do with Qfits, but with the (unintended) way the files are written.

    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //    Allocate memory
    inData = (short int **) calloc (format->numOfDetectorRegions, sizeof (short int *));
    foundData = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    indexData = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    dataName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        
    pTable = qfits_table_open (inFitsFile, extNumOfImagingDataFile);
    if (!pTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load IMAGING_DATA from");
        free (inData);
        free (foundData);
        free (indexData);
        free (dataName);
        *error = 1;
        return;
    }

    //    Get data table information
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        foundData[region] = 0;
        indexData[region] = 0;
    }
    for (i = 0; i < pTable->nc; i++)
    {
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            sprintf (dataName, "DATA%d", region+1);
            if (strcmp (pTable->col[i].tlabel, dataName) == 0)
            {
                foundData[region] = 1;
                indexData[region] = i;
                if (diagnostic)cpl_msg_info(cpl_func,"Found 'DATA%d' at column %d in data file %s \n", region+1, i+1, inFitsFile);
                if (diagnostic) fprintf(midiReportPtr, "Found 'DATA%d' at column %d in data file %s \n", region+1, i+1, inFitsFile);
            }
        }
        if (strcmp (pTable->col[i].tlabel, "TARTYP2") == 0)    // Now changed to TARTYP2, for all newer files
        {
            foundTarType = 1;
            indexTarType = i;
        }
    }

    //    Now issue warnings
    if (foundTarType == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find TARTYP2 in data FITS file");
        *error = 1;
    }
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        if (foundData[region] == 0)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find requested DATA column in data FITS file");
            *error = 1;
        }
    }
    
    //    Release memory and exit this module if there has been a warning. No point going any further
    if (*error)
    {
        qfits_table_close (pTable);
        free (inData);
        free (foundData);
        free (indexData);
        free (dataName);
        return;
    }

    //    Get TARTYP2
    inTarType = (char *) qfits_query_column (pTable, indexTarType, NULL);
    for (frame = 0; frame < format->numOfFrames; frame++)
        compressed->tarType[frame] = inTarType[frame*tartypMult];     // New way around tartype problem (JM)

    //    Check if the data is chopped
    transitions = 0;
    lastTT = compressed->tarType[0];
    for (frame = 0; frame < format->numOfFrames; frame++)
    {
        if (compressed->tarType[frame] == lastTT) continue;
        transitions++;    // INCLUDES transitions to U etc.
        lastTT = compressed->tarType[frame];
    }
    if ((format->chopped = (transitions > 3)))    // Apparently is a chopped file
    {
        if (diagnostic)cpl_msg_info(cpl_func,"\nAcquision data is CHOPPED, since %d  'TARTYP2'  transitions were found \n", transitions);
        fprintf (midiReportPtr, "\nAcquision data is CHOPPED, since %d  'TARTYP2'  transitions were found \n", transitions);
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Acquisition data is NOT chopped");
        *error = 1;
        qfits_table_close (pTable);
        free (inData);
        free (foundData);
        free (indexData);
        free (dataName);
        return;
    }

    //    Read all regions from input files
    for (region = 0; region < format->numOfDetectorRegions; region++)
        inData[region] = (short int*) qfits_query_column (pTable, indexData[region], NULL);

    //    Get the scaling offset
    for (i = 14; i < 25; i++)
    {
        sprintf (dataName, "TZERO%d", i);
        tempStr = qfits_query_ext (inFitsFile, dataName, extNumOfImagingDataFile);
        if (tempStr != NULL)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"Scaling Offset = %s\n", tempStr);
            if (diagnostic) fprintf (midiReportPtr, "Scaling Offset = %s\n", tempStr);
            sscanf (tempStr, "%d", &scalingOffset);
            break;
        }
    }
    if (tempStr == NULL)
    {
        scalingOffset = 0;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Scaling Offset. It is set to 0");
    }

    //    Compression starts here
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        for (pixel = 0; pixel < format->subWindowSize; pixel++)
        {
            //    Along the time axis (frame)
            numOfUndefinedFrames = 0;
            numOfBadFrames = 0;
            numOfFramesUsed = 0;
            numOfSkyFrames = 0;
            (compressed->aveImage)[region][pixel] = 0.0;
            (compressed->aveImageSky)[region][pixel] = 0.0;
            for (frame = 0; frame < format->numOfFrames; frame++)
            {
                //    Only consider Target, "T" and Sky, "S"
                if (compressed->tarType[frame] == 'U')
                {
                    if (diagnostic > 2 && !pixel) 
                    {
                        sprintf (midiMessage, "Found undefined, 'U'  'TARTYP2' at frame %d of DATA%d in %s \n", 
                            frame+1, region+1, inFitsFile);
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    }
                    numOfUndefinedFrames++;
                    continue;
                }
                
                //    Load each pixel data along the time axis
                i = frame * format->subWindowSize + pixel;
/*
                if (isnan (inData[region][i]))
                {
                    numOfBadFrames++;
                    sprintf (midiMessage, "Found bad pixel %d of DATA%d in %s \n", i, region, inFitsFile);
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                }
                else
*/
                {
                    numOfFramesUsed++;
                    (compressed->aveImage)[region][pixel] += (float) (inData[region][i]);
                    if (compressed->tarType[frame] == 'S')
                    {
                        numOfSkyFrames++;
                        (compressed->aveImageSky)[region][pixel] += (float) (inData[region][i]);
                    }
                }
            }

            //    Compute average image
            if (diagnostic && !pixel)
            {
               cpl_msg_info(cpl_func,"\nData Statistics \n");
               cpl_msg_info(cpl_func,"--------------- \n");
               cpl_msg_info(cpl_func,"Number of frames used in DATA%d      = %d \n", region+1, numOfFramesUsed);
               cpl_msg_info(cpl_func,"Number of Sky frames in DATA%d       = %d \n", region+1, numOfSkyFrames);
               cpl_msg_info(cpl_func,"Number of Bad frames in DATA%d       = %d \n", region+1, numOfBadFrames);
               cpl_msg_info(cpl_func,"Number of Undefined frames in DATA%d = %d \n", region+1, numOfUndefinedFrames);
                fprintf (midiReportPtr, "\nData Statistics \n");
                fprintf (midiReportPtr, "--------------- \n");
                fprintf (midiReportPtr, "Number of frames used in DATA%d      = %d \n", region+1, numOfFramesUsed);
                fprintf (midiReportPtr, "Number of Sky frames in DATA%d       = %d \n", region+1, numOfSkyFrames);
                fprintf (midiReportPtr, "Number of Bad frames in DATA%d       = %d \n", region+1, numOfBadFrames);
                fprintf (midiReportPtr, "Number of Undefined frames in DATA%d = %d \n", region+1, numOfUndefinedFrames);
            }
            if (numOfFramesUsed) (compressed->aveImage)[region][pixel] /= numOfFramesUsed;        // Target + Sky
            (compressed->aveImage)[region][pixel] += scalingOffset;                                // Target + Sky
            if (numOfSkyFrames) (compressed->aveImageSky)[region][pixel] /= numOfSkyFrames;        // Sky
            (compressed->aveImageSky)[region][pixel] += scalingOffset;                            // Sky
            (compressed->aveImage)[region][pixel] -= (compressed->aveImageSky)[region][pixel];    // Target
        }
    }

    //    Clean up now:
    for (region = 0; region < format->numOfDetectorRegions; region++) free (inData[region]);
    if (pTable) qfits_table_close (pTable);
    if (inTarType) free(inTarType);
    free (inData);
    free (foundData);
    free (indexData);
    free (dataName);
        
    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  assessImageQuality
* Input/Output: See function arguments to avoid duplication
* Description:  Evaluates the image quality in acquisition mode
*
* History:      
* 25-Apr-05     (csabet) Created
******************************************************************************/
void assessImageQuality (
    char            *fileName,        // In: Input FITS name
    ImageQuality    *compressed,    // In: Compressed data
    ImageFormat        *format,        // In: File format 
    int                *error)            // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char  routine[] = "assessImageQuality";
    int            region, searchSize;
    char        *title=NULL, *fileString=NULL;
    MidiCoords    *target;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;
    searchSize = SIZE_SEARCH_ACQ;

    //    Allocate memory
    target = (MidiCoords *) calloc (1, sizeof (MidiCoords));

    //    Analyse iamges
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        //    Create image FITS files
        if (diagnostic)
        {
            title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            sprintf (title , "AveImgSkyDATA%d", region+1);
            sprintf (fileString , "region %d", region+1);
            createFitsImage (fileString, title, fileName, format->iXWidth, format->iYWidth, 
                (compressed->aveImageSky)[region]);
            sprintf (title , "AveImgDATA%d", region+1);
            createFitsImage (fileString, title, fileName, format->iXWidth, format->iYWidth, 
                (compressed->aveImage)[region]);
            free (fileString);
            free (title);
        }
    
        if (plotFile)
        {
            fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            sprintf (fileString, "3dImgDATA%d", region+1);
            sprintf (title, "Dark removed, region %d", region+1);
            midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                (compressed->aveImage)[region], format->iXWidth, format->iYWidth, "lines", "3");
            if (diagnostic) 
            {
                sprintf (fileString, "3dImgSkyDATA%d", region+1);
                sprintf (title, "Sky, region %d", region+1);
                midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                    (compressed->aveImageSky)[region], format->iXWidth, format->iYWidth, "lines", "3");
            }
            free (fileString);
            free (title);
        }

        //    Determine target size and location
        midiGetFWHM (region+1, (compressed->aveImage)[region], 
            format->iXWidth, format->iYWidth, searchSize, &(compressed->coordX[region]), 
            &(compressed->coordY[region]), &(compressed->sizeX[region]), &(compressed->sizeY[region]), error);
       cpl_msg_info(cpl_func,"Target Coordinates and size (X, Y, X-Width, Y-Width): %0.2f, %0.2f, %0.2f %0.2f \n", 
            compressed->coordX[region], compressed->coordY[region], compressed->sizeX[region], compressed->sizeY[region]);
        fprintf (midiReportPtr, "Target Coordinates and size (X, Y, X-Width, Y-Width): %0.2f, %0.2f, %0.2f %0.2f \n", 
            compressed->coordX[region], compressed->coordY[region], compressed->sizeX[region], compressed->sizeY[region]);
            
        //    Compute target Intensity
        target->xCoord = compressed->coordX[region] - (0.5 * compressed->sizeX[region]);
        target->yCoord = compressed->coordY[region] - (0.5 * compressed->sizeY[region]);
        target->dxCoord = compressed->coordX[region] + (0.5 * compressed->sizeX[region]);
        target->dyCoord = compressed->coordY[region] + (0.5 * compressed->sizeY[region]);
//       cpl_msg_info(cpl_func,"Target Coordinates (x, y, dx, dy): %f, %f, %f %f \n", target->xCoord, target->yCoord, 
//            target->dxCoord, target->dyCoord);
//        fprintf (midiReportPtr, "Target Coordinates (x, y, dx, dy): %f, %f, %f %f \n", target->xCoord, target->yCoord, 
//            target->dxCoord, target->dyCoord);

        computeImageFlux ((compressed->aveImage)[region], format, target, &((compressed->targetFlux)[region]), 
            &((compressed->targetPixelCount)[region]), error);
       cpl_msg_info(cpl_func,"Target flux in region %d        = %0.2f \n", region+1, (compressed->targetFlux)[region]);
       cpl_msg_info(cpl_func,"Target pixel count in region %d = %d \n", region+1, (compressed->targetPixelCount)[region]);
        fprintf (midiReportPtr, "Target flux in region %d        = %0.2f \n", region+1, (compressed->targetFlux)[region]);
        fprintf (midiReportPtr, "Target pixel count in region %d = %d \n", region+1, (compressed->targetPixelCount)[region]);
    }
    
    //    Release memory
    free (target);
    
    return; 
}
/*****************************************************************************/

