
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  procDetLin.c
* Description:  Contains routines for processing the templates
*
* History:      
* 16-Jun-04     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <math.h>
#include <stdio.h>
#include <cpl.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "imageProcessing.h"
#include "memoryHandling.h"
#include "errorHandling.h"
#include "createProdDetLin.h"
#include "midiFitsUtility.h"
#include "fitsAnalysisTec.h"
#include "procDetLin.h"
#include "statistics.h"
#include "diagnostics.h"
#include "cpl_polynomial.h"
#include "qfits.h"
#include "midi_cplutils.h"
#include "midi_cplupgrade.h"

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  procDetLin
* Input/Output: See function arguments to avoid duplication
* Description:  Computes the detector linearity. This is the main routine for this
*                application.
*
* History:      
* 03-May-05     (csabet) Created
* 09-May-06     (csabet) Modified to characterise each pixel
******************************************************************************/
void procDetLin (
    MidiFiles    *fileNames,    // In: Pointer to file names
    int            *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "procDetLin";
    ImageFormat        *format=NULL;
    DetLinearity     *linearity=NULL;
    int                numOfFiles;
    FILE            *signaturePtr=NULL;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Write a signature
    signaturePtr = fopen ("MIDI_sig_lin.log", "w");
    fclose (signaturePtr);

    //    Reset status
    *error = 0;
    numOfFiles = 0;
    
    //    Allocate memory
    format = callocImageFormat ();
    
    analyseFitsDetLin (fileNames, format, &numOfFiles, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot analyse DETLIN");
        freeImageFormat (format);
        return;
    }
    
    linearity = callocDetLin (numOfFiles, format);
    computeDetLin (numOfFiles, fileNames, format, linearity, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot process DETLIN");
        freeDetLin (linearity);
        freeImageFormat (format);
        return;
    }
    
    createDetLinProd (fileNames, format, numOfFiles, linearity, error);
    if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create DETLIN products");

    //    Release memory
    freeDetLin (linearity);
    freeImageFormat (format);

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  computeDetLin
* Input/Output: See function arguments to avoid duplication
* Description:  Computes the detector linearity. 
*                Compute average flux for each pixel in a given file. Exclude saturated pixels.
*                Find coefficients of a polynomial describing the linearity over the integration 
*                time.
*
* History:      
* 15-Jun-04     (csabet) Created
* 09-May-06     (csabet) Modified to characterise each pixel
******************************************************************************/
void computeDetLin (
    int                numOfFiles,    // In: Number of data files
    MidiFiles        *fileNames,    // In: Pointer to the MIDI file structure
    ImageFormat        *format,    // In: Data format
    DetLinearity     *linearity,    // Ou: Pointer to the detector linearity data structure
    int                *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "computeDetLin";
    char        *fileTemp, *classification, *firstFitsFile;
    FILE        *inFitsBatchPtr;
    ImageFormat    *formatLocal;
    int            localError, fileNumber, extNumOfImagingDataFile;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    firstFitsFile = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    formatLocal = callocImageFormat ();

    //    Reset status
    *error = 0;
    localError = 0;
    fileNumber = 0;
    linearity->exists = 0;
    formatLocal->hasData = 0;
            
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot open input FITS file list\n                 No compression has been carried out for this batch");
        freeImageFormat (formatLocal);
        free (fileTemp);
        free (classification);
        *error = 1;
        return;
    }

    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Processing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Processing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile  = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, &localError);
        if (localError) 
        {
            *error = 1;
            break;
        }

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, formatLocal, &localError);
            if (localError) 
            {
                *error = 1;
                break;
            }
        }
        else formatLocal->hasData = 0;

        //    Check if the file has data
        if (formatLocal->hasData)
        {
            //    Check Categ, Tech and Type and then compute the image size
            if ((strcmp (formatLocal->obsCatg, "CALIB") == 0) &&
                ((strcmp (formatLocal->obsTech, "IMAGE") == 0) || 
                (strcmp (formatLocal->obsTech, "SPECTRUM") == 0)) &&
                (strcmp (formatLocal->obsType, "FLAT") == 0))
            {
                //    Save grism ID
                sprintf (linearity->grismId, "%s", formatLocal->grismId);
                if (diagnostic)cpl_msg_info(cpl_func,"   Grism ID for file %d           = %s\n", fileNumber+1, formatLocal->grismId);
                fprintf (midiReportPtr, "   Grism ID for file %d           = %s\n", fileNumber+1, formatLocal->grismId);
                
                //    Create and display averaged images
                createAndDisplayImageDetLin (fileNumber, fileNames->inFitsName, extNumOfImagingDataFile, 
                    format, linearity, &localError);
                if (localError) *error = 1;
                    
                //    Calculate mean of counts. Ignore this file if pixels are saturated
                getIntegrationTime (fileNumber, fileNames->inFitsName, linearity, &localError);
                if (localError) *error = 1;
                        
                //    Set the flags
                linearity->exists = 1;

                //    Increment file number but make sure it is not greater than the allocated
                fileNumber++;
                if (fileNumber > numOfFiles)
                {
                    *error = 1;
                    break;
                }
                
                //    Save name of the first FITS file
                if (fileNumber == 1) sprintf (firstFitsFile, "%s", fileNames->inFitsName);
            }
            else
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, "The above file is not suitable for this task");
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    
    //    Check if processing has been carried out
    if ((linearity->exists) && !(*error) && (fileNumber == numOfFiles))
    {
       cpl_msg_info(cpl_func,"\nDetector Linearity Inventry: \n");
       cpl_msg_info(cpl_func,"=========================== \n");
       cpl_msg_info(cpl_func,"   Expected number of data files  = %d\n", numOfFiles);
       cpl_msg_info(cpl_func,"   Number of data files processed = %d\n", fileNumber);
       cpl_msg_info(cpl_func,"\n");

        fprintf (midiReportPtr, "\nDetector Linearity Inventry: \n");
        fprintf (midiReportPtr, "=========================== \n");
        fprintf (midiReportPtr, "   Expected number of data files  = %d\n", numOfFiles);
        fprintf (midiReportPtr, "   Number of data files processed = %d\n", fileNumber);
        fprintf (midiReportPtr, "\n");

        //    Assess linerity
        //    ---------------
        assessLinearity (format, firstFitsFile, linearity, error);
    }

    if (*error || localError)
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot assess Detector Linearity");

    //    Close the file list
    fclose (inFitsBatchPtr);
    
    //    Release memory
    freeImageFormat (formatLocal);
    free (fileTemp);
    free (classification);
    free (firstFitsFile);

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createAndDisplayImageDetLin
* Input/Output: See function arguments to avoid duplication
* Description:  Creates averaged images for display and later processing
*                purposes.
*
* History:      
* 08-May-06     (csabet) Created
******************************************************************************/
void createAndDisplayImageDetLin ( 
    int                fileNumber,            // In: File number
    char            *fileName,            // In: Name of file to process
    int                extensionNumber,    // In: Extension number of the IMAGE_DATA
    ImageFormat        *format,            // In: Pointer to the image format
    DetLinearity     *linearity,            // Ou: Pointer to the detector linearity data structure
    int                *error)                // Ou: Error status

{
  
    //    Local Declarations
    //    ------------------
    const char  routine[] = "createAndDisplayImageDetLin";
    qfits_table *pTable  = NULL;
    short int   *inData;
    char        *tempStr, *string, *title, *dataName;
    int         i, foundData = 0, scalingOffset, 
                indexData;
       
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;
    linearity->saturated[fileNumber] = 0;
    
    //    Open IMAGING_DATA = INDEX 2
    pTable = qfits_table_open (fileName, extensionNumber);
    if (!pTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load IMAGING_DATA");
        *error = 1;
        return;
    }
        
    //    Get data table information
    for (i = 0; i < pTable->nc; i++)
    {
        if (strcmp (pTable->col[i].tlabel, "DATA1") == 0)
        {
            foundData = 1;
            indexData = i;
        }
    }
    if (foundData == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find requested columns in data FITS file");
        qfits_table_close (pTable);
        *error = 1;
        return;
    }

    //    Read column DATA
    inData = (short int*) qfits_query_column (pTable, indexData, NULL); 

    //    Get the scaling offset
    dataName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    for (i = 14; i < 25; i++)
    {
        sprintf (dataName, "TZERO%d", i);
        tempStr = qfits_query_ext (fileName, dataName, extensionNumber);
        if (tempStr != NULL)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"Scaling Offset = %s\n", tempStr);
            if (diagnostic) fprintf (midiReportPtr, "Scaling Offset = %s\n", tempStr);
            sscanf (tempStr, "%d", &scalingOffset);
            break;
        }
    }
    if (tempStr == NULL)
    {
        scalingOffset = 0;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Scaling Offset. It is set to 0");
    }
    free (dataName);
    
    //    Create averaged image for the whole file. This also allows a check on existence of a target
    createAveragedImage (inData, scalingOffset,    format, linearity->aveImage[fileNumber]);

    //    Create an image FITS file
    title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    string = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sprintf (title, "AveImg%d", fileNumber+1);
    sprintf (string, "file %d", fileNumber+1);
    createFitsImage (string, title, fileName, format->iXWidth, format->iYWidth, linearity->aveImage[fileNumber]);

    //    Create plot file
    if (plotFile)
    {
        sprintf (string, "3dAveImg%d", fileNumber+1);
        sprintf (title, "Averaged Image, file %d", fileNumber+1);
        midiCreatePlotFile3D (string, title, "X", "Y", "Flux", 
            0, linearity->aveImage[fileNumber], format->iXWidth, format->iYWidth, "lines", "3");
    }

    //    Create an averaged image. This time excluding the saturated pixels
    checkSaturationDetLin (inData, scalingOffset, format, linearity->aveImage[fileNumber], 
        &(linearity->saturated[fileNumber]));

    //    Create plot file excluding saturated pixels
    if (diagnostic > 2)
    {
        sprintf (string, "3dAveImgClean%d", fileNumber+1);
        sprintf (title, "Averaged Image. Excluding saturated pixels, file %d", fileNumber+1);
        if (plotFile) midiCreatePlotFile3D (string, title, "X", "Y", "Flux", 
            0, linearity->aveImage[fileNumber], format->iXWidth, format->iYWidth, "lines", "3");
    }
    free (title);
    free (string);
    
    //    Release memory
    qfits_table_close (pTable);
    free (inData);

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getIntegrationTime
* Input/Output: See function arguments to avoid duplication
* Description:  Retrieves Integration time as well as the Number of Integrations.
*
* History:      
* 09-May-06     (csabet) Created
******************************************************************************/
void getIntegrationTime ( 
    int                fileNumber,    // In: File number
    char            *fileName,    // In: Name of file to process
    DetLinearity    *linearity,    // Ou: Pointer to the detector linearity data structure
    int                *error)        // Ou: Error status

{
 
    //    Local Declarations
    //    ------------------
    const char  routine[] = "getIntegrationTime";
    char        *tempStr;
    float         numOfSubInteg, subIntegTime;
         
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;

    //    Get integration time
    tempStr = qfits_query_hdr (fileName, "HIERARCH ESO DET DIT");
    if (tempStr != NULL)
        sscanf(tempStr, "%f", &subIntegTime);
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Integration time");
        *error = 1;
        return;
    }
    tempStr = qfits_query_hdr (fileName, "HIERARCH ESO DET NDIT");
    if (tempStr != NULL)
        sscanf(tempStr, "%f", &numOfSubInteg);
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Number of Integrations");
        *error = 1;
        return;
    }
    linearity->integTime[fileNumber] = numOfSubInteg * subIntegTime;

    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  assessLinearity
* Input/Output: See function arguments to avoid duplication
* Description:  The dependence of the count rate on the integration time and therewith
*                the number of detected photons is over wide range a linear function.
*                It is known, that the function above a count rate of 50000 up to
*                saturation (65536 counts) differs from linarity.
*                Fit a second or third order polynomial to the count levels as a function 
*                of the actual exposure time. That is y = A0 + A1 * t + A2 * t^2 + A3 * t^3
*                The values of the coeficients are the required relevant QC parameters.
*
*                1. Now we have the following 10 images (im)
*                    linearity->aveImage[im][pix]    im = 0 to 9, pix = 0 to subWinSize
*
*                2. We know which file has saturation from
*                    linearity->saturated[im]        im = 0 to 9
*
*                3. We have Integration Time
*                    linearity->integTime[im]        im = 0 to 9
*
*                4. Mean flux for each pixel (pix) of each image (im) is of course
*                    given directly from (1) above. That is
*                    linearity->aveImage[im][pix]    im = 0 to 9, pix = 0 to subWinSize
*
*                5. We can first create a 3D plot of raw linearity for each pixel. The
*                    releveant axes are:
*                    x = Integration Time (given from 3 above)
*                    y = Pixel position                 pix = 0 to subWinSize
*                    z = Mean Flux
*
*                6. For each pixel in 5 above we can compute the linearity coefficients:
*                    A0[pix], A1[pix], A2[pix], A3[pix]
*
*                7. Finally we find the averages of the above coefficients over a desired
*                    region of the image.
*
* History:      
* 23-Jun-04     (csabet) Created
* 09-May-06     (csabet) Modified to characterise each pixel
******************************************************************************/
void assessLinearity (
    ImageFormat        *format,        // In: Pointer to the image format
    char            *firstFitsFile,    // In: Name of first FITS file
    DetLinearity    *linearity,        // Ou: Pointer to the detector linearity data structure
    int                *error)            // Ou: Error status
{
    //  Local Declarations
    //    ------------------
    const char      routine[] = "assessLinearity";
    cpl_polynomial  *coefficients;
    cpl_vector        *xPosition, *values;
    int                j, f, x, y, p, xLength, yLength, xCoord, dxCoord, yCoord, dyCoord,
                    polyDeg = 3, numOfUnsatImages, imageSize, subImageSize;
    cpl_size        power=0;
    double            *integTime, *pixel, *cA0, *cA1, *cA2, *cA3, *linFit;
    char            *fileName, *title;
    float            *buffer;
    
    //    Statistics for pixel regression test
    float            sigData = 0.0, coeff1, coeff2, siga, sigb, chi2, q;
    int                sigDataAvailable = 0;            
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //    Count number of unsaturated images
    numOfUnsatImages = 0;
    for (j = 0; j < linearity->size; j++)
    {
        if (!(linearity->saturated[j])) numOfUnsatImages++;
    }

    //    Compute linearity coefficients for each pixel in a well illuminated central area
    //    --------------------------------------------------------------------------------
    xLength = DET_LIN_WIN_PU * format->iXWidth;
    if (xLength < DET_LIN_WIN_MIN) xLength = format->iXWidth;
    xCoord = (format->iXWidth - xLength) / 2;
    dxCoord = format->iXWidth - xCoord;
    xLength = dxCoord - xCoord;    // Avoing round-off error

    yLength = DET_LIN_WIN_PU * format->iYWidth;
    if (yLength < DET_LIN_WIN_MIN) yLength = format->iYWidth;
    yCoord = (format->iYWidth - yLength) / 2;
    dyCoord = format->iYWidth - yCoord;
    yLength = dyCoord - yCoord;    // Avoing round-off error

    imageSize = format->iXWidth * format->iYWidth;

    //    Load values of the window for QC log
    linearity->winx = xCoord;
    linearity->windx = dxCoord;
    linearity->winy = yCoord;
    linearity->windy = dyCoord;
    if (diagnostic)
    {
       cpl_msg_info(cpl_func,"\nSelected window size \n");
       cpl_msg_info(cpl_func,"-------------------- \n");
       cpl_msg_info(cpl_func,"xCoord  = %3d \n", linearity->winx);
       cpl_msg_info(cpl_func,"dxCoord = %3d \n", linearity->windx);
       cpl_msg_info(cpl_func,"yCoord  = %3d \n", linearity->winy);
       cpl_msg_info(cpl_func,"dyCoord = %3d \n", linearity->windy);
    }
    fprintf (midiReportPtr, "\nSelected window size QCLOG \n");
    fprintf (midiReportPtr, "-------------------- QCLOG \n");
    fprintf (midiReportPtr, "xCoord  = %3d QCLOG \n", linearity->winx);
    fprintf (midiReportPtr, "dxCoord = %3d QCLOG \n", linearity->windx);
    fprintf (midiReportPtr, "yCoord  = %3d QCLOG \n", linearity->winy);
    fprintf (midiReportPtr, "dyCoord = %3d QCLOG \n", linearity->windy);
    
    //    Allocate memory
    fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    buffer = (float *) calloc (imageSize, sizeof (float));
    cA0 = (double *) calloc (imageSize, sizeof (double));
    cA1 = (double *) calloc (imageSize, sizeof (double));
    cA2 = (double *) calloc (imageSize, sizeof (double));
    cA3 = (double *) calloc (imageSize, sizeof (double));
    linFit = (double *) calloc (numOfUnsatImages, sizeof (double));
    
    //    Create a sub-image from each unsaturated image
    for (j = 0; j < numOfUnsatImages; j++)
    {
        for (y = 0; y < format->iYWidth; y++)
        {
            f = y * format->iXWidth;
            for (x = 0; x < format->iXWidth; x++)
            {
                if (!((x >= xCoord) && (x < dxCoord) &&    (y >= yCoord) && (y < dyCoord)))
                {
                    p = f + x;
                    linearity->aveImage[j][p] = 0.0;
                }
            }
        }
        if (plotFile && diagnostic)
        {
            sprintf (fileName, "3dCentralImage%d", j+1);
            sprintf (title, "Averaged central image, file %d", j+1);
            midiCreatePlotFile3D (fileName, title, "X", "Y", "Flux", 0, linearity->aveImage[j], 
                format->iXWidth, format->iYWidth, "lines", "3");
        }
    }

    //    Allocate memory
    pixel = (double *) calloc (numOfUnsatImages, sizeof (double));
    integTime = (double *) calloc (numOfUnsatImages, sizeof (double));
    
    //    For all unsaturated images Load integration time into a buffer
    for (j = 0; j < numOfUnsatImages; j++)
            integTime[j] = (double) (linearity->integTime[j]);

    for (y = 0; y < format->iYWidth; y++)
    {
        f = y * format->iXWidth;
        for (x = 0; x < format->iXWidth; x++)
        {
            //    For all unsaturated images Load pixel flux into a buffer
            if ((x >= xCoord) && (x < dxCoord) &&
                (y >= yCoord) && (y < dyCoord))
            {
                p = f + x;
                for (j = 0; j < numOfUnsatImages; j++)
                    pixel[j] = (double) (linearity->aveImage[j][p]);
                
                //    Plot raw linearity and delete plot file
                if (plotFile && diagnostic > 2)
                {
                    sprintf (title, "Flux versus integration time. Pixel %d, %d", x, y);
                    midiCreatePlotFileDouble2D2P ("temp1", title, "Integration time", "Pixel flux",
                        1, integTime, pixel, 0,    numOfUnsatImages, 1, "lines 3");
                }
            
                //    Compute coefficients of a 3rd order polynomial
                xPosition = cpl_vector_wrap (numOfUnsatImages, integTime);
                values = cpl_vector_wrap (numOfUnsatImages, pixel);
                coefficients = cpl_polynomial_fit_1d_create (xPosition, values, polyDeg, NULL);
                power = 0;
                cA0[p] = cpl_polynomial_get_coeff (coefficients, &power);
                power = 1;
                cA1[p] = cpl_polynomial_get_coeff (coefficients, &power);
                power = 2;
                cA2[p] = cpl_polynomial_get_coeff (coefficients, &power);
                power = 3;
                cA3[p] = cpl_polynomial_get_coeff (coefficients, &power);
                cpl_vector_unwrap (xPosition);
                cpl_vector_unwrap (values);
                cpl_polynomial_delete (coefficients);

                //    Plot fit linearity and delete plot file
                if (plotFile && diagnostic > 2)
                {
                    //    Fill in the result
                    for (j = 0; j < numOfUnsatImages; j++)
                        linFit[j] = cA0[p] + cA1[p] * j + cA2[p] * (j * j) + cA3[p] * (j * j * j);

                    sprintf (title, "Flux versus integration time (polynomial fit). Pixel %d, %d", x, y);
                    midiCreatePlotFileDouble2D2P ("temp1", title, "Integration time", "Pixel flux",
                        1, integTime, linFit, 0, numOfUnsatImages, 1, "lines 1");
                }
            
                //    Now do a regression test (degree of deviation from linearity)
                midiGetLinearFit (integTime, pixel, numOfUnsatImages, sigData, sigDataAvailable, 
                    &coeff1, &coeff2, &siga, &sigb, &chi2, &q, error);
                
                //    Load a measure of linear deviation. Here we use the
                //    combined standard deviation of the linear coefficients
                linearity->deviation[p] = sqrt ((siga+sigb)/2.0);
                
                if (diagnostic > 2)
                {
                   cpl_msg_info(cpl_func,"\nPixel (%d, %d) statistics \n", x, y);
                   cpl_msg_info(cpl_func,"------------------------- \n");
                   cpl_msg_info(cpl_func,"coeff1 =   %f \n", coeff1);
                   cpl_msg_info(cpl_func,"coeff2 =   %f \n", coeff2);
                   cpl_msg_info(cpl_func,"siga   =   %f \n", siga);
                   cpl_msg_info(cpl_func,"sigb   =   %f \n", sigb);
                   cpl_msg_info(cpl_func,"chi2   =   %f \n", chi2);
                   cpl_msg_info(cpl_func,"q      =   %f \n", q);
                    fprintf (midiReportPtr, "\nPixel (%d, %d) statistics \n", x, y);
                    fprintf (midiReportPtr, "------------------------- \n");
                    fprintf (midiReportPtr, "coeff1 =   %f \n", coeff1);
                    fprintf (midiReportPtr, "coeff2 =   %f \n", coeff2);
                    fprintf (midiReportPtr, "siga   =   %f \n", siga);
                    fprintf (midiReportPtr, "sigb   =   %f \n", sigb);
                    fprintf (midiReportPtr, "chi2   =   %f \n", chi2);
                    fprintf (midiReportPtr, "q      =   %f \n", q);
                }
            }
        }
    }

    //    Create plots and FITS images of coefficients
    if (plotFile && diagnostic)
    {
        midiCreatePlotFileDouble3D ("3dCoefficientMapA0", "Map of A0 coefficients", "X", "Y", "Coefficient value", 
            0, cA0, format->iXWidth, format->iYWidth, "points", "1");
        midiCreatePlotFileDouble3D ("3dCoefficientMapA1", "Map of A1 coefficients", "X", "Y", "Coefficient value", 
            0, cA1, format->iXWidth, format->iYWidth, "points", "1");
        midiCreatePlotFileDouble3D ("3dCoefficientMapA2", "Map of A2 coefficients", "X", "Y", "Coefficient value", 
            0, cA2, format->iXWidth, format->iYWidth, "points", "1");
        midiCreatePlotFileDouble3D ("3dCoefficientMapA3", "Map of A3 coefficients", "X", "Y", "Coefficient value", 
            0, cA3, format->iXWidth, format->iYWidth, "points", "1");
    }
    for (p = 0; p < imageSize; p++)    buffer[p] = (float) (cA0[p]);
    removeDc (imageSize, buffer, buffer);
    createFitsImage ("Coefficients A0", "CoefficientMapA0", firstFitsFile, format->iXWidth, format->iYWidth, buffer);
    for (p = 0; p < imageSize; p++)    buffer[p] = (float) (cA1[p]);
    removeDc (imageSize, buffer, buffer);
    createFitsImage ("Coefficients A1", "CoefficientMapA1", firstFitsFile, format->iXWidth, format->iYWidth, buffer);
    for (p = 0; p < imageSize; p++)    buffer[p] = (float) (cA2[p]);
    removeDc (imageSize, buffer, buffer);
    createFitsImage ("Coefficients A2", "CoefficientMapA2", firstFitsFile, format->iXWidth, format->iYWidth, buffer);
    for (p = 0; p < imageSize; p++)    buffer[p] = (float) (cA3[p]);
    removeDc (imageSize, buffer, buffer);
    createFitsImage ("Coefficients A3", "CoefficientMapA3", firstFitsFile, format->iXWidth, format->iYWidth, buffer);

    //    Create plot and FITS image of the linear deviation
    if (plotFile) midiCreatePlotFile3D ("3dLinearitySigmaMap", "Map of Linearity Standard Deviation", "X", "Y", "Sigma", 
        0, linearity->deviation, format->iXWidth, format->iYWidth, "lines", "1");

    //    Suitably prepare image for FITS
    removeDc (imageSize, linearity->deviation, buffer);
    createFitsImage ("Map of Linearity Standard Deviation", "LinearitySigmaMap", firstFitsFile, format->iXWidth, 
        format->iYWidth, buffer);

    //    Compute mean value of the coefficients and the overall linearity deviation
    linearity->meanCoeffA0 = 0.0;
    linearity->meanCoeffA1 = 0.0;
    linearity->meanCoeffA2 = 0.0;
    linearity->meanCoeffA3 = 0.0;
    subImageSize = 0;
    for (y = 0; y < format->iYWidth; y++)
    {
        f = y * format->iXWidth;
        for (x = 0; x < format->iXWidth; x++)
        {
            if ((x >= xCoord) && (x < dxCoord) && (y >= yCoord) && (y < dyCoord))
            {
                p = f + x;
                linearity->meanCoeffA0 += cA0[p];
                linearity->meanCoeffA1 += cA1[p];
                linearity->meanCoeffA2 += cA2[p];
                linearity->meanCoeffA3 += cA3[p];
                linearity->meanSigma += linearity->deviation[p];
                subImageSize++;
            }
        }
    }
    if (subImageSize)
    {
        linearity->meanCoeffA0 /= subImageSize;
        linearity->meanCoeffA1 /= subImageSize;
        linearity->meanCoeffA2 /= subImageSize;
        linearity->meanCoeffA3 /= subImageSize;
        linearity->meanSigma /= subImageSize;
    }
    
   cpl_msg_info(cpl_func,"\nMean coefficients for polynomial fit: y = a0 + a1 * t + a2 * t^2 + a3 * t^3 \n");
   cpl_msg_info(cpl_func,"--------------------------------------------------------------------------- \n");
   cpl_msg_info(cpl_func,"a0, a1, a2, a3 = %f, %f, %f, %f \n", 
        linearity->meanCoeffA0, linearity->meanCoeffA1, linearity->meanCoeffA2, linearity->meanCoeffA3);
    fprintf (midiReportPtr, "\nMean coefficients for polynomial fit: y = a0 + a1 * t + a2 * t^2 + a3 * t^3 (QCLOG) \n");
    fprintf (midiReportPtr, "--------------------------------------------------------------------------- (QCLOG) \n");
    fprintf (midiReportPtr, "a0, a1, a2, a3 = %f, %f, %f, %f (QCLOG) \n", 
        linearity->meanCoeffA0, linearity->meanCoeffA1, linearity->meanCoeffA2, linearity->meanCoeffA3);

   cpl_msg_info(cpl_func,"\nMean linearity standard deviation = %f \n\n", linearity->meanSigma);
    fprintf (midiReportPtr, "\nMean linearity standard deviation = %f (QCLOG) \n\n", linearity->meanSigma);

    //    Plot linear fit of the mean
    for (j = 0; j < numOfUnsatImages; j++)
    {
        linFit[j] = linearity->meanCoeffA0 + linearity->meanCoeffA1 * j + linearity->meanCoeffA2 * (j * j) + 
            linearity->meanCoeffA3 * (j * j * j);
        linearity->mean[j] = (float) linFit[j];
        
    }
    midiCreatePlotFileDouble2D2P ("MeanFluxVersusTimePolyFit", "Meam flux versus time (polynomial fit)", 
        "Integration time", "Mean flux", 0, integTime, linFit, 0, numOfUnsatImages, 1, "lines 1");
    
    //    Release memory
    free (linFit);
    free (cA0);
    free (cA1);
    free (cA2);
    free (cA3);
    free (fileName);
    free (title);
    free (pixel);
    free (integTime);
    free (buffer);
     
    return;

}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  checkSaturationDetLin
* Input/Output: See function arguments to avoid duplication
* Description:  Checks if any pixel is saturated. If so this exposure
*                will not be included in the later processing
*
* History:      
* 05-May-06     (csabet) Created
******************************************************************************/
void checkSaturationDetLin (
    short int    *inData,        // In: Pointer to the input data
    float        scalingOffset,    // In: Scaling Offset
    ImageFormat    *format,        // In: Pointer to the image format
    float        *image,            // Ou: Output image
    int            *saturated)        // Ou: Saturation indicator

{
    //  Local Declarations
    //    ------------------
    const char  routine[] = "checkSaturationDetLin";
    int            i, count, pixel, frame, imageSize;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    imageSize = format->iXWidth * format->iYWidth;
    *saturated = 0;
    
    for (pixel = 0; pixel < imageSize; pixel++)
    {
        //    Along the time axis (frame)
        image[pixel] = 0;
        count = 0;
        for (frame = 0; frame < format->numOfFrames; frame++)
        {
            //    Load each pixel data along the time axis
            i = frame * imageSize + pixel;
/*            if (isnan (inData[i]))
            {
                sprintf (midiMessage, "Found bad pixel %d on frame %d", frame, pixel);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
            else */
            if (((float) (inData[i] + scalingOffset)) >= PIXEL_SATURATION)
            {
                if (diagnostic > 2)
                {
                    sprintf (midiMessage, "Found saturated pixel %d on frame %d", frame, pixel);
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                }
                *saturated = 1;
            }
            else
            {
                count++;
                image[pixel] += (float) (inData[i] + scalingOffset);
            }
        }

        //    Compute average image
        if (count) image[pixel] /= count;
    }

    if (*saturated)
    {
        sprintf (midiMessage, "Found saturated pixels in this image");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }

    return; 
}
/*****************************************************************************/
